package com.agilex.healthcare.veteranappointment.restservice.appointments;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.net.URI;

import javax.ws.rs.core.UriBuilder;

import org.junit.AfterClass;
import org.junit.Assume;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agilex.healthcare.veteranappointment.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.veteranappointment.domain.VARAppointmentMetric;
import com.agilex.healthcare.veteranappointment.testutility.IntegrationTestConfiguration;
import com.agilex.healthcare.veteranappointment.testutility.TestHelper;
import com.agilex.healthcare.utility.ModeHelper;

public class AppointmentMetricsResourceTest {

	private static final org.apache.commons.logging.Log LOGGER = org.apache.commons.logging.LogFactory.getLog(AppointmentMetricsResourceTest.class);

	private static MobileHealthClientTestVersion client;
	
	@BeforeClass
	public static void checkWhetherTestsAreRunningInProperModes() {
		client = TestHelper.createMobileHealthClient();
	}
	
	@AfterClass
	public static void destroy() {
		if (client != null) {
			client.getJerseyClient().destroy();
			client = null;
		}
	}
	
	@Test
	public void verifyNoIcnOrDfnMetric() {
		URI uri = createAppointmentNoIcnOrDfnMetricUri();
		testMetric(uri);
	}
	
	@Test
	public void verifyNoDfnCancelRequestMetric() {
		URI uri = createAppointmentNoDfnCancelRequestMetricUri();
		testMetric(uri);
	}

	private void testMetric(URI uri) {
		VARAppointmentMetric metric = client.getJerseyClient().resource(uri).post(VARAppointmentMetric.class);
		int value1 = metric.getValue();
		assertTrue(value1 > 0);
		
		metric = client.getJerseyClient().resource(uri).post(VARAppointmentMetric.class);
		int value2 = metric.getValue();
		
		assertEquals(value1 + 1, value2);
	}
	
	private URI createAppointmentNoIcnOrDfnMetricUri() {
		String uriFormat = String.format("%s/appointment-service/metrics/no-icn-or-dfn",
				IntegrationTestConfiguration.getMobileHealthServerBaseUri());
		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
	}
	
	private URI createAppointmentNoDfnCancelRequestMetricUri() {
		String uriFormat = String.format("%s/appointment-service/metrics/no-dfn-cancel-request",
				IntegrationTestConfiguration.getMobileHealthServerBaseUri());
		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
	}
}
