package com.agilex.healthcare.veteranappointment.restservice.directscheduling;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.net.URI;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.UriBuilder;

import junit.framework.Assert;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agilex.healthcare.directscheduling.domain.AppointmentTimeSlots;
import com.agilex.healthcare.directscheduling.domain.BookedAppointment;
import com.agilex.healthcare.directscheduling.domain.BookedAppointmentCollection;
import com.agilex.healthcare.directscheduling.domain.CancelReasons;
import com.agilex.healthcare.directscheduling.domain.DirectSchedulingFacilityProvider;
import com.agilex.healthcare.directscheduling.domain.DirectSchedulingFacilityProviders;
import com.agilex.healthcare.directscheduling.domain.FacilityMemberClinic;
import com.agilex.healthcare.directscheduling.domain.FacilityMemberTeam;
import com.agilex.healthcare.directscheduling.domain.VARBookedAppointmentCollections;
import com.agilex.healthcare.directscheduling.domain.facility.DSFacilities;
import com.agilex.healthcare.directscheduling.domain.facility.DSFacility;
import com.agilex.healthcare.directscheduling.domain.facility.DSFacilityClinic;
import com.agilex.healthcare.directscheduling.domain.facility.DSFacilityProvider;
import com.agilex.healthcare.mobilehealthplatform.domain.AtomLink;
import com.agilex.healthcare.mobilehealthplatform.domain.ResourceDirectory;
import com.agilex.healthcare.utility.DateHelper;
import com.agilex.healthcare.veteranappointment.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.veteranappointment.testutility.IntegrationTestConfiguration;
import com.agilex.healthcare.veteranappointment.testutility.TestHelper;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.GenericType;

public class DSResourceTest {
	private static MobileHealthClientTestVersion client;
    private static ResourceDirectory rs;
    
    @BeforeClass
    public static void fixtureSetup() {
        client = TestHelper.createVeteranMobileHealthClient();
        rs = client.getJerseyClient().resource(getDSResourceUri()).type(MediaType.APPLICATION_XML).accept("application/xml").get(ResourceDirectory.class);
        Assert.assertNotNull(rs);
    }

    @AfterClass
    public static void tearDown(){
    ClientResponse result = client.getJerseyClient().resource(getUriByTitle("ds-logout")).type(MediaType.APPLICATION_XML).accept("application/xml").get(ClientResponse.class);
    Assert.assertEquals("Logout Test",200,result.getStatus());	
    if (client != null) {
            client.getJerseyClient().destroy();
            client = null;
        }
    }
    
    private static URI getDSResourceUri() {
        String uriFormat = String.format("%s/direct-scheduling/directscheduling-resource-directory?siteCode=%s",
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(),"777");
        UriBuilder uriBuilder = UriBuilder.fromUri(uriFormat);
        URI uri = uriBuilder.build();
		return uri;
    }
    
    private static URI getUriByTitle(String title){
    	AtomLink link = rs.getLink().getLinkByTitle(title);
    	return link.getHref();
    }
 
    @Test
    public void getAppointmentSlots(){
    	UriBuilder uriBuilder = UriBuilder.fromUri(getUriByTitle("ds-appointment-slots"));
    	uriBuilder.queryParam("startDate", DateHelper.formatDate(DateHelper.getYesterday()));
        uriBuilder.queryParam("endDate", DateHelper.formatDate(DateHelper.get120DaysFromNow()));
        uriBuilder.queryParam("clinicIds", "32,23");
    	Collection<AppointmentTimeSlots> appointmentTimeSlotsC = client.getJerseyClient().resource(uriBuilder.build()).type(MediaType.APPLICATION_XML).accept("application/xml").get(new   
    			GenericType<Collection<AppointmentTimeSlots>>() {});
    	assertNotNull(appointmentTimeSlotsC);
    	Iterator<AppointmentTimeSlots> appointmentTimeSlotsIterator = appointmentTimeSlotsC.iterator();
    	while(appointmentTimeSlotsIterator.hasNext()){
    		AppointmentTimeSlots appointmentTimeSlots = appointmentTimeSlotsIterator.next( ) ; 
    		assertNotNull( appointmentTimeSlots ) ; 
	    	Assert.assertTrue("Check for only primary care slots for 32 or 23",appointmentTimeSlots.getClinicId().equals("32") || appointmentTimeSlots.getClinicId().equals("23"));
	    	Assert.assertTrue("Check to make sure some slots are returned", appointmentTimeSlots.getSize()>=1);
    	}
    }
    
    @Test
    public void getCancelReasonsDropDownList(){
    	CancelReasons cancelReasons = client.getJerseyClient().resource(getUriByTitle("ds-cancel-reasons-list")).type(MediaType.APPLICATION_XML).accept("application/xml").get(CancelReasons.class);
    	assertNotNull(cancelReasons);
    	Assert.assertTrue("Check to make sure cancel Reasons are returned", cancelReasons.getSize()>=1);
    }

    @Test
    public void getAllPendingAppointmentsForPatient(){
    	UriBuilder uriBuilder = UriBuilder.fromUri(getUriByTitle("ds-booked-appointments"));
    	uriBuilder.queryParam("startDate", DateHelper.formatDate(DateHelper.getYesterday()));
    	VARBookedAppointmentCollections bookedAppointmentsCollections = client.getJerseyClient().resource(uriBuilder.build()).type(MediaType.APPLICATION_XML).accept("application/xml").get(VARBookedAppointmentCollections.class);
    	assertNotNull(bookedAppointmentsCollections);
    }
    
    @Test
    public void getPrimaryCarePendingAppointmentsForPatient(){
		UriBuilder uriBuilder = UriBuilder.fromUri(getUriByTitle("ds-booked-appointments"));
		uriBuilder.queryParam("startDate", DateHelper.formatDate(DateHelper.getYesterday()));
		uriBuilder.queryParam("clinicIds", "32");

		VARBookedAppointmentCollections bookedAppointmentCollections = client.getJerseyClient().resource(uriBuilder.build()).type(MediaType.APPLICATION_XML).accept("application/xml").get(VARBookedAppointmentCollections.class);
		assertNotNull(bookedAppointmentCollections);

		Collection<BookedAppointmentCollection> bookedAppointmentCollectionContainer = bookedAppointmentCollections.getBookedAppointmentCollections();
		assertNotNull(bookedAppointmentCollectionContainer);

		for (BookedAppointmentCollection bookedAppointmentCollection : bookedAppointmentCollectionContainer) {

			assertNotNull(bookedAppointmentCollection);
			if (!(bookedAppointmentCollection.getCollectionName().equalsIgnoreCase("PRIMARY_CARE"))) {

				continue;
			}

			Collection<BookedAppointment> bookedAppointmentsCollection = bookedAppointmentCollection.getBookedAppointments();

			for (BookedAppointment bookedAppointment : bookedAppointmentsCollection) {

				Assert.assertEquals("Test for only primary care clinic", "32", bookedAppointment.getClinic().getId());
			}

		}
    }
    
    @Test
    public void getAllFacilityProvidersForPatient( ) { 
    	
    	UriBuilder uriBuilder = UriBuilder.fromUri(getUriByTitle("facility-provider-all")) ; 
    	DSFacilities dsFacilities = client.getJerseyClient().resource( uriBuilder.build() ).type(MediaType.APPLICATION_XML).accept("application/xml").get( DSFacilities.class);
    	assertNotNull( dsFacilities ) ; 
   
    	List< DSFacility > dsFacilityList = dsFacilities.getFacilityList( ) ; 
    	assertNotNull( dsFacilityList ) ; 
    	assertTrue( dsFacilityList.size( ) > 0 ) ; 
    	
		for (DSFacility dsFacility : dsFacilities.getFacilityList()) {

			assertNotNull(dsFacility);

			assertNotNull(dsFacility.getFacilityAddr1());
			assertNotNull(dsFacility.getFacilityAddr2());
			assertNotNull(dsFacility.getFacilityCity());
			assertNotNull(dsFacility.getFacilityCode());
			assertNotNull(dsFacility.getFacilityName());
			assertNotNull(dsFacility.getFacilityParentSiteCode());
			assertNotNull(dsFacility.getFacilityState());
			assertNotNull(dsFacility.getFacilityType());
			assertNotNull(dsFacility.getPatientICN());
			assertNotNull(dsFacility.getFacilityClinics());

			for (DSFacilityClinic dsFacilityClinic : dsFacility.getFacilityClinics()) {

				assertNotNull(dsFacilityClinic.getStopCode());
				assertNotNull(dsFacilityClinic.getProviders());

				for (DSFacilityProvider dsFacilityProvider : dsFacilityClinic.getProviders()) {

					assertNotNull(dsFacilityProvider.getFirstName());
					assertNotNull(dsFacilityProvider.getLastName());
					assertNotNull(dsFacilityProvider.getProviderClass());
					assertNotNull(dsFacilityProvider.getProviderId());
					assertNotNull(dsFacilityProvider.getProviderNamePrefix());
					assertNotNull(dsFacilityProvider.getProviderNameSuffix());
				}

			}

		}
    }
    
    
    @Test
    public void getPactFacilityProvidersForPatient(){

    	UriBuilder uriBuilder = UriBuilder.fromUri(getUriByTitle("facility-provider")) ; 

    	DirectSchedulingFacilityProviders dsfps = client.getJerseyClient().resource( uriBuilder.build() ).type(MediaType.APPLICATION_XML).accept("application/xml").get( DirectSchedulingFacilityProviders.class);
    	assertNotNull( dsfps ) ; 
    	
    	List< DirectSchedulingFacilityProvider > directSchedulingFacilityProviderList = dsfps.getFacilityProviderList( ) ; 
    	assertNotNull( directSchedulingFacilityProviderList ) ; 
    	assertTrue( directSchedulingFacilityProviderList.size( ) > 0 ) ; 
    	
    	for( DirectSchedulingFacilityProvider directSchedulingFacilityProvider : directSchedulingFacilityProviderList ) { 

    		String facilityId = directSchedulingFacilityProvider.getFacilityId( ) ; 
    		assertNotNull( facilityId ) ; 
    		
    		String institutionName = directSchedulingFacilityProvider.getInstitutionName( ) ; 
    		assertNotNull( institutionName ) ; 
    		
    		Collection< FacilityMemberClinic > clinicList = directSchedulingFacilityProvider.getClinicList( ) ; 
    		assertNotNull( clinicList ) ; 
    		assertTrue( clinicList.size( ) > 0 ) ; 
    		for( FacilityMemberClinic clinic : clinicList ) { 
    			
    			// values are either null/not null depending on whether the getter was
    			// annotated with XML annotation, which also determines what we send to the front-end
    			assertNotNull( clinic.getLocationIEN( ) ) ; 
    			assertNotNull( clinic.getPhysicalLocation( ) ) ; 
    			
    			assertNull( clinic.getFacilityId( ) ) ; 
    			assertNull( clinic.getInstitutionCode() ) ; 
    			assertNull( clinic.getInstitutionIEN()) ; 
    			assertNull( clinic.getInstitutionName()) ; 
    			assertNull( clinic.getInstitutionSID()) ; 
    			assertNull( clinic.getLocationName()) ; 
    			assertNull( clinic.getLocationSID()) ; 
    			assertNull( clinic.getLocationType()) ; 
    			assertNull( clinic.getPrimaryStopCode()) ; 
    			assertNull( clinic.getPrimaryStopCodeName()) ; 
    			assertNull( clinic.getTeamName() ) ; 
    			assertNull( clinic.getTeamPurpose() ) ; 
    			assertNull( clinic.getTeamSID() ) ; 
    		}
    		
    		Collection< FacilityMemberTeam > teamList = directSchedulingFacilityProvider.getTeamList( ) ; 
    		assertNotNull( teamList ) ; 
    		assertTrue( teamList.size( ) > 0 ) ; 
    		for( FacilityMemberTeam teamMember : teamList ) { 
    			
    			assertNotNull( teamMember.getStaffName( ) ) ; 
    			assertNotNull( teamMember.getProviderStandardPosition( ) ) ; 
    			
    			assertNull( teamMember.getFacilityId() ) ;
    			assertNull( teamMember.getPossiblePrimary()) ; 
    			assertNull( teamMember.getProviderPosition());
    			assertNull( teamMember.getProviderSid()) ; 
    			assertNull( teamMember.getTeamName() ) ; 
    			assertNull( teamMember.getTeamPurpose()) ;
    			assertNull( teamMember.getTeamSid()) ; 
    		}
    		
    		Collection< FacilityMemberTeam > primaryList = directSchedulingFacilityProvider.getPrimaryList( ) ; 
    		assertNotNull( primaryList ) ; 
    		assertTrue( primaryList.size( ) > 0 ) ; 
    		for( FacilityMemberTeam primary : primaryList ) { 
    			
    			assertNotNull( primary.getStaffName() ) ; 
    			assertNotNull( primary.getProviderStandardPosition()) ; 
    			
    			assertNull( primary.getFacilityId()) ; 
    			assertNull( primary.getPossiblePrimary()) ; 
    			assertNull( primary.getProviderPosition()) ; 
    			assertNull( primary.getProviderSid()) ; 
    			assertNull( primary.getTeamName()) ; 
    			assertNull( primary.getTeamPurpose()) ; 
    			assertNull( primary.getTeamSid()) ; 
    		}
    	}
    }
}