package com.agilex.healthcare.veteranappointment.datalayer.appointment;

import javax.annotation.Resource;

import org.junit.After;
import org.junit.Assume;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agilex.healthcare.veteranappointment.utils.SpringBasedIntegrationRepo;
import com.agilex.healthcare.veteranappointment.domain.VARAppointmentMetric;
import com.agilex.healthcare.utility.ModeHelper;

public class AppointmentMetricsDataLayerRepoTest extends SpringBasedIntegrationRepo {
	@Resource
	private AppointmentMetricsDataLayer appointmentMetricsDataLayer;
	private static VARAppointmentMetric testMetric;
	private static final String TEST_METRIC_NAME = "test name";
	
	static {
		testMetric = new VARAppointmentMetric();
		testMetric.setName(TEST_METRIC_NAME);
		testMetric.setValue(1);
	}
	
	public AppointmentMetricsDataLayerRepoTest() {
	}
	
	@BeforeClass
	public static void checkWhetherTestsAreRunningInVeteranDevMode() {
		Assume.assumeTrue(ModeHelper.isMode("va-veteran-dev", "dev"));
	}
	
	@Before
	public void setup() {
		appointmentMetricsDataLayer.saveMetric(testMetric);
	}
	
	@After
	public void tearDown() {
		appointmentMetricsDataLayer.removeMetric(testMetric.getName());
	}
	
	@Test
	public void getMetric() {
		VARAppointmentMetric metric = appointmentMetricsDataLayer.getMetric("test name");
		assertNotNull(metric);
		assertEquals(testMetric.getValue(), metric.getValue());
	}
	
	@Test
	public void updateMetric() {
		VARAppointmentMetric metric = appointmentMetricsDataLayer.getMetric("test name");
		metric.setValue(3);
		VARAppointmentMetric savedMetric = appointmentMetricsDataLayer.saveMetric(metric);
		
		assertEquals(3, savedMetric.getValue());
	}
}
