package com.agilex.healthcare.directscheduling.dataservice;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Service;

import com.agilex.healthcare.directscheduling.dao.ClinicByProviderDao;
import com.agilex.healthcare.directscheduling.dao.ClinicProvider;
import com.agilex.healthcare.directscheduling.dao.PatientRelationshipDaoImpl;
import com.agilex.healthcare.directscheduling.domain.DirectSchedulingFacilityProvider;
import com.agilex.healthcare.directscheduling.domain.DirectSchedulingFacilityProviders;
import com.agilex.healthcare.directscheduling.domain.FacilityMember;
import com.agilex.healthcare.directscheduling.domain.FacilityMemberClinic;
import com.agilex.healthcare.directscheduling.domain.FacilityMemberProvider;
import com.agilex.healthcare.directscheduling.domain.FacilityMemberTeam;
import com.agilex.healthcare.directscheduling.domain.VARFacilityMembers;
import com.agilex.healthcare.directscheduling.domain.facility.DSFacilities;
import com.agilex.healthcare.directscheduling.domain.facility.DSFacility;
import com.agilex.healthcare.directscheduling.domain.facility.DSFacilityClinic;
import com.agilex.healthcare.directscheduling.domain.facility.DSFacilityProvider;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifiers;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientProvider;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientProviders;
import com.agilex.healthcare.veteranappointment.dataservice.ProviderDataService;
import com.agilex.healthcare.veteranappointment.domain.PatientProviderWithFacility;

@Service
@Scope(BeanDefinition.SCOPE_SINGLETON)
public class DSFacilityProviderDataService extends DSDataService {

	@Resource( name = "patientRelationshipDaoClinicImpl" )
	private PatientRelationshipDaoImpl patientRelationshipDaoClinicImpl ;

	@Resource( name = "patientRelationshipDaoTeamImpl" )
	private PatientRelationshipDaoImpl patientRelationshipDaoTeamImpl ;
	
	@Resource( name = "patientRelationshipDaoFacilityProvidersImpl" )
	private PatientRelationshipDaoImpl patientRelationshipDaoFacilityProvidersImpl ;
	
	@Resource(name="clinicByProviderDao")
	private ClinicByProviderDao clinicByProviderDao;
	
	@Resource
	ProviderDataService providerDataService;
	
	private List<ClinicProvider> clinicProviderList;

	public DSFacilityProviderDataService() { }

	public DSFacilities getAllFacilityProvidersForPatient(PatientIdentifier patientIdentifer) {

		PatientIdentifiers correspondIdentifiers = patientCorrelationGateway.getCorrespondIdentifiers(patientIdentifer);

		List<String> icnIdentifiers = getIcnIdentifiers(correspondIdentifiers);
		if (icnIdentifiers == null || icnIdentifiers.isEmpty()) {
			return (null);
		}

		DSFacilities dsFacilities = new DSFacilities();
		for (String icnIdentifier : icnIdentifiers) {
			
			PatientProviders providers = providerDataService.fetchPrimaryCareProviders(patientIdentifer, null, false);
			providers.addAll(providerDataService.fetchMentalHealthProviders(patientIdentifer, null, false));
			
			VARFacilityMembers facilityProvider = patientRelationshipDaoFacilityProvidersImpl.getPatientData(icnIdentifier);
			if( facilityProvider != null ) { 
				
				loadVARFacilityMembersIntoDSFacilities(providers, facilityProvider.getFacilityMembers() , dsFacilities);
			}
		}

		return (dsFacilities.getFacilityList().size() == 0 ? null : dsFacilities);
	}  
	 
 	public DirectSchedulingFacilityProviders getPactFacilityMembersForPatient(PatientIdentifier patientIdentifer) {	
 		
 		PatientIdentifiers correspondIdentifiers = patientCorrelationGateway.getCorrespondIdentifiers(patientIdentifer);
 		
 		List<String>  icnIdentifiers  = getIcnIdentifiers(correspondIdentifiers);
 		if(icnIdentifiers == null || icnIdentifiers.isEmpty()){
 			return null;
 		}
		
        DirectSchedulingFacilityProviders dsfps = null;
		for(String icnIdentifier:icnIdentifiers){
			VARFacilityMembers facilityTeam = patientRelationshipDaoTeamImpl.getPatientData(icnIdentifier);
			VARFacilityMembers facilityClinics = patientRelationshipDaoClinicImpl.getPatientData(icnIdentifier);
			dsfps = getPactFacilityMembers(facilityTeam, facilityClinics);
		}
		
		List<String>  dfnIdentifiers = getDfnIdentifiers(correspondIdentifiers,dsfps);
		if(dfnIdentifiers == null || dfnIdentifiers.isEmpty()){
 			return null;
 		}
		
		return (dsfps);
	}

	public void setPatientRelationshipDaoFacilityProvidersImpl( PatientRelationshipDaoImpl patientRelationshipDaoImpl ) { 
		
		this.patientRelationshipDaoFacilityProvidersImpl = patientRelationshipDaoImpl ;
	}
	
	public void setPatientRelationshipDaoClinicImpl( PatientRelationshipDaoImpl patientRelationshipDaoImpl ) { 
		
		this.patientRelationshipDaoClinicImpl = patientRelationshipDaoImpl ;
	}
	
	public void setPatientRelationshipDaoTeamImpl( PatientRelationshipDaoImpl patientRelationshipDaoImpl ) { 
		
		this.patientRelationshipDaoTeamImpl = patientRelationshipDaoImpl ;
	}
	
	private DirectSchedulingFacilityProviders getPactFacilityMembers(VARFacilityMembers facilityTeam, VARFacilityMembers facilityClinics) {

		Collection<FacilityMember> facilityTeamPrimaryCollection = facilityTeam.getFacilityMembers();
		Collection<FacilityMember> facilityTeamClinicCollection = facilityClinics.getFacilityMembers();

		// filter only doctors who are primary
		Map<String,Collection<FacilityMemberTeam>> validDoctors = filterFacilityByPossiblePrimary(facilityTeamPrimaryCollection, "Y");
		if (validDoctors == null) {
			return (null);
		}
		Collection<FacilityMemberTeam> teamMembers = filterFacilityByTeamMember(facilityTeamPrimaryCollection);
		Collection<FacilityMemberClinic> validClinics = filterFacilityByInstitutionName(facilityTeamClinicCollection);
		if (validClinics == null) {
			return (null);
		}

		DirectSchedulingFacilityProviders providers = new DirectSchedulingFacilityProviders();
		
		for (String ftd : validDoctors.keySet()) {
			DirectSchedulingFacilityProvider facilityProviderDoctor = getFacilityProviderByFacilityId(providers, ftd);
			facilityProviderDoctor.setTeamList(teamMembers);

			Collection<FacilityMemberTeam> doctorList = getFacilityProviderDoctorList(facilityProviderDoctor);
			for(FacilityMemberTeam members : validDoctors.get(ftd)){
				doctorList.add(members);
			}
			
			for (FacilityMemberClinic ftc : validClinics) {
				if (ftd.equalsIgnoreCase(ftc.getFacilityId())) {
					DirectSchedulingFacilityProvider facilityProviderClinic = getFacilityProviderByFacilityId(providers, ftd);
					Collection<FacilityMemberClinic> clinicList = getFacilityClinicList(facilityProviderClinic,ftc);
					
					// Should only update on the first try (if existing name is
					// null for this institution).
					updateInstitutionName(facilityProviderClinic,ftc.getInstitutionName());		
				}
			}
			if(facilityProviderDoctor!=null && (facilityProviderDoctor.getClinicList()==null || facilityProviderDoctor.getClinicList().isEmpty())){
				providers.getFacilityProviderList().remove(facilityProviderDoctor);
			}
		}
		return (providers);
	}
	
	private void updateInstitutionName(DirectSchedulingFacilityProvider facilityProvider,String institutionName) {
		if (facilityProvider.getInstitutionName() == null) {
			facilityProvider.setInstitutionName(institutionName);
		}
	}

	private Collection<FacilityMemberTeam> getFacilityProviderDoctorList(DirectSchedulingFacilityProvider facilityProvider) {
		Collection<FacilityMemberTeam> facilityMemberCollection = facilityProvider.getPrimaryList();
		if (facilityMemberCollection == null) {
			facilityMemberCollection = new ArrayList<FacilityMemberTeam>();
			facilityProvider.setPrimaryList(facilityMemberCollection);
		}
		return (facilityMemberCollection);
	}

	private Collection<FacilityMemberClinic> getFacilityClinicList(DirectSchedulingFacilityProvider facilityProvider,FacilityMemberClinic ftc) {
		clinicProviderList = getClinicByProvider();
		Collection<FacilityMemberClinic> facilityMemberCollection=null;
		for(FacilityMemberTeam primaryDoc:facilityProvider.getPrimaryList()){
			ClinicProvider clinicProvider = new ClinicProvider();
			clinicProvider.setFacilityCode(primaryDoc.getFacilityId());
			clinicProvider.setStaffName(primaryDoc.getStaffName());
			clinicProvider.setClinicId(ftc.getLocationIEN());
			int index = clinicProviderList.indexOf(clinicProvider);
			if(clinicProviderList!=null && !clinicProviderList.isEmpty() && index!=-1){	
				facilityMemberCollection = facilityProvider.getClinicList();
				if (facilityMemberCollection == null) {
					facilityMemberCollection = new HashSet<FacilityMemberClinic>();
					facilityProvider.setClinicList(facilityMemberCollection);
				}
				ftc.setPhysicalLocation(clinicProviderList.get(index).getClinicName());
				facilityMemberCollection.add(ftc); 
			}
		}
		return (facilityMemberCollection);
	}

	private DirectSchedulingFacilityProvider getFacilityProviderByFacilityId(DirectSchedulingFacilityProviders providers, String faciltyId) {
		List<DirectSchedulingFacilityProvider> facilityProviderCollection = providers.getFacilityProviderList();
		
		if (facilityProviderCollection == null) {
			facilityProviderCollection = new ArrayList<DirectSchedulingFacilityProvider>();
			providers.setFacilityProviderList(facilityProviderCollection);
		}
		
		Iterator<DirectSchedulingFacilityProvider> it = facilityProviderCollection.iterator();
		
		while (it.hasNext()) {
			DirectSchedulingFacilityProvider dsfp = it.next();;
			if (dsfp.getFacilityId().equalsIgnoreCase(faciltyId)) {
				return (dsfp);
			}
		}

		DirectSchedulingFacilityProvider newFacility = new DirectSchedulingFacilityProvider();
		newFacility.setFacilityId(faciltyId);
		facilityProviderCollection.add(newFacility);
		return (newFacility);
	}

	private Collection<FacilityMemberClinic> filterFacilityByInstitutionName(Collection<FacilityMember> facilityTeamClinics) {
		Collection<FacilityMemberClinic> clinicCollection = new ArrayList<FacilityMemberClinic>();
		Iterator<FacilityMember> teamIt = facilityTeamClinics.iterator();

		while (teamIt.hasNext()) {
			FacilityMemberClinic ftd = (FacilityMemberClinic) teamIt.next();

			if (StringUtils.isNotEmpty(ftd.getInstitutionName())) {
				clinicCollection.add(ftd);
			}
		}
		return (clinicCollection.size() == 0 ? null : clinicCollection);
	}

	private Collection<FacilityMemberTeam> filterFacilityByTeamMember(Collection<FacilityMember> facilityMemberCollection) {
		Collection<FacilityMemberTeam> doctorCollection = new ArrayList<FacilityMemberTeam>();
		Iterator<FacilityMember> teamIt = facilityMemberCollection.iterator();

		while (teamIt.hasNext()) {
			FacilityMemberTeam ftd = (FacilityMemberTeam) teamIt.next();

			if (StringUtils.isEmpty(ftd.getPossiblePrimary())) {
				doctorCollection.add(ftd);
			}
		}
		return (doctorCollection.size() == 0 ? null : doctorCollection);
	}

	private Map<String,Collection<FacilityMemberTeam>> filterFacilityByPossiblePrimary(Collection<FacilityMember> facilityMemberCollection,String possiblePrimaryValue) {
		Map<String,Collection<FacilityMemberTeam>> doctorCollectionByFacility =  new HashMap<String,Collection<FacilityMemberTeam>>();
		
		Iterator<FacilityMember> teamIt = facilityMemberCollection.iterator();

		while( teamIt.hasNext( ) ) {
			FacilityMemberTeam ftd = (FacilityMemberTeam) teamIt.next();
			if(StringUtils.equalsIgnoreCase(ftd.getPossiblePrimary( ),possiblePrimaryValue)) {
				if(!doctorCollectionByFacility.containsKey(ftd.getFacilityId()))
				{
					doctorCollectionByFacility.put(ftd.getFacilityId(),new ArrayList<FacilityMemberTeam>());
				}
				doctorCollectionByFacility.get(ftd.getFacilityId()).add(ftd);
			}
		}
		return (doctorCollectionByFacility.size() ==0?null:doctorCollectionByFacility);
	}
	
	private void loadVARFacilityMembersIntoDSFacilities( Collection<FacilityMember> facilityMemberCollection , DSFacilities dsFacilities ) {

		if (facilityMemberCollection == null || facilityMemberCollection.size() == 0) {

			return;
		}

		List<DSFacility> dsFacilityCollection = dsFacilities.getFacilityList() ;
		for (FacilityMember facilityMember : facilityMemberCollection) {

			if (!(facilityMember instanceof FacilityMemberProvider)) {

				continue;
			}
			
			FacilityMemberProvider facilityMemberProvider = (FacilityMemberProvider) facilityMember;

			String fmpFacilityParentSiteCode = facilityMemberProvider.getFacilityParentSiteCode() ;
			//String fmpFacilityId = facilityMemberProvider.getFacilityId() ; 
			String fmpFacilityType = facilityMemberProvider.getFacilityType() ; 
			String fmpFacilityCode = facilityMemberProvider.getFacilityCode() ; 
			String fmpFacilityName = facilityMemberProvider.getFacilityName() ; 
			String fmpFacilityCity = facilityMemberProvider.getFacilityCity() ; 
			String fmpFacilityAddr1 = facilityMemberProvider.getFacilityStreetAddress1( ) ; 
			String fmpFacilityAddr2 = facilityMemberProvider.getFacilityStreetAddress2( ) ;  
			String fmpFacilityState = facilityMemberProvider.getFacilityState( ) ;  
			
			String fmpProviderSId = facilityMemberProvider.getProviderSID();
			String fmpProviderClass = facilityMemberProvider.getProviderClass() ; 
			String fmpProviderFirstName = facilityMemberProvider.getProviderFirstName();
			String fmpProviderLastName = facilityMemberProvider.getProviderLastName();
			String fmpProviderNamePrefix = facilityMemberProvider.getProviderNamePrefix( ) ; 
			String fmpProviderNameSuffix = facilityMemberProvider.getProviderNameSuffix( ) ; 
			
			String fmpPatientICN = facilityMemberProvider.getPatientICN() ; 
			String fmpClinicStopCode = facilityMemberProvider.getClinicStopCode();
			
			DSFacility tempFacility = findFacilityByFacilityCode( fmpFacilityCode , dsFacilityCollection );
			if (tempFacility == null) {

				tempFacility = new DSFacility();
				tempFacility.setFacilityParentSiteCode(fmpFacilityParentSiteCode);
				tempFacility.setFacilityCode(fmpFacilityCode);
				tempFacility.setFacilityType(fmpFacilityType);
				tempFacility.setFacilityName( fmpFacilityName ); 
				tempFacility.setFacilityCity(fmpFacilityCity); 
				tempFacility.setFacilityAddr1(fmpFacilityAddr1);
				tempFacility.setFacilityAddr2(fmpFacilityAddr2);
				tempFacility.setFacilityState(fmpFacilityState);
				tempFacility.setPatientICN(fmpPatientICN);

				dsFacilityCollection.add(tempFacility);
			}
	
			DSFacilityClinic facilityClinic = findClinicByStopCode(fmpClinicStopCode, tempFacility.getFacilityClinics());
			if (facilityClinic == null) {

				facilityClinic = new DSFacilityClinic();
				facilityClinic.setStopCode(fmpClinicStopCode);
				tempFacility.getFacilityClinics().add(facilityClinic);
			}

			DSFacilityProvider facilityProvider = new DSFacilityProvider();
			facilityProvider.setProviderId(fmpProviderSId);
			facilityProvider.setProviderClass(fmpProviderClass);
			facilityProvider.setFirstName(fmpProviderFirstName);
			facilityProvider.setLastName(fmpProviderLastName);
			facilityProvider.setProviderNamePrefix(fmpProviderNamePrefix);
			facilityProvider.setProviderNameSuffix(fmpProviderNameSuffix);

			facilityClinic.getProviders().add(facilityProvider);
		}

	}
	
	private DSFacility findFacilityByFacilityCode(String facilityCode , Collection<DSFacility> facilityList) {

		if (facilityList == null || facilityList.size() == 0) {

			return (null);
		}

		for (DSFacility facility : facilityList) {

			if (facility.getFacilityCode() != null && facility.getFacilityCode().equalsIgnoreCase(facilityCode)) {

				return (facility);
			}
		}

		return (null);
	}


	private DSFacilityClinic findClinicByStopCode(String stopCode, Collection<DSFacilityClinic> dsFacilityClinicCollection) {

		if (dsFacilityClinicCollection == null || dsFacilityClinicCollection.size() == 0) {

			return (null);
		}

		for (DSFacilityClinic dsFacilityClinic : dsFacilityClinicCollection) {

			if (dsFacilityClinic.getStopCode() != null && dsFacilityClinic.getStopCode().equals(stopCode)) {

				return (dsFacilityClinic);
			}
		}

		return (null);
	}
	
	private List<ClinicProvider> getClinicByProvider()
	{	
		if(clinicProviderList==null){
			return clinicByProviderDao.getClinicNameByProvider();
		}
		return(clinicProviderList);
	}
	
	private void loadVARFacilityMembersIntoDSFacilities(PatientProviders providers, Collection<FacilityMember> facilityMemberCollection , DSFacilities dsFacilities ) {

		if (providers == null || providers.size() == 0) {
			return;
		}

		List<DSFacility> dsFacilityCollection = dsFacilities.getFacilityList() ;
		for(PatientProvider provider : providers){
			PatientProviderWithFacility providerWithFacility = (PatientProviderWithFacility) provider;
			String fmpFacilityCode = providerWithFacility.getFacilityCode() ; 
			String fmpFacilityName = providerWithFacility.getFacilityName() ; 
			String fmpPatientICN = providerWithFacility.getPatientId();
			
			DSFacility tempFacility = findFacilityByFacilityCode( fmpFacilityCode , dsFacilityCollection );
			if (tempFacility == null) {

				tempFacility = new DSFacility();
				tempFacility.setFacilityCode(fmpFacilityCode);
				tempFacility.setFacilityName( fmpFacilityName ); 
				tempFacility.setPatientICN(fmpPatientICN);

				dsFacilityCollection.add(tempFacility);
			}
		}
	}
}
