package com.agilex.healthcare.veteranappointment.datalayer.appointment;

import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.veteranappointment.domain.VARAppointmentRequest;
import com.agilex.healthcare.veteranappointment.domain.VARAppointmentRequestMessage;
import com.agilex.healthcare.veteranappointment.domain.VARAppointmentRequestMessages;
import com.agilex.healthcare.veteranappointment.domain.VARAppointmentRequests;

import javax.annotation.Resource;

public class AppointmentRequestDataLayerRepo implements AppointmentRequestDataLayer {

	private static final org.apache.commons.logging.Log LOGGER = org.apache.commons.logging.LogFactory.getLog(AppointmentRequestDataLayerRepo.class);

	@Resource
	private AppointmentRequestDao dao;

	@Override
	public VARAppointmentRequests fetchAppointmentRequestsByPatient(String patientId, DateFilter filter) {
		LOGGER.debug(String.format("Getting AppointmentRequest with [patientId: %s]", patientId));
		return dao.getActiveAppointmentRequests(patientId, filter);
	}

	public VARAppointmentRequest saveAppointmentRequest(VARAppointmentRequest appointmentRequest) {
		LOGGER.debug(String.format("Saving AppointmentRequest with [id: %s][patientId: %s]", appointmentRequest.getUniqueId(), appointmentRequest.getPatientId()));
		return dao.saveAppointmentRequest(appointmentRequest);
	}

	public void deleteAppointmentRequest(VARAppointmentRequest appointmentRequest) {
		LOGGER.debug(String.format("Deleting AppointmentRequest with [id: %s]", appointmentRequest.getUniqueId()));
		dao.deleteAppointmentRequest(appointmentRequest);
	}

	@Override
	public VARAppointmentRequest fetchAppointmentRequest(String patientId, String appointmentRequestId) {
		LOGGER.debug(String.format("Getting Filtered AppointmentRequests [AppointmentRequestId: %s] [PatientId: %s]", appointmentRequestId, patientId));
		return dao.getActiveAppointmentRequest(patientId, appointmentRequestId);
	}

    @Override
    public VARAppointmentRequestMessages fetchAppointmentRequestMessages(String patientId, String appointmentRequestId) {
    	LOGGER.debug(String.format("Getting AppointmentRequestMessages for [AppointmentRequestId: %s] [PatientId: %s]", appointmentRequestId, patientId));
    	return dao.getAppointmentRequestMessagesByAppointmentRequestId(patientId, appointmentRequestId);
    }

    @Override
    public VARAppointmentRequestMessage saveAppointmentRequestMessage(String patientId, VARAppointmentRequestMessage appointmentRequestMessage, boolean isProvider) {
    	LOGGER.debug(String.format("Saving appointmentRequestMessage "));

		return dao.saveAppointmentRequestMessage(patientId, appointmentRequestMessage, isProvider);
    }

    @Override
	public VARAppointmentRequests fetchAppointmentRequests(AppointmentRequestFilter filter) {
		LOGGER.debug(String.format("Getting AppointmentRequest with [Filter: %s]", filter));
		return dao.getAppointmentRequests(filter);
	}

	@Override
	public VARAppointmentRequest updateMessageFlag(String patientId, String appointmentRequestId, boolean isProvider) {
		LOGGER.debug("Updating message flag for Appointment Request Id = " + appointmentRequestId + " PatientId = " + patientId + " IsProvider = " + isProvider);
		return dao.updateAppointmentRequestMessageFlag(patientId, appointmentRequestId, isProvider);
	}

	@Override
	public VARAppointmentRequest markAppointmentRequestIsSeenByProvider(String patientId, String appointmentRequestId) {
		LOGGER.debug("Updating Provider_Seen_Appointment_Request flag for Appointment Request Id = " + appointmentRequestId + " PatientId = " + patientId);
		return dao.updateProviderSeenAppointmentRequestFlag(patientId, appointmentRequestId);
	}
}
