package com.agilex.healthcare.veteranappointment.datalayer.facility;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.TypedQuery;

import org.springframework.stereotype.Repository;

import com.agilex.healthcare.veteranappointment.datalayer.facility.FacilityDataLayer;
import com.agilex.healthcare.veteranappointment.datalayer.facility.FacilityFilter;
import com.agilex.healthcare.veteranappointment.datalayer.facility.FacilityFilterer;
import com.agilex.healthcare.veteranappointment.domain.CDWFacilities;

@Repository
public class CdwFacilityDataLayer implements FacilityDataLayer {
	
	@PersistenceContext(unitName="cdwPersistenceUnit")
    protected EntityManager entityManager;
	
	private static final String FACILITY_QUERY = "FROM CDWFacilityPo";

	@Override
	public CDWFacilities fetchFacilities(FacilityFilter filter) {
		final TypedQuery<CDWFacilityPo> query = this.entityManager.createQuery(FACILITY_QUERY, CDWFacilityPo.class);
		List<CDWFacilityPo> facilityPos = query.getResultList();
		
		CDWFacilities facilities = toFacilities(facilityPos);
		return filter(facilities, filter);
	}
	
	CDWFacilities toFacilities(List<CDWFacilityPo> facilityPos) {
		CDWFacilities facilities = new CDWFacilities();
		
		for(CDWFacilityPo facilityPo : facilityPos) {
			facilities.add(facilityPo.createFacility());
		}
		
		return facilities;
	}
	
	CDWFacilities filter(CDWFacilities facilities, FacilityFilter filter) {
		FacilityFilterer filterer = new FacilityFilterer(); 

		return filterer.filter(facilities, filter);
	}
}
