package com.agilex.healthcare.veteranappointment.datalayer.facility;

import java.util.List;

import javax.annotation.Resource;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Response.Status;

import org.springframework.stereotype.Component;

import com.agilex.healthcare.veteranappointment.datalayer.facility.FacilityDataLayer;
import com.agilex.healthcare.veteranappointment.datalayer.facility.FacilityFilter;
import com.agilex.healthcare.veteranappointment.datalayer.facility.FacilityFilterFactory;
import com.agilex.healthcare.veteranappointment.domain.CDWFacilities;
import com.agilex.healthcare.veteranappointment.domain.CDWFacility;
import com.agilex.healthcare.utility.NullChecker;

@Component
public class FacilityDataService {
	
	@Resource
	FacilityDataLayer facilityDataLayer;
	
	public CDWFacilities fetchFacilities(FacilityFilter facilityFilter) {
		return fetchFacilities(facilityFilter,true);
	}
	
	public CDWFacilities fetchFacilities(FacilityFilter facilityFilter, boolean useCache) {
		CDWFacilities facilities = null;
		if (useCache) {
			facilities = FacilityCache.getInstance().getFacilities(facilityFilter);
			if (facilities == null) {
				facilities = facilityDataLayer.fetchFacilities(facilityFilter);
				FacilityCache.getInstance().addFacilitiesToCache(facilityFilter,facilities);
			}
		} else {
			facilities =  facilityDataLayer.fetchFacilities(facilityFilter);
		}
		return facilities;
	}
		
	public CDWFacility fetchFacilityByCode(List<String> pilotSites, String code) {
		return fetchIndividualFacility(pilotSites, code, null);
	}

	
	public CDWFacility fetchFacilityByName(List<String> pilotSites, String name) {
		return fetchIndividualFacility(pilotSites, null, name);
	}

	
	private CDWFacility fetchIndividualFacility(List<String> pilotSites, String code, String name) {
		FacilityFilterFactory facilityFilterFactory = new FacilityFilterFactory(pilotSites);
		FacilityFilter filter = facilityFilterFactory.createEmptyFacilityFilter();
		if (NullChecker.isNotNullish(code)) {
			filter.addFacilityCode(code);
		}
		if (NullChecker.isNotNullish(name)) {
			filter.setFacilityName(name);
		}
		CDWFacilities facilities = fetchFacilities(filter);
		if (facilities.size() == 0) {
			throw new WebApplicationException(Status.NOT_FOUND);
		} else if (facilities.size() > 1) {
			throw new WebApplicationException(Status.PRECONDITION_FAILED);
		}
		
		return facilities.get(0);
	}
}
