package com.agilex.healthcare.veteranappointment.datalayer.facility;

import com.agilex.healthcare.utility.NullChecker;
import com.agilex.healthcare.veteranappointment.domain.CDWFacilities;
import com.agilex.healthcare.veteranappointment.domain.CDWFacility;

public class FacilityFilterer {
	private String delims = "[ ,]+";
	
	public CDWFacilities filter(CDWFacilities facilities, FacilityFilter filter) {
		CDWFacilities filtered;
		
		if (filter == null) {
			filtered = facilities;
		} else {
			String[] tokens = filter.getTerm() == null ? null : filter.getTerm().toLowerCase().split(delims);
			filtered = new CDWFacilities();
			
			for(CDWFacility facility : facilities) {
				if (!filter.isParentFacilitiesOnly() || facility.getFacilityCode().equals(facility.getParentSiteCode()) ) {
					String stringToSearch = buildString(facility.getName(), facility.getCity(), facility.getState());
					
					boolean match = true;
					if (tokens != null) {
						for (String token : tokens) {
							if (!stringToSearch.contains(token)) {
								match = false;
								break;
							}
						}
					}
					
					if (match && NullChecker.isNotNullish(filter.getParentSiteCodes()) && !filter.getParentSiteCodes().contains(facility.getParentSiteCode())) {
						match = false;
					}
					
					if (match && NullChecker.isNotNullish(filter.getFacilityCodes()) && !filter.getFacilityCodes().contains(facility.getFacilityCode())) {
						match = false;
					}
					
					if (match && NullChecker.isNotNullish(filter.getFacilityName()) && !filter.getFacilityName().toLowerCase().equals(facility.getName().toLowerCase())) {
						match = false;
					}
					
					if (match) {
						filtered.add(facility);
					}
				}
			}
		}
		
		return filtered;
	}
	
	private String buildString(String...args) {
		StringBuilder sb = new StringBuilder();
		
		if (args != null) {
			for (String arg : args) {
				if (NullChecker.isNotNullish(arg)) {
					sb.append(arg.toLowerCase());
					sb.append(" ");
				}
			}
		}
		
		return sb.toString();
	}
}
