define(['Global', 'BookedAppointment', 'moment'],
    function(Global, BookedAppointment) {
    	'use strict';

        var BookedAppointments = Backbone.Collection.extend({
            model: BookedAppointment,
            initialize:function(models, config){
            	config = config || {};
            	this.messageForNone = config.messageForNone;
            	this.messageForFound = config.messageForFound;
            	this.messageForLoading = config.messageForLoading;
            },
            parse: function(response) {
	            var appointments = [];

	            _.each(response.bookedAppointmentCollections, function(collection){
	            	_.each(collection.bookedAppointments, function(appt){
	            		appt.appointmentDesignation = collection.collectionName;
	            	});
	            });

	            appointments = _.flatten(_.map(response.bookedAppointmentCollections, function(i){ return i.bookedAppointments }));
	            if(appointments[0]){
		            appointments = _.sortBy(appointments, function(appt){
		            	return moment(appt.appointmentTime);
		            });
		            return appointments;
	        	}
	        	return [];
				
            },
			findById: function (id) {
				return this.find(function (model) {
					return model.get('appointmentId') === id;
				});
			},

			onlyPrimaryCare: function(noCancelled){
				var appts = this.filter(function(model){
					return model.isPrimaryCareAppt();
				});
				if(noCancelled){
					return this.noCancelledAppts(this.clone().reset(appts));
				}
				return appts;
			},

			noCancelledAppts: function(collection) {
				var that = this,
					coll = this,
					appts;

				if(collection){
					coll = collection;
				}

				appts = coll.filter(function (model) {
					return !that.isCancelled(model.get('currentStatus'));
				});

				return appts;
			},
			isCancelled: function(status) {
				var cancelled = false;

				switch (status) {
					case "NO-SHOW":
					case "CANCELLED BY CLINIC":
					case "NO-SHOW & AUTO RE-BOOK":
					case "CANCELLED BY CLINIC & AUTO RE-BOOK":
					case "INPATIENT APPOINTMENT":
					case "CANCELLED BY PATIENT":
					case "CANCELLED BY PATIENT & AUTO-REBOOK":
					case "NO ACTION TAKEN":
						cancelled = true;
				}

				return cancelled;
			},
			getFoundMessage: function() {
				if(this.length){
					return this.messageForFound;
				}
				return this.messageForNone;
			},
			getLoadingMessage: function() {
				if(!this.models.length){
					return this.messageForLoading;
				}
				return this.messageForLoading.replace(/Loading/i, 'Updating');
			},
	        setFacility: function(facility) {
		        this.facility = facility.get('facilityName');

		        this.messageForFound = this.messageForFound.replace('(facility)', this.facility);
	        }
        });

        return BookedAppointments;
    }
);
