define([
	'Global',
	'Resources',
	'VarResources',
	'RequestMetaData',
	'AppointmentRequest'
], function(Global, Resources, VarResources, RequestMetaData, AppointmentRequest){
	'use strict';

	return Backbone.Collection.extend({
		model: AppointmentRequest,
		url: VarResources.instance().get("var-appointments").get('href'),
		initialize: function(options) {
			var that 		= this,
				endDate 	= moment(),
				startDate 	= moment(),
				numberOfDaysInThePast;
				
			RequestMetaData.deferred.done(function(){
				numberOfDaysInThePast = RequestMetaData.get('farthestTimeVeteranCanRequestAppointmentInDays');
				startDate.subtract(numberOfDaysInThePast, 'days');

				that.url += "?startDate=" + startDate.format('MM/DD/YYYY') + "&endDate=" + endDate.format('MM/DD/YYYY');
				that.deferred = that.fetch({
					cache: false,
					success: function () {
						that.fetched = true;
					}
				});
			});
		},
		parse: function(response) {
			var that = this;
			this.lastAccessDate = response.lastAccessDate;
			_.each(response.appointmentRequest, function(appt){
				appt.lastAccessDate = that.lastAccessDate;
				Global.getUtilities().flattenLinks(appt);
				appt.hidden = "";
				if(that.isBlank(appt.providerOption)){
					appt.hidden = "display-none-important";
				}
			});
			return response.appointmentRequest;
		},
		countUnreadMessages: function () {
			return _.filter(this.models, function(model){ return model.get("hasVeteranNewMessage") } ).length;
		},
		primaryCareRequests: function(){
			return _.filter(this.models, function(model){ return model.isPrimaryCareAppt(); });
		},
		mentalHealthRequests: function(){
			return _.filter(this.models, function(model){ return model.isMentalHealth(); });
		},
		primaryCareLimitReached: function(){
			var primaryCareRequests = this.primaryCareRequests();
			return _.filter(primaryCareRequests, function(model){ return model.isActiveStatus(); }).length >= 1;
		},
		mentalHealthLimitReached: function(){
			var mentalHealthRequests = this.mentalHealthRequests();
			return _.filter(mentalHealthRequests, function(model){ return model.isActiveStatus(); }).length >= 2;
		},
		isBlank: function (value) {
			return ( value == undefined || value == null || value == 'undefined' || value === "" )
		}

	});
	
});