define([
    'Global'
],
    function(Global) {
        var BookedAppointment = Backbone.Model.extend({
            defaults: {
                "object-type": "",
                "appointmentTime": "",
				"appointmentId": "",
				"apptLength": 30,
				"bookingNote": "",
				"clinic": {
					"askForCheckIn": "",
					"name": ""
				},
				"currentStatus": "",
				"providerName": "",
				"purpose": "",
                "type": ""
            },
			initialize: function () {
				Global.getUtilities().flattenLinks(this);
			},
            parse: function(response) {
				var startDate = moment(response.appointmentTime || response.appointmentStartDate, "MM/DD/YYYY HH:mm:ss"),
					endDate = moment(startDate);

				response.appointmentId = startDate + ";" + response.clinic.name;
				response.start = startDate;
				response.appointmentDate = startDate.format("MM/DD/YYYY");
				response.appointmentTime = startDate.format("HH:mm");
				response.apptLength = response.appointmentLength || 30;
				response.end = endDate.add(response.apptLength, "minutes");
				response.endTime = response.end.format("MM/DD/YYYY HH:mm:ss");
				response.detailsLink = response.appointmentId;
				response.clinicName = response.clinic.name;
				response.clinicId = response.clinic.id;
				response.askForCheckIn = response.clinic.askForCheckIn ? "Yes" : "No";

                return response;
            },
            getFormattedStartDate: function(){
				return this.get('start')._i;
            },
            isPrimaryCareAppt: function(){
            	return /^PRIMARY_CARE$/i.test(this.get('appointmentDesignation'));
            },
            getFormattedDate: function(property, format){
            	return moment(this.get(property)).format((format || "MM/DD/YYYY HH:mm:ss"));
            }
        });

        return BookedAppointment;
    }
);

