define(['Global', 'Patient'],
	function (Global, patient) {
		'use strict';

		return Backbone.Model.extend({
			defaults: {
				"object-type": "AppointmentRequest",
				"dataIdentifier": {
					"uniqueId": "",
					"systemId": ""
				},
				"patientIdentifier": {
					"uniqueId": "",
					"assigningAuthority": ""
				},
				"link": {
					"href": ""
				},
				"patient": {
					"patientId": "",
					"firstName": "",
					"lastName": "",
					"textMessagingAllowed": false,
					"textMessagingPhoneNumber": ""
				},
				"createdDate": "",
				"lastUpdatedDate": "",
				"secondRequest": false,
				"secondRequestSubmitted": false,
				"appointmentDate": "",
				"appointmentTime": "",
				"requestedPhoneCall": "",
				"optionDate1": "",
				"optionTime1": "",
				"optionDate2": "",
				"optionTime2": "",
				"optionDate3": "",
				"optionTime3": "",
				"appointmentType": "",
				"visitType": "",
				"purposeOfVisit": "",
				"otherPurposeOfVisit": "",
				"bestTimetoCall": [],
				"providerId": "",
				"providerName": "",
				"providerPersonClass": "",
				"providerOption": "",
				"facilityName": "",
				"facility": {
					"name": "",
					"type": "",
					"facilityCode": "",
					"state": "",
					"city": "",
					"address": "",
					"parentSiteCode": ""
				},
				"email": "",
				"textMessagingAllowed": false,
				"textMessagingPhoneNumber": "",
				"phoneNumber": "",
				"status": "",
				"hidden": "",
				"hasVeteranNewMessage": false,
				"hasProviderNewMessage": false,
				"providerSeenAppointmentRequest": false,
				"appointmentRequestDetailCode": []
			},
			initialize: function (a, b, c) {
				if (!this.has('appointmentRequestDetailCode')) {
					this.set('appointmentRequestDetailCode', []);
				}
				window.apptRequest = this;
			},
			prepareForSave: function () {
				var requestedPhoneCall = this.get('requestedPhoneCall') ? true : false,
					optionDate2 = this.get('optionDate2'),
					optionTime2 = this.get('optionTime2'),
					optionDate3 = this.get('optionDate3'),
					optionTime3 = this.get('optionTime3'),
					providerName = this.get('providerName') || this.getProviderName(),
					patientInst = patient.instance(),
					timestamp = moment().lang('en').format('L HH:mm:ss');

				if (!optionDate2) {
					optionDate2 = "No Date Selected";
					optionTime2 = "No Time Selected";
				}

				if (!optionDate3) {
					optionDate3 = "No Date Selected";
					optionTime3 = "No Time Selected";
				}

				this.set({
					requestedPhoneCall: requestedPhoneCall,
					optionDate2: optionDate2,
					optionTime2: optionTime2,
					optionDate3: optionDate3,
					optionTime3: optionTime3,
					patient: patientInst.toJSON(),
					patientIdentifiers: patientInst.get("patientIdentifier"),
					patientIdentifier: patientInst.get("patientIdentifier"),
					createdDate: timestamp,
					"lastUpdatedDate": timestamp,
					providerName: providerName
				});
			},
			setDetailCodeForSave: function () {
				if (this.isSecondAttempt() || this.getDetailCode().length === 0) {
					this.unset("appointmentRequestDetailCode", {silent: true});
				}
			},
			containsDetailCode: function (code) {
				var codes = this.getDetailCode();
				if (codes.length == 0) {
					return false;
				}
				return _.find(codes, function (code) {
					return code.detailCode.code === code;
				}) ? true : false;
			},
			hasDetailCode: function () {
				return this.getDetailCode().length > 0
			},
			hasAccessChanged: function (lastAccessDate) {
				lastAccessDate = new Date(lastAccessDate);
				if (lastAccessDate < this.getLastUpdatedDate(true) && !this.isSubmitted()) {
					var codes = this.getDetailCode();
					return !this.isCancelled() || ( codes.length > 0 && !this.containsDetailCode("DETCODE8") );
				}
				return false;
			},
			isSecondAttempt: function () {
				return this.has("parentRequest");
			},
			isValidSecondRequestDates: function (originalRequest) {
				if (originalRequest.getOptionDateTime1() != this.getOptionDateTime1()
					|| this.getOptionDateTime2() == this.getOptionDateTime3()) {
					return false;
				}

				if (_.intersection(
					[originalRequest.getOptionDateTime1(), originalRequest.getOptionDateTime2(), originalRequest.getOptionDateTime3()],
					[this.getOptionDateTime2(), this.getOptionDateTime3()]
				).length > 0
					) {
					return false;
				}

				return true;
			},
			isSecondRequestAllowed: function () {
				var lessThanSevenDays,
					secondRequestSubmitted = this.isSecondRequestSubmitted(),
					secondRequest = this.isSecondRequest(),
					firstDay = moment(new Date(this.get('lastUpdatedDate'))),
					seventhDay = moment(firstDay).add(7, 'days'),
					today = moment();

				seventhDay.hours(23);
				seventhDay.minutes(59);
				seventhDay.seconds(59);
				today.hours(0);
				today.minutes(0);
				today.seconds(0);

				lessThanSevenDays = today < seventhDay;

				return (lessThanSevenDays && !secondRequest && !secondRequestSubmitted);
			},
			isBookedWithDaysLeft: function (daysLeft) {
				return this.isBooked() && moment().add(daysLeft, 'days') > moment(new Date(this.getAppointmentDate()));
			},
			isSecondRequestSubmitted: function () {
				return this.get("secondRequestSubmitted");
			},
			isSecondRequest: function () {
				return this.get("secondRequest");
			},
			isBooked: function () {
				return this.getStatus() === "Booked";
			},
			isSubmitted: function () {
				return this.getStatus() === "Submitted";
			},
			isNotBooked: function () {
				return this.getStatus() === "Not Booked";
			},
			isCancelled: function () {
				return this.getStatus() === "Cancelled";
			},
			isActiveStatus: function () {
				return this.isSubmitted();
			},
			getStatus: function () {
				return this.get("status");
			},
			getLastUpdatedDate: function (asDate) {
				var date = this.get("lastUpdatedDate");
				if (asDate) {
					return new Date(date || null);
				}
				return date;
			},
			getAppointmentDate: function (asDate) {
				var date = this.get("appointmentDate");
				if (asDate) {
					return new Date(date || null);
				}
				return this.get("appointmentDate");
			},
			getDetailCode: function () {
				return this.get("appointmentRequestDetailCode") || [];
			},
			getOptionDateTime1: function () {
				return moment(this.get("optionDate1"), 'MM/DD/YYYY') + this.get("optionTime1");
			},
			getOptionDateTime2: function () {
				return moment(this.get("optionDate2"), 'MM/DD/YYYY') + this.get("optionTime2");
			},
			getOptionDateTime3: function () {
				return moment(this.get("optionDate3"), 'MM/DD/YYYY') + this.get("optionTime3");
			},
			getProviderName: function () {
				var appointmentType = this.get('appointmentType'),
					providerId = this.get('providerId'),
					providers;

				switch (appointmentType) {
					case "Primary Care":
						providers = this.primaryCareProviders;

						break;
					case "Mental Health":
						providers = this.mentalHealthProviders;

						break;
				}

				if(providers){
					providers = providers.get('provider');

					return $.grep(providers, function (provider) {
						return provider.providerId === providerId;
					})[0].providerName;
				}
				return "";
			},
			getAppointmentType: function () {
				return this.get('appointmentType');
			},
			isPrimaryCareAppt: function (config) {
				var primaryCare = /^primary care$/i.test(this.getAppointmentType());
				if (config && config.filterStatus) {
					return primaryCare && this['is' + config.filterStatus]();
				}
				return primaryCare;
			},
			isMentalHealth: function (config) {
				var mentalHealth = /^mental health$/i.test(this.getAppointmentType());
				if (config && config.filterStatus) {
					return mentalHealth && this['is' + config.filterStatus]();
				}
				return mentalHealth;
			}
		});

	});