define(
	['Resources', 'Vent'],
	function(resources, vent) {
		var UserSession = Backbone.Model.extend({
			initialize: function(data, config) {
				config = $.extend(config, {});
				this.url = resources.get('public-user-session').get('href');
				this.fetch({async: false});
				this.set("sessionTimeoutIn", this.get("clientTimeout") );
				this.set("timeUnits", "minutes" );
				var sessionTimer = _.once(this._sessionTimer);
				if( !/false/.test(config.trackSession) ){
					sessionTimer.call(this);
				}
			},
			isLoggedIn: function(){
				var mhpUser 	= this.get("mhpuser");
				return mhpUser && mhpUser.id;
			},
			_sessionTimer: function(){
				var that					= this;
				var minuteAsMilliseconds 	= 60 * 1000;
				var warningThreshold 		= 3 * minuteAsMilliseconds;
				var triggerTimeout			= 0;
				var logoutThreshold			= 0;
				var logoutTimer;

				function checksession(){
					$.ajax({
						url: resources.get('last-accesstime').toJSON().href,
						dataType: 'json',
						global: false,
						success: function(data){
							logoutThreshold = data.timeToExpireInSeconds * 1000;
							var minutes 	= logoutThreshold/1000/60;
							var timeLeft 	= Math.floor(minutes);
							var fix			= (timeLeft <= 0 && logoutThreshold > 0 ) ? 1 : 0;
							timeLeft += fix;
							if(timeLeft < 0){
								timeLeft = 0;
							}
							that.set("sessionTimeoutIn", timeLeft );
							that.set("timeUnits", timeLeft === 1 ? "minute" : "minutes" );
							triggerTimeout = logoutThreshold - warningThreshold;
							if( logoutThreshold <= warningThreshold && logoutThreshold > 0 ){
								vent.trigger("show:session:warning");
								startCountDown();
							}
							else if( timeLeft <= 0 || logoutThreshold <= 0 ){
								clearTimeout(countdownTimer);
								clearTimeout(logoutTimer);
								vent.trigger("do:logout:MDWS");
								vent.trigger("show:session:expired");
							}
							else{
								sessionTimeout();
							}
						}
					});
				}

				function startCountDown(){
					countdownTimer = setTimeout(function(){
						checksession();
					}, 60000 ); //check against logoutThreshold difference from a full minute .... 115 seconds, call timer in 55 seconds instead of 60 seconds to sync back up.
				}

				function sessionTimeout(){
					logoutTimer = setTimeout(function(){
						checksession();
					}, Math.abs(triggerTimeout) );
				}

				checksession();
			}

		});
		return UserSession;
	}
);