define(function(require) {
    var Global = require('Global');
    var vent = require('Vent');
    var Resources = require('Resources');
    var UserSession = require('UserSession');
    var MainView = require('MainView');
    var LandingPageHeaderView = require('LandingPageHeaderView');
    var LandingPageNotLoggedInView = require('LandingPageNotLoggedInView');
    var FooterView = require('FooterView');

    var RedirectRouter = Backbone.Router.extend({
        route: function(route, name, callback){
            var that = this;
            return Backbone.Router.prototype.route.call(this, route, name, function(){
                if( that.loggedIn() ){
                    that.app();
                }
                else{
                    if (!callback) callback = this[name];
                    callback.apply(this, arguments);
                }
            });
        },
        initialize: function(){
            this.userSession = new UserSession({},{trackSession:false});
        },
        routes:{
            '': 'checklogin',
            'app': 'app',
            '*actions': 'landing'
        },
        loggedIn: function(){
            return this.userSession && this.userSession.isLoggedIn();
        },
        app: function(){
            if( !this.loggedIn() ){
                return this.navigate("", {trigger:true, replace:false});
            }
            Backbone.history.stop();
            require(['AppRouter'], function(AppRouter){});
        },
        checklogin: function(){
            this.navigate("landing", {trigger:true});
        },
        landing: function(){
            this.mainView = new MainView().render();
            this.mainView.header.show(new LandingPageHeaderView());
            this.mainView.content.show(new LandingPageNotLoggedInView());
            this.mainView.footer.show(new FooterView({},{showUser: false}));

            this.setUpWindowResize();
        },
        toggleRegion: function( region, action ){
            this.mainView[region][action]();
            $(this.mainView[region].el)[(action=="close"? "hide" : "show")]();
        },
        setUpWindowResize: function () {
            $(window).resize(function(){vent.trigger("resize:panes")});
            this.mainView.content.on("show", function(){
                vent.trigger("resize:panes");
            }).trigger("show");
            $('.ui-page').animationComplete(function () {
                $(window).resize();
            });
        }
    });

    var redirectRouter = new RedirectRouter();

    vent.on("resize:panes", function(){
        Global.getUtilities().resizePanes();
    });


    Backbone.history.start();
    return redirectRouter;

});