define([
    'Global', 'Vent',

    'AppointmentRequests', 'AppointmentRequest', 'BookedAppointments', 'DSCancelReasons',

    'AppointmentRequestMessages', 'AppointmentRequestMessage',
    'ErrorMessages', 'Facilities', 'CalendarData', 'Navigation',
    'PatientLinks', 'Resources', 'VarResources',
    'DSResources',

    'User', 'UserSession', 'RequestMetaData',
    'Patient', 'PatientIdentifiers', 'NotificationPreferences', 'FacilitiesProvidersCollection',
    'ProviderClinics', 'LandingPage',
    'RoaPageView', 'FacilitySelectionList',

    'MainView', 'AppointmentRequestDetailsView', 'AppointmentRequestDetailsHeaderView',
    'AppointmentRequestsListView', 'RequestListHeaderView', 'AppointmentRequestDetailsStatusView',
    'AppointmentRequestDetailsDetailsView', 'AppointmentRequestDetailsMessagesView',
    'AppointmentRequestSubmittedView',

    'NotificationPreferenceSettingsListView', 'LandingPageView', 'LandingPageHeaderView', 'BookingLandingPageView',

    'DSAppointmentListing', 'DSCancelAppt', 'DSCancelApptConfirm', 'ErrorCancelingApptView', 'DSFacilityProviderInfoView',
    'DSFacilitySelectionView', 'DSViewApptFacilitySelectionView', 'DSClinicSelectionView', 'DSClinicFacilityInfoView',
    'DSHowToEnroll', 'DSNoFacilitiesView', 'DSNotEligible', 'PACTListView', 'DSPACTFacilityListView', 'DSNonPACTFacilityListView',

    'CalendarDayNavView', 'CalendarLegendView', 'CalendarMonthNavView', 'CalendarView',
    'CalendarDayCompositeView', 'CalendarDayItemView', 'CalendarDayEmptyView', 'ConfirmBookingApptView', 'ErrorBookingApptView',
    'TimeSlotSelectedView', 'DSBackBtnNavView', 'DSVeteranEmailView', 'EmailConfirmationView',

    'HeaderView', 'PopupHeaderView', 'MenuView', 'UserMenuView', 'DontSeeExpectedView',
    'InstructionalHeader',

    'FooterView', 'EulaFooterView', 'EulaView', 'EulaHeaderView',
    'StatusFilterView',

    'AboutDialogView', 'ConfirmCancellationAlertDialogView', 'ErrorPopupView',
    'FeedbackDialogView', 'RequestNoticeDialogView', 'SessionWarningView',
    'SessionExpiredView',
    'NewAppointmentRequestReviewView',

    'NewRequestDialogView',
    'UnrecognizedNoticeDialogView',
    'UnfamiliarFacilityDialogView'
], function (Global, vent, AppointmentRequests, AppointmentRequest, BookedAppointments, DSCancelReasons, AppointmentRequestMessages, AppointmentRequestMessage, ErrorMessages, Facilities, CalendarData, Navigation, PatientLinks, Resources, VarResources, DSResources, User, UserSession, RequestMetaData, Patient, PatientIdentifiers, NotificationPreferences, FacilitiesProvidersCollection, ProviderClinics, LandingPage, RoaPageView, FacilitySelectionList, MainView, AppointmentRequestDetailsView, AppointmentRequestDetailsHeaderView, AppointmentRequestsListView, RequestListHeaderView, AppointmentRequestDetailsStatusView, AppointmentRequestDetailsDetailsView, AppointmentRequestDetailsMessagesView, AppointmentRequestSubmittedView, NotificationPreferenceSettingsListView, LandingPageView, LandingPageHeaderView, BookingLandingPageView, DSAppointmentListing, DSCancelAppt, DSCancelApptConfirm, ErrorCancelingApptView, DSFacilityProviderInfoView, DSFacilitySelectionView, DSViewApptFacilitySelectionView, DSClinicSelectionView, DSClinicFacilityInfoView, DSHowToEnroll, DSNoFacilitiesView, DSNotEligible, PACTListView, DSPACTFacilityListView, DSNonPACTFacilityListView, CalendarDayNavView, CalendarLegendView, CalendarMonthNavView, CalendarView, CalendarDayCompositeView, CalendarDayItemView, CalendarDayEmptyView, ConfirmBookingApptView, ErrorBookingApptView, TimeSlotSelectedView, DSBackBtnNavView, DSVeteranEmailView, EmailConfirmationView, HeaderView, PopupHeaderView, MenuView, UserMenuView, DontSeeExpectedView, InstructionalHeader, FooterView, EulaFooterView, EulaView, EulaHeaderView, StatusFilterView, AboutDialogView, ConfirmCancellationAlertDialogView, ErrorPopupView, FeedbackDialogView, RequestNoticeDialogView, SessionWarningView, SessionExpiredView, NewAppointmentRequestReviewView, NewRequestDialogView, UnrecognizedNoticeDialogView, UnfamiliarFacilityDialogView) {

    'use strict';
    $.mobile.ignoreContentEnabled = true;
    var $window = $(window);

    var AppRouter = Backbone.Router.extend({
        routes: {
            '': 'showLanding',
            'book-appt': 'redirectToBookValidation',
            'view-appts': 'redirectToViewValidation',
            'scheduling': 'showBookingLandingPageView',
            'requests': 'showRequestListing',
            'eula': 'showEula',
            'request-cancel': 'cancelRequest',
            'request': 'showRequest',
            'new-request': 'showNewRequest',
            'new-request-phone': 'showNewRequestByPhone',
            '*actions': 'redirectToLanding'
        },
        initialize: function () {
            var that = this;
            this.utilities = Global.getUtilities();

            if (this.globalInit()) {
                this.models = {
                    user: User,
                    patient: Patient.instance(),
                    userSession: new UserSession(),
                    // mhpUser: new UserSession().get('mhpuser'),
                    requestMetaData: RequestMetaData,
                    notificationPreferences: new NotificationPreferences(),
                    varResources: VarResources.instance(),
                    patientIdentifiers: PatientIdentifiers,
                    calendarData: new CalendarData(),
                    dsCancelReasons: new DSCancelReasons(),
                    headerModel: new Backbone.Model({headerTitle: "VA Appointments"}),
                    instructionModel: new Backbone.Model({instructions: ""})
                };

                this.models.mhpUser = this.models.userSession.get('mhpuser');

                _.bindAll(this);

                this.collections = {
                    appointmentRequests: new AppointmentRequests(),
                    appointmentRequestMessages: new AppointmentRequestMessages(),
                    bookedAppointmentsSchedule: new BookedAppointments(),
                    bookedAppointments: new BookedAppointments(),
                    errorMessages: new ErrorMessages(),
                    landingPage: new LandingPage(),
                    navigation: Navigation,
                    nonPACTFacilities: new FacilitiesProvidersCollection({varResources: this.models.varResources}),
                    PACTs: new FacilitiesProvidersCollection({varResources: this.models.varResources, PACTs: true}),
                    PACTMembers: new Backbone.Collection(),
                    patientLinks: PatientLinks.instance(),
                    primaryCareNotCancelled: new BookedAppointments([], {messageForFound: 'Which (facility) appointment would you like to cancel?', messageForNone: 'No Primary Care Appointments found', messageForLoading: 'Loading Primary Care appointments'}),
                    providerClinics: new ProviderClinics(),
                    facilityResources: new Backbone.Collection()
                };

                this.views = {
                    mainView: new MainView().render(),
                    roaView: new RoaPageView(),
                    instructionalHeader: new InstructionalHeader({model: this.models.instructionModel}),
                    newRequestDialogView: new NewRequestDialogView({requests: this.collections.appointmentRequests}),
                    newRequestByPhone: new NewRequestDialogView({requests: this.collections.appointmentRequests, byPhone: true}),
                    notificationPreferenceSettings: new NotificationPreferenceSettingsListView({model: this.models.notificationPreferences}),
                    landingPageView: new LandingPageView({collection: this.collections.landingPage, requests: this.collections.appointmentRequests}),
                    requestListHeaderView: new RequestListHeaderView(),
                    appointmentRequestDetailsView: new AppointmentRequestDetailsView(),
                    appointmentRequestDetailsHeaderView: new AppointmentRequestDetailsHeaderView(),
                    appointmentRequestDetailStatusView: new AppointmentRequestDetailsStatusView(),
                    appointmentRequestDetailsDetailsView: new AppointmentRequestDetailsDetailsView(),
                    appointmentRequestSubmittedView: new AppointmentRequestSubmittedView(),
                    bookingLandingPageView: new BookingLandingPageView(),
                    eulaView: new EulaView(),
                    eulaFooterView: new EulaFooterView(),
                    eulaHeaderView: new EulaHeaderView(),
                    footerView: new FooterView({collection: this.collections.navigation, model: this.models.user}, {showUser: true}),
                    dsAppointmentListing: new DSAppointmentListing({collection: this.collections.bookedAppointments}),
                    dsCancelAppt: new DSCancelAppt({model: this.models.dsCancelReasons}),
                    dsCancelApptConfirm: new DSCancelApptConfirm(),
                    dsBackBtnNavView: new DSBackBtnNavView(),
                    dsNoFacilities: new DSNoFacilitiesView(),
                    dsNotEligible: new DSNotEligible(),
                    dsFacilityProviderInfoView: new DSFacilityProviderInfoView({model: this.models.calendarData}),
                    dsFacilitySelectionView: new DSFacilitySelectionView({model: this.models.calendarData}),
                    dsViewApptFacilitySelectionView: new DSViewApptFacilitySelectionView({model: this.models.calendarData}),
                    dsClinicFacilityInfoView: new DSClinicFacilityInfoView({model: this.models.calendarData}),
                    dsClinicSelectionView: new DSClinicSelectionView({model: this.models.calendarData, collection: this.collections.providerClinics}),
                    dsCalendarDayView: new CalendarDayCompositeView({model: this.models.calendarData}),
                    dsCalendarDayNavView: new CalendarDayNavView({model: this.models.calendarData}),
                    dsCalendarLegendView: new CalendarLegendView(),
                    dsCalendarMonthView: new CalendarView({model: this.models.calendarData}),
                    dsCalendarMonthNavView: new CalendarMonthNavView({model: this.models.calendarData}),
                    dsConfirmBookingApptView: new ConfirmBookingApptView(),
                    dsPACTFacilityListView: new DSPACTFacilityListView(),
                    dsNonPACTFacilityListView: new DSNonPACTFacilityListView(),
                    dsVeteranEmailView: new DSVeteranEmailView(),
                    emailConfirmationView: new EmailConfirmationView(),
                    timeSlotSelectedView: new TimeSlotSelectedView(),
                    headerView: new HeaderView({model: this.models.headerModel}),
                    menuView: new MenuView(),
                    newAppointmentRequestReviewView: new NewAppointmentRequestReviewView(),
                    userMenuView: new UserMenuView()
                };

                this.popups = {
                    aboutDialogView: new AboutDialogView(),
                    dsHowToEnroll: new DSHowToEnroll(),
                    errorBookingApptView: new ErrorBookingApptView(),
                    errorCancelingApptView: new ErrorCancelingApptView(),
                    feedbackDialogView: new FeedbackDialogView(),
                    popupHeaderView: new PopupHeaderView(),
                    popupFooterView: _.extend({}, this.views.footerView),
                    confirmCancellationAlertDialogView: new ConfirmCancellationAlertDialogView(),
                    requestNoticeDialogView: new RequestNoticeDialogView(),
                    sessionExpiredView: new SessionExpiredView(),
                    sessionWarningView: new SessionWarningView({model: this.models.userSession}),
                    unfamiliarFacilityDialogView: new UnfamiliarFacilityDialogView(),
                    unrecognizedNoticeDialogView: new UnrecognizedNoticeDialogView(),
                    PACTListView: new PACTListView({collection: this.collections.PACTMembers})
                };
                this.views.mainView.menu.show(this.views.menuView);
                this.views.mainView.userMenu.show(this.views.userMenuView);
                this.views.mainView.footer.show(this.views.footerView);
                this.setUpWindowResize();
                this.listenForValidation();
            }
        },
        globalInit: function () {
            var utilities = Global.getUtilities();

            utilities.internetConnectionCheck();
            //Check browser for compatibility
            utilities.browserAllowed();

            utilities.removeToken();
            return true;
        },
        setUpWindowResize: function () {
            $window.resize(function () {
                vent.trigger("resize:panes");
            });

            this.views.mainView.content.on("show", function () {
                $window.resize();
                $('#main-content').scrollTop(0);
            });

            $('.ui-page').animationComplete(function () {
                $window.resize();
            });
        },
        setUpLogoutOnClose: function () {
            var that = this;

            window.onbeforeunload = function (e) {
                that.logout(e);

                return null;
            };
        },
        redirectToLanding: function () {
            vent.trigger("show:landing");
        },
        wasEulaAccepted: function () {
            var value, eulaHtml, eulaModifiedDate;

            eulaHtml = new EulaView().template.call();
            eulaModifiedDate = $('#modified-date', eulaHtml).text();
            value = localStorage.getItem(window.location.pathname + 'eula/' + this.utilities.encrypt(User.id));

            return !(value === null || new Date(eulaModifiedDate).getTime() > value);
        },
        showLanding: function () {
            if (!this.models.user.checkForROA()) {
                var $main = $('main');
                this.changeMainPage();
                this.showPage('roaView', {title: 'Right of Access'});
                $main.removeAttr('style').children().eq(0).css('height', $(window).height());
                return;
            }
            if (this.wasEulaAccepted()) {
                this.changeMainPage();
                this.showPage('landingPageView', {title: 'VA Appointments'});
                if (!this.views.footerView.rendered) {
                    this.views.mainView.footer.show(this.views.footerView);
                    this.views.footerView.rendered = true;
                }
                return;
            }
            this.showEula();
        },
        showRequestListing: function (config) {
            config = config || {};
            this.changeMainPage();
            this.views.requestListView = this.views.requestListView || new AppointmentRequestsListView({collection: this.collections.appointmentRequests, model: this.models.notificationPreferences});
            this.views.requestListView.openSection = config.openSection;
            this.showPage('requestListView', {title: 'Notifications'});
            if (config.refetch) {
                this.collections.appointmentRequests.routerFetched = false;
                this.collections.appointmentRequests.fetched = false;
                this.collections.appointmentRequests.fetch({cache: false});
            }
            if (!this.collections.appointmentRequests.fetched) {
                this.showInstructionalHeader({instructions: 'Loading Appointment Requests'});
            } else if (!this.collections.appointmentRequests.length) {
                this.showInstructionalHeader({instructions: 'You have not requested any appointments'});
            }
            $window.resize();
        },
        redirectToBookValidation: function () {
            var that = this;

            this.showDirectSchedulingLanding({
                done: function () {
                    if (that.collections.PACTs.models.length) {
                        that.showDirectSchedulingBookApptFacility();
                    }
                    else {
                        that.showDirectSchedulingNotEligible();
                    }
                }
            });
        },
        redirectToViewValidation: function () {
            var that = this;

            this.showDirectSchedulingLanding({
                done: function () {
                    that.showDirectSchedulingViewApptFacility();
                }
            });
        },
        showDirectSchedulingLanding: function (config) {
            config = config || {};
            var that = this,
                done = $.isFunction(config.done) ? config.done : function () {
                };

            this.changeMainPage();
            if (this.collections.PACTs.isValidated) {
                return done();
            }

            this.views.instructionalHeader.close();
            this.views.mainView.content.close();

            this.showPage('', {title: 'Profile Validation', instructions: 'Please wait while we find your Primary Care team assignment'});
            this.collections.PACTs.fetch().done(function () {
                that.collections.PACTs.isValidated = true;
                done();
            });
        },
        showDirectSchedulingNotEligible: function () {
            this.changeMainPage();
            this.showPage('dsNotEligible', {title: 'Enrollment Not Found', instructions: ''});
        },
        showDirectSchedulingHowToEnroll: function (config) {
            config = config || {};
            this.popups.dsHowToEnroll.fromBooking = config.fromBooking;
            this.showPopup('dsHowToEnroll', 'Enroll');
        },

        showDirectSchedulingBookApptFacility: function () {
            if (!this.views.dsFacilitySelectionView.isClosed) {
                this.views.dsFacilitySelectionView.close();
            }

            this.changeMainPage();
            this.showPage('dsFacilitySelectionView', {title: 'Book a New Appointment', instructions: 'Your facilities have been found', collection: this.collections.PACTs});
        },

        showDirectSchedulingViewApptFacility: function () {
            var _that = this;

            this.changeMainPage();

            this.collections.nonPACTFacilities.fetch().done(function () {
                _that.collections.nonPACTFacilities.removeDuplicates(_that.collections.PACTs);
                if (_that.collections.PACTs.length || _that.collections.nonPACTFacilities.length) {
                    _that.showPage('dsViewApptFacilitySelectionView', {title: 'View or Cancel Appointments', instructions: 'Select a facility to see appointments'});

                    _that.views.dsPACTFacilityListView.collection = _that.collections.PACTs;
                    _that.views.dsViewApptFacilitySelectionView.PACTFacilities.show(_that.views.dsPACTFacilityListView);

                    _that.views.dsNonPACTFacilityListView.collection = _that.collections.nonPACTFacilities;
                    _that.views.dsViewApptFacilitySelectionView.nonPACTFacilities.show(_that.views.dsNonPACTFacilityListView);
                } else {
                    _that.showPage('dsNoFacilities', {title: 'No Facilities Found', instructions: ''});
                }
            });
        },

        showDirectSchedulingAppointments: function (facility) {
            if (facility) {
                this.models.calendarData.updateFacility(facility);
                this.models.calendarData.facility = facility;
            } else {
                facility = this.models.calendarData.facility
            }

            var that = this;
            this.showPage('dsAppointmentListing', {title: 'Booked VA Appointments', model: facility});
            (DSResources.deferred.readyState == 1) ? DSResources.deferred.done(function () {
                that.fetchBookedAppointments(facility)
            }) : this.fetchBookedAppointments(facility);
        },

        showDirectSchedulingClinicDate: function (facility) {
            var that = this;

            this.changeMainPage();

            if (facility) {
                this.models.calendarData.updateFacility(facility);
                this.models.calendarData.facility = facility;
            }

            that.views.mainView.content.close();

            this.collections.providerClinics.reset();

            DSResources.deferred.done(function () {
                that.showDirectSchedulingClinicsList();
                that.retrieveClinics(facility.get('clinicList'));
            });
        },

        showDirectSchedulingClinicsList: function (showView) {
            this.views.dsClinicSelectionView.showView = showView;
            this.showPage('dsClinicSelectionView', {
                title: 'Schedule My Appointment',
                instructions: showView ? 'Choose a Primary Care Clinic' : 'Loading Primary Care Clinics'
            });
            this.views.mainView.subHeader.show(this.views.dsClinicFacilityInfoView);
            $(window).resize();
        },

        showDirectSchedulingCalendarMonth: function () {
            var that = this,
                startDate = moment().format('MM/DD/YYYY');

            this.changeMainPage();
            this.showHeader('Schedule My Appointment');
            this.showInstructionalHeader({instructions: 'Loading available appointments'});
            that.views.mainView.content.close();

            this.onSuccess = function () {
                that.collections.bookedAppointmentsSchedule.reset(that.collections.bookedAppointmentsSchedule.noCancelledAppts());
                that.models.calendarData.addSlots(that.collections.bookedAppointmentsSchedule.toJSON());
                that.views.mainView.content.show(that.views.dsCalendarMonthView);
                that.views.mainView.infoBox.show(that.views.dsCalendarLegendView.updateTemplate());
                that.views.mainView.subHeader.show(that.views.dsCalendarMonthNavView);
                that.views.mainView.facilityProviderInfo.show(that.views.dsFacilityProviderInfoView);
                that.showInstructionalHeader({instructions: 'Select a day with available appointments'});
                $window.resize();
            };

            var clinicList = this.models.calendarData.facility.get('clinicList'),
                clinicIds = [];

            _.each(clinicList, function (clinic) {
                clinicIds.push(clinic.clinicId);
            });

            that.collections.bookedAppointmentsSchedule.fetch({
                url: DSResources.at(0).get("ds-booked-appointments").href + "?startDate=" + startDate + "&clinicIds=" + clinicIds.join(',')
            }).done(function () {
                if (that.onSuccess && $.isFunction(that.onSuccess)) {
                    that.onSuccess();
                }
            });
        },
        showDirectSchedulingCalendarDay: function () {
            this.showInstructionalHeader({instructions: 'Select an appointment time'});
            this.views.mainView.subHeader.show(this.views.dsCalendarDayNavView);
            this.views.mainView.content.show(this.views.dsCalendarDayView.refreshSlots());
            this.views.mainView.infoBox.show(this.views.dsCalendarLegendView.updateTemplate(true));
            this.views.mainView.facilityProviderInfo.show(this.views.dsFacilityProviderInfoView);
            $window.resize();
        },
        showDirectSchedulingCancelAppt: function (appointment, facility) {
            var that = this;
            this.views.dsCancelAppt.appt = appointment;
            this.views.dsCancelAppt.facility = facility;
            this.showPage('dsCancelAppt', {
                title: 'Cancel Appointment',
                instructions: 'Select a reason for cancelling the appointment'
            });

            if (!this.models.dsCancelReasons.fetched) {
                this.models.dsCancelReasons.fetch({
                    url: DSResources.at(0).get("ds-cancel-reasons-list").href
                }).always(function () {
                    that.models.dsCancelReasons.fetched = true;
                });
            }
        },
        showDirectSchedulingCancellationConfirmation: function (appt, cancellationData, facility) {
            $.mobile.changePage("#main", {changeHash: false});
            this.views.dsCancelApptConfirm.model = cancellationData.set($.extend(true, cancellationData.toJSON(), appt.toJSON(), {facility: facility.toJSON()}));
            this.showPage('dsCancelApptConfirm', {title: 'Cancellation Confirmed', instructions: 'This appointment has been cancelled.  All other appointments remain active.'});
            this.views.mainView.subHeader.close();
        },
        showDirectSchedulingConfirmBookingAppt: function (apptDetails) {
            var that = this;

            apptDetails.url = DSResources.at(0).get("ds-booked-appointments").href;

            this.models.calendarData.set('alreadyFetched', false);

            vent.noErrorPopup = true;

            apptDetails.save({}, {
                statusCode: {
                    200: function () {
                        that.showHeader('Booking Confirmation');
                        that.showInstructionalHeader({instructions: 'Your appointment is booked'});
                        that.views.dsConfirmBookingApptView.model = apptDetails;
                        that.views.mainView.content.show(that.views.dsConfirmBookingApptView);
                        that.views.mainView.subHeader.close();
                        $window.resize();
                    },
                    500: function () {
                        that.showPopup('errorBookingApptView', 'Unable to Book');
                    }
                }
            });
        },
        showDirectSchedulingTimeSelection: function (slot) {
            this.views.timeSlotSelectedView.setSlot(slot, this.models.calendarData);
            this.showPage('timeSlotSelectedView', {
                title: 'Schedule My Appointment',
                instructions: 'Select Book to confirm appointment'
            });
            this.views.mainView.infoBox.close();
            this.views.mainView.facilityProviderInfo.reset();
            $window.resize();
        },
        showRequest: function (request) {
            return vent.trigger("show:request:listing");

            if (_.isString(request)) {
                return this.fetchRequestByUniqueId(request);
            }
            return this.showRequestDetail(request);
        },
        showRequestDetail: function (request) {
            this.changeMainPage();
            this.setRequestDetailModel(request);

            this.showPage('appointmentRequestDetailsView', {title: 'Details'});

            this.views.mainView.subHeader.show(this.views.appointmentRequestDetailsHeaderView);
            this.views.appointmentRequestDetailsView.requestStatus.show(
                this.views.appointmentRequestDetailStatusView
            );
            this.views.appointmentRequestDetailsView.messages.show(
                new AppointmentRequestDetailsMessagesView({model: request, collection: this.collections.appointmentRequestMessages}) /*this.views.appointmentRequestDetailsMessagesView*/
            );
            this.views.appointmentRequestDetailsView.requestDetails.show(
                this.views.appointmentRequestDetailsDetailsView
            );
            this.collections.appointmentRequestMessages.fetch({
                url: request.get("appointment-request-messages").href
            });
            $window.resize();
        },
        showNewRequest: function (config) {
            config = config || {};
            this.popups.requestNoticeDialogView.config = config;
            this.showNewRequestForm('newRequestDialogView', config);
        },
        showNewRequestByPhone: function (config) {
            config = config || {};
            this.popups.requestNoticeDialogView.config = config;
            this.showNewRequestForm('newRequestByPhone', $.extend(config, {byPhone: true}));
        },
        showNewRequestForm: function (view, config) {
            var that = this;
            config = config || {byPhone: false};
            if (!config.doNotShowAgain && this.popups.requestNoticeDialogView.showNotice(config)) {
                that.showPopup('requestNoticeDialogView', 'Notice');
            }
            else {
                this.collections.patientLinks.deferred.done(function () {
                    that.models.patientIdentifiers.deferred.done(function () {
                        that.collections.appointmentRequests.deferred.done(function () {
                            if (that.models.patientIdentifiers.hasDfnOrIcn()) {
                                that.views[view].requestMetaData = RequestMetaData.toJSON();
                                that.views[view].collection = that.collections.appointmentRequests;
                                that.views[view].model = new AppointmentRequest();

                                that.showPage(view, {
                                    title: 'New Request'
                                });

                                var facilitySelectionList = new FacilitySelectionList({
                                    collection: new Facilities(),
                                    model: that.views[view].model,
                                    getFacilityLink: that.models.varResources.get('var-facilities').get('href')
                                });
                                that.views[view].facilitiesContainer.show(facilitySelectionList);
                            } else {
                                that.showPopup('unrecognizedNoticeDialogView', 'Unrecognized User');
                            }
                        });
                    });
                });
            }
        },
        showRequestSubmitted: function (model) {
            this.views.appointmentRequestSubmittedView.model = model;
            this.views.appointmentRequestSubmittedView.prefs = this.models.notificationPreferences;
            this.showPage('appointmentRequestSubmittedView', {
                title: "Request Submitted",
                instructions: "Your appointment request has been submitted"
            });
        },
        showUnfamiliarFacility: function () {
            this.showPopup('unfamiliarFacilityDialogView', 'Unfamiliar Facility', {showCloseBtn: false});
        },
        showFeedback: function () {
            this.showPopup('feedbackDialogView', 'Feedback');
        },
        showAbout: function () {
            this.showPopup('aboutDialogView', 'About');
        },
        showPopup: function (view, title, options) {

            options = $.extend({ showCloseBtn: true }, options, {headerTitle: title});
            this.popups.popupHeaderView.model.set(options);
            this.views.mainView.popupHeader.show(this.popups.popupHeaderView);
            this.views.mainView.popupContent.show(this.popups[view]);
            $('#non-popup').hide();
            $(appRouter.views.mainView.popupContent.el).addClass('popup-content-active');
            this.views.mainView.popupFooter.show(this.views.footerView);
            $('#popup').show();
            $(window).resize();
        },
        showEula: function () {
            this.views.mainView.header.show(this.views.eulaHeaderView);
            this.views.mainView.footer.show(this.views.eulaFooterView);
            this.views.mainView.content.show(this.views.eulaView);
        },
        regions: function (action) {
            $(this.views.mainView.regions.nonPopup)[action]();
        },
        showRequestCancel: function (request) {
            if (!request) {
                return vent.trigger('show:request:listing');
            }
            this.popups.confirmCancellationAlertDialogView.model = request;
            this.showPopup('confirmCancellationAlertDialogView', 'Confirm Cancel');
        },
        showSessionWarning: function () {
            this.showPopup('sessionWarningView', 'Session Warning');
        },
        showSessionExpired: function () {
            this.views.mainView.infoBox.close();
            this.views.footerView.options.showUser = false;
            this.models.user.clear();
            this.showPopup('sessionExpiredView', 'Login Session Ended');
        },
        cancelMyAppointment: function (appt, cancelDetails, facility) {
            var that = this,
                cancelAppt = new Backbone.Model({
                    clinicId: appt.get("clinicId"),
                    appointmentTime: appt.getFormattedStartDate(),
                    cancelReason: cancelDetails.get("reason"),
                    remarks: cancelDetails.get("notes")
                });

            vent.noErrorPopup = true;

            cancelAppt.save({}, {
                url: appt.get("cancel-appointment").href,
                method: "PUT",
                statusCode: {
                    200: function () {
                        cancelDetails.set("reason", _.find(that.models.dsCancelReasons.get("cancelReasonsList"), function (reason) {
                            return reason.number == cancelDetails.get("reason");
                        }).text);
                        that.collections.bookedAppointments.remove(appt);
                        vent.trigger("show:direct-scheduling:cancellation", appt, cancelDetails, facility);
                    },
                    500: function () {
                        that.showPopup('errorCancelingApptView', 'Unable to Cancel');
                    }
                }
            });
        },
        showPACTList: function (facility) {
            var teamList = facility.get('primaryProviderList');

            teamList = teamList.concat(facility.get('teamList'));

            this.popups.PACTListView.model = facility;
            this.showPopup('PACTListView', 'Members of PACT');
            this.popups.PACTListView.collection.set(teamList);
        },
        getDirectSchedulingResourcesDirectory: function (facility) {
            var facility = facility || this.models.calendarData.facility;

            var that = this,
                facilityId = facility.get('facilityId'),
                model = new Backbone.Model({facilityId: facilityId}),
                accessed;

            if (DSResources.models.length) {
                accessed = this.collections.facilityResources.findWhere(model.toJSON());
            }

            if (accessed) {
                DSResources.at(0).set(accessed.get('resources'));
            }
            else {
                DSResources.deferred = DSResources.fetch({
                    url: _.find(this.models.varResources.models, function (item) {
                        return item.get("title") == "directscheduling-resource-directory";
                    }).get("href") + "?siteCode=" + facilityId
                })
                    .done(function () {
                        model.set('resources', DSResources.at(0).toJSON());
                        that.collections.facilityResources.add(model);
                    });
            }
        },
        fetchBookedAppointments: function (facility) {
            var that = this,
                clinicList,
                clinicIds = [];

            if (facility) {
                clinicList = facility.get('clinicList');
            }
            else if (this.models.calendarData.facility) {
                clinicList = this.models.calendarData.facility.get('clinicList');
            }

            _.each(clinicList, function (clinic) {
                clinicIds.push(clinic.clinicId);
            });

            this.collections.bookedAppointments.fetch({
                url: DSResources.at(0).get("ds-booked-appointments").href + "?startDate=" + moment().format('MM/DD/YYYY HH:mm:ss') + "&clinicIds=" + clinicIds.join(',')
            }).done(function () {
                that.collections.bookedAppointments.reset(that.collections.bookedAppointments.noCancelledAppts());
            });
        },
        logoutMDWSConnection: function () {
            //loop this.collections.facilityResources
            var that = this;
            _.each(this.collections.facilityResources.models, function (resources) {
                resources.url = resources.get('resources')['ds-logout'].href;
                resources.fetch();
            });
            this.collections.facilityResources.reset();
        },
        logout: function (e) {
            e.preventDefault();
            if (typeof this.models.mhpUser !== "undefined") {
                var logoutUrl = Resources.get('logout').get('href');

                this.logoutMDWSConnection();

                $.ajax({
                    url: Resources.get('token').get('href'),
                    type: 'DELETE',
                    async: false,
                    success: function () {
                        sessionStorage.clear();
                    }
                });

                window.location = logoutUrl;
            } else {
                window.location = this.utilities.getReferencePath() + "../oauthlogin";
            }
        },
        launchpad: function () {
            window.location = "../../../launchpad";
        },
        login: function () {
            window.location = this.utilities.getReferencePath() + '../MobileHealthPlatformWeb/oauthlogin?redirect_uri=' + this.utilities.getReferencePathWithHash();
        },
        sessionRefresh: function (preventRouting) {
            var hash, route;
            $.ajax({
                url: this.models.user.url,
                success: function () {
                    if (!preventRouting) {
                        hash = Backbone.history.getHash();
                        if (hash === "" || hash === "landing") {
                            return vent.trigger("show:landing");
                        }

                        route = appRouter[appRouter.routes[hash]];
                        route ? route() : vent.trigger("show:landing");
                    }
                }
            });
        },
        fetchRequestByUniqueId: function (request) {
            this.reqId = request.replace(":", "");
            if (this.collections.appointmentRequests.routerFetched) {
                this.filterRequestByUniqueIdToDisplay(this.reqId);
            }
            else {
                this.collections.appointmentRequests.on("sync", this.filterRequestByUniqueIdToDisplay);
            }
        },
        filterRequestByUniqueIdToDisplay: function (reqId) {
            var reqId = _.isString(reqId) ? reqId : this.reqId,
                request;
            delete this.reqId;

            this.collections.appointmentRequests.off("sync", this.filterRequestByUniqueIdToDisplay);
            this.collections.appointmentRequests.routerFetched = true;

            request = _.find(this.collections.appointmentRequests.models, function (model) {
                return model.get("dataIdentifier").uniqueId === reqId;
            });
            if (!request) {
                return vent.trigger("show:request:listing");
            }
            this.showRequestDetail(request);
        },
        retrieveClinics: function (clinicsList) {
            var that = this,
                clinicIds = [];

            _.each(clinicsList, function (clinic) {
                clinicIds.push(clinic.clinicId);
            });

            this.collections.providerClinics.facility = that.models.calendarData.facility;

            this.collections.providerClinics.fetch({
                type: 'GET',
                url: DSResources.at(0).get('ds-appointment-slots').href + '?startDate=' + moment().format('MM/DD/YYYY') + '&endDate=' + moment().add(90, 'days').format('MM/DD/YYYY') + "&clinicIds=" + clinicIds.join(","),
                success: function () {
                    that.showInstructionalHeader({instructions: "Choose a Primary Care Clinic"});
                }
            });
        },
        setRequestDetailModel: function (request) {
            this.views.appointmentRequestDetailsHeaderView.model = request;
            this.views.appointmentRequestDetailStatusView.model = request;
            this.views.appointmentRequestDetailsDetailsView.model = request;
        },
        sendAppointmentMessage: function (config) {
            var that = this,
                message = config.message,
                request = config.request,
                messageModel = {};

            messageModel.messageText = message;
            messageModel.AppointmentRequestId = request.get("dataIdentifier").uniqueId;
            messageModel.senderId = User.get("id");

            new AppointmentRequestMessage().save(messageModel, {
                url: request.get("appointment-request-messages").href,
                success: function (resp) {
                    that.collections.appointmentRequestMessages.fetch({
                        url: request.get("appointment-request-messages").href
                    });
                },
                error: function (resp, error) {
                    if (navigator.onLine) {
                        vent.trigger("show:errorpopup", "Request Message was not sent.");
                    }
                },
                statusCode: {
                    302: function () {
                        loginRedirect();
                    }
                }
            });
        },
        listenForValidation: function () {
            var that = this;
            this.views.dsFacilitySelectionView.on('before:render', function (view) {
                that.showDirectSchedulingLanding({path: view.path});
            });
        },
        showHeader: function (title) {
            this.views.mainView.header.show(this.views.headerView);
            this.models.headerModel.set("headerTitle", title);
        },
        showInstructionalHeader: function (config) {
            config = config || {};
            this.models.instructionModel.set("instructions", config.instructions);
            this.views.mainView.instructionalHeader[config.instructions ? "show" : "close"](this.views.instructionalHeader);
            this.models.instructionModel.set("instructions", "", {silent: true});
        },
        showPage: function (view, config) {
            config = config || {};

            this.showHeader(config.title);
            this.showInstructionalHeader(config);
            this.views.mainView.subHeader.close();
            if (view) {
                if (config.model) {
                    this.views[view].model = config.model;
                }
                if (config.collection) {
                    this.views[view].collection = config.collection;
                }
                this.views.mainView.content.show(this.views[view]);
                this.views.mainView.subHeader.show(this.views.dsBackBtnNavView.setCallBack(this.views[view].goBack, this.views[view]));
            }
            vent.trigger('close:popup');
        },
        changePage: function (view, preventRender, config) {
            config = $.extend(true, {transition: 'pop', changeHash: false}, config);

            if (!preventRender) {
                view.render();
            }

            $.mobile.changePage("#" + view.attributes.id, config);
            $("#" + view.attributes.id).animationComplete(function () {
                $('.scrollable').scrollTop(0);
            });
        },
        changeMainPage: function () {
            $.mobile.changePage('#main', {changeHash: false});

            $('.content').removeClass('cal-content');
            this.views.mainView.facilityProviderInfo.reset();
            this.views.mainView.subHeader.close();
            this.views.mainView.infoBox.close();

            this.onSuccess = undefined;

            $('#main').animationComplete(function () {
                $window.resize();
            });
        },
        showVeteranEmailForm: function (apptDetails, path) {
            this.views.dsVeteranEmailView.model = apptDetails.set({path: path});
            this.changeMainPage();
            this.showPage('dsVeteranEmailView', {
                title: 'Email Confirmation',
                instructions: 'Please provide a valid email to send the confirmation'
            });
        },
        showEmailConfirmation: function (apptDetails, path) {
            var that = this,
                linkName = path === "book" ? "booking-confirmation" : "cancel-confirmation";

            $.ajax({
                url: DSResources.at(0).get(linkName).href,
                type: "PUT",
                data: JSON.stringify(apptDetails.toJSON()),
                contentType: "application/json",
                success: function () {
                    that.changeMainPage();
                    that.showPage('emailConfirmationView', {
                        title: 'Email Confirmation',
                        instructions: 'Your email confirmation has been sent'
                    });
                },
                error: function () {
                    that.changeMainPage();
                    that.showPage('emailConfirmationView', {
                        title: 'Email Confirmation',
                        instructions: 'There was an error sending your email'
                    });
                }
            });
        },
        showBookingLandingPageView: function () {
            this.showPage('bookingLandingPageView', {
                title: "Book New Appointments",
                instructions: "There are 3 booking options"
            })
        },
        showRequestNoticeDialog: function (options) {
            options = options || {};
            this.popups.requestNoticeDialogView.config = options;
            this.showPopup('requestNoticeDialogView', 'Notice');
        }
    });

    var appRouter = new AppRouter();

    vent.on("change:page:main", function (hash) {
        $.mobile.changePage('#main', {changeHash: false});
        $("#main").animationComplete(function () {
            Backbone.history.start({silent: true});
            if (hash) {
                appRouter.navigate(hash, {trigger: false, replace: false})
            }
        });
    });

    vent.on("show:landing", function () {
        appRouter.navigate("", {trigger: false, replace: false});
        appRouter.showLanding();
    });

    vent.on("close:popup", function () {
        $(appRouter.views.mainView.regions.nonPopup).show();
        $('#popup').hide();
        $('#popup-close-btn').show();
        $(appRouter.views.mainView.popupContent.el).removeClass('popup-content-active');
        appRouter.views.mainView.popupFooter.close();
        appRouter.views.mainView.footer.show(appRouter.views.footerView);
        $window.resize();
    });

    vent.on("show:request:listing", function (config) {
        appRouter.navigate("requests", {trigger: false, replace: false});
        appRouter.showRequestListing(config);
    });

    vent.on("show:request:detail", function (model) {
        appRouter.navigate("request", {trigger: false, replace: false});
        appRouter.showRequestDetail(model);
    });

    vent.on("show:request:cancel", function (model) {
        appRouter.navigate('request-cancel');
        appRouter.showRequestCancel(model);
    });

    vent.on("show:request:new", function (config) {
        appRouter.navigate("new-request");
        appRouter.showNewRequest(config);
    });

    vent.on("show:request:new-phone", function (config) {
        appRouter.navigate("new-request-phone");
        appRouter.showNewRequestByPhone(config);
    });

    vent.on("show:request:submitted", function (model) {
        appRouter.navigate("request-submitted");
        appRouter.showRequestSubmitted(model);
    });

    vent.on("show:direct-scheduling:how-to-enroll", function (config) {
        appRouter.showDirectSchedulingHowToEnroll(config);
    });

    vent.on("show:profile-validation", function () {
        appRouter.navigate("profile-validation", {trigger: false, replace: true});
        appRouter.showDirectSchedulingLanding();
    });

    vent.on("show:profile-validation:not-eligible", function () {
        appRouter.navigate("enrollment-not-found", {trigger: false, replace: true});
        appRouter.showDirectSchedulingNotEligible();
    });

    vent.on("show:direct-scheduling:cancellation", function (appt, cancellationData, facility) {
        appRouter.navigate("cancellation-confirmation", {trigger: false, replace: false});
        appRouter.showDirectSchedulingCancellationConfirmation(appt, cancellationData, facility);
    });

    vent.on("show:direct-scheduling:book", function () {
        appRouter.navigate("book-appt", {trigger: false, replace: false});
        appRouter.redirectToBookValidation();
    });

    vent.on("show:direct-scheduling:view", function () {
        appRouter.navigate("view-appts", {trigger: false, replace: false});
        appRouter.redirectToViewValidation();
    });

    vent.on("show:PACT-list", function (facility) {
        appRouter.showPACTList(facility);
    });

    vent.on("show:direct-scheduling:appointment-listing", function (facility) {
        appRouter.getDirectSchedulingResourcesDirectory(facility);
        appRouter.navigate("my_appointments", {trigger: false, replace: false});
        appRouter.showDirectSchedulingAppointments(facility);
    });

    vent.on("show:direct-scheduling:cancel", function (appointment, facility) {
        appRouter.navigate("cancel-my-appointment", {trigger: false, replace: false});
        appRouter.showDirectSchedulingCancelAppt(appointment, facility);
    });

    vent.on("show:ds-clinic-date:view", function (facility) {
        appRouter.getDirectSchedulingResourcesDirectory(facility);
        appRouter.showDirectSchedulingClinicDate(facility);
    });

    vent.on("show:ds-clinics-no-fetch:view", function () {
        appRouter.changeMainPage();
        appRouter.showDirectSchedulingClinicsList(true);
    });

    vent.on("show:ds-calendar-month:view", function (alreadyFetched) {
        appRouter.showDirectSchedulingCalendarMonth(alreadyFetched);
    });

    vent.on("show:ds-calendar-day:view", function () {
        appRouter.showDirectSchedulingCalendarDay();
    });

    vent.on("show:showDirectSchedulingConfirmBookingAppt:view", function (apptDetails) {
        appRouter.showDirectSchedulingConfirmBookingAppt(apptDetails);
    });
    vent.on("show:time-selection:view", function (slot) {
        appRouter.showDirectSchedulingTimeSelection(slot);
    });

    vent.on("send:appointment:message", function (config) {
        appRouter.sendAppointmentMessage(config);
    });

    vent.on("show:errorpopup", function (message) {
        new ErrorPopupView({message: message});
    });

    vent.on("show:feedback", function () {
        appRouter.showFeedback();
    });

    vent.on("show:about", function () {
        appRouter.showAbout();
    });

    vent.on("show:unfamiliar-facility", function () {
        appRouter.showUnfamiliarFacility();
    });

    vent.on("show:session:warning", function () {
        appRouter.showSessionWarning();
    });

    vent.on("show:session:expired", function () {
        appRouter.showSessionExpired();
    });

    vent.on("fetch:appointment-requests", function () {
        appRouter.collections.appointmentRequests.fetched = false;
        appRouter.collections.appointmentRequests.fetch({cache: false});
    });

    vent.on("set:new-request:provider", function (provider) {
        if (/^false$/i.test(appRouter.views.newRequestDialogView.isClosed)) {
            appRouter.views.newRequestDialogView.setProvider(provider);
        } else {
            appRouter.views.newRequestByPhone.setProvider(provider);
        }
    });

    vent.on("set:new-request:facility", function (facility) {
        if (/^false$/i.test(appRouter.views.newRequestDialogView.isClosed)) {
            appRouter.views.newRequestDialogView.setFacility(facility);
        } else {
            appRouter.views.newRequestByPhone.setFacility(facility);
        }
    });

    vent.on("set:direct-scheduling:cancellation-appointment", function (appt) {
        appRouter.views.dsCancelAppt.appointment = appt;
    });

    vent.on("set:direct-scheduling:facility", function (facility) {
        appRouter.views.dsFacilitySelectionView.facility = facility;
        appRouter.views.dsViewApptFacilitySelectionView.facility = facility;
        appRouter.views.dsCancelAppt.facility = facility;
    });

    vent.on("set:direct-scheduling:clinic", function (clinic) {
        appRouter.models.calendarData.set(clinic.toJSON());
        // appRouter.views.dsClinicSelectionView.clinic = clinic;
    });

    vent.on("update:instructional-header", function (message) {
        appRouter.showInstructionalHeader({instructions: message});
    });

    vent.on("do:logout:MDWS", function () {
        appRouter.logoutMDWSConnection();
    });

    vent.on("do:logout", function (e) {
        appRouter.logout(e);
    });

    vent.on("do:launchpad", function () {
        appRouter.launchpad();
    });

    vent.on("do:login", function () {
        appRouter.login();
    });

    vent.on("do:session:refresh", function (preventRouting) {
        appRouter.sessionRefresh(preventRouting);
    });

    vent.on("do:cancel:my-appointment", function (appt, cancelDetails, facility) {
        cancelDetails.set("appointmentId", appt.get("appointmentId"));
        cancelDetails.set("clinicId", "");
        appRouter.cancelMyAppointment(appt, cancelDetails, facility);
    });

    vent.on("resize:panes", function () {
        appRouter.utilities.resizePanes();
    });

    vent.on("show:veteran-email-form", function (apptDetails, path) {
        appRouter.showVeteranEmailForm(apptDetails, path);
    });
    vent.on("show:email-confirmation", function (apptDetails, path) {
        appRouter.showEmailConfirmation(apptDetails, path);
    });

    vent.on("set:ds-calendar-data", function (calendarData) {
        appRouter.models.calendarData.set(calendarData.toJSON());
    });

    vent.on("show:bookingLandingPage", function () {
        appRouter.navigate("scheduling", {trigger: false, replace: false});
        appRouter.showBookingLandingPageView();
    });

    vent.on("show:requestNoticeDialog", function (options) {
        appRouter.showRequestNoticeDialog(options);
    });

    vent.on("enable:view-button", function () {
        appRouter.views.dsViewApptFacilitySelectionView.enableButton();
    });

    Backbone.history.start();
    return appRouter;
});
