define([
	'jquery',
	'Underscore',
	'Vent',
	'text!../html/internetConnectionError.html',
	'JQM',
	'JQMConfig',
	'text',
	'Backbone',
	'Marionette',
	'Moment',
	'FormParams'
],
    function ($, _, vent, template) {

        //Global should not have any dependencies
        var utilities = {};

		if (!window.location.origin) {
			window.location.origin = window.location.protocol + "//" + window.location.hostname;
			if (window.location.port) {
				window.location.origin = window.location.origin + ':' + window.location.port;
			}
		}
        utilities.internetConnectionCheck = function(){
            var registerInternetDisconnectHandler = function() {
                $(document).ajaxSend(function(jqxhr, settings, exception) {
                    if (!navigator.onLine) {
                        vent.trigger('show:errorpopup', 'Your internet appears to be unavailable. Please try again when you regain connectivity.');
                    }
                });
            };
            registerInternetDisconnectHandler();
            
        };
        utilities.flattenLinks = function (object) {
			"use strict";
            var links = ( "get" in object ) ? object.get('link') : null;
            if (Array.isArray(links)) {
                for (var i = 0, size = links.length; i < size; i++) {
					object.set(links[i].title ? links[i].title : 'self', links[i]);
                }
                object.unset('link');
            }
            else if(_.isObject(object)){
                links = object.link || [];
                _.each(links, function(link){
                    object[link.title || "self"] = link;
                });

            }
        };

        utilities.globalInit = function (app) {
			"use strict";
            if (app.user) {
                utilities.resizePanes();
                $(window).resize(utilities.resizePanes);
            }

            $('body').on('pageshow', function (event, ui) {
                var prevPage = $(ui.prevPage);
                if (prevPage.hasClass('ui-dialog')) {
                    prevPage.remove();
                }

                utilities.resizePanes(app.customHeight || 0);
                utilities.registerLoadingSpinner(app.spinnerOptions || 0);
            });
        };

        utilities.clearLeftPanel = function () {
            "use strict";
            var $leftPanel = $('.ui-panel-position-left.ui-panel-open');

            if ($leftPanel.length) {
                $('#content-secondary').toggleClass('hidden');
                $('#content-primary').toggleClass('no-float');
                $leftPanel.panel('toggle');
            }
        };

        utilities.slidePrimary = function (selector) {
            var $moveContent = $(selector || '#content-primary');
            if ($moveContent.hasClass('closed')) {
                $moveContent.removeAttr('style').toggleClass('closed');
            } else {
                $moveContent.animate({
                    left: '0'
                }, 300, function () {
                    $moveContent.addClass('closed');
                });
            }
        };

        utilities.resizePanes = function () {
            var headerCompleteHeight = $('header').outerHeight()+1,
                headerTopLevelHeight = $('#header').outerHeight()+2,
	            $footer = $('footer'),
                footerHeight = $footer.is(':visible') ? $footer.outerHeight()+1 : 0,
                theContainer = $(window);

            // only need to initialize jquery objects once, once found;
            utilities.$mainTag  = utilities.$mainTag ? utilities.$mainTag : $('main');
            utilities.$mainPage = utilities.$mainPage || $("#main");
            utilities.$mainId   = utilities.$mainId ? utilities.$mainId : $('#main-content');
            utilities.$menuPanel = utilities.$menuPanel ? utilities.$menuPanel : $('.menu-panel');

            utilities.$mainTag.css({
                'top': headerCompleteHeight + 'px',
                'bottom': footerHeight + 'px'
            });

            utilities.$menuPanel.css({
                'top': headerTopLevelHeight + 'px',
                'bottom': '0'
            });

            //Issue with AppleWebKit 536.26 - need to use document instead of window.
            if(navigator.userAgent.indexOf('AppleWebKit/536.26') > -1 ){
                theContainer = $(document);
            }

            utilities.$mainId.css({
                'height': ( theContainer.height() - headerCompleteHeight - footerHeight ) + "px"
            });

            utilities.resizeUserMenu();
        };

        utilities.resizeUserMenu = function(){
            utilities.$menuPanel = utilities.$menuPanel ? utilities.$menuPanel : $('.menu-panel');
            utilities.$userMenuTop = utilities.$userMenuTop || $('#user-menu .panel-upper-section');
            utilities.$userMenuHeading = utilities.$userMenuHeading || $('#user-menu .panel-upper-section .menu-header');
            utilities.$userMenuTopScroll = utilities.$userMenuTopScroll || $('#user-menu .panel-upper-section .scrollable');
            utilities.$userMenuBottom = utilities.$userMenuBottom || $('#user-menu .panel-lower-section');
            utilities.$userMenuTopScroll.css({
                'height': ( utilities.$menuPanel.height() - utilities.$userMenuBottom.height() - utilities.$userMenuHeading.outerHeight() ) + 5 + "px"
            });
        };

        utilities.encrypt = function (name) {
            return name.replace(/[a-z0-9]/gi, function (a) { return a.charCodeAt(0) - 11 + '|'; }).slice(0, -1);
        };

        utilities.removeFromStorage = function (key) {
			"use strict";
            if (key) {
                try {
                    window.localStorage.removeItem(key);
                } catch (e) {
                    if (e == QUOTA_EXCEEDED_ERR) {
                        var message = 'Local-Storage Quota exceeded!';
                        if (window.console && window.console.log) {
                            console.log(message);
                        } else {
                            alert(message);
                        }
                    }
                }
            }
        };

        utilities.writeToStorage = function (key, value) {
            if (!value) {
                value = null;
            }

            try {
                window.localStorage[key] = JSON.stringify(value);
            } catch (e) {
                if (e == QUOTA_EXCEEDED_ERR) {
                    var message = 'Local-Storage Quota exceeded!';
                    if (window.console && window.console.log) {
                        console.log(message);
                    } else {
                        alert(message);
                    }
                }
            }
        };

        utilities.getReferencePath = function () {
	        var referencePath = window.location.protocol + "//" + window.location.host + window.location.pathname;
	        var index = referencePath.indexOf("index.html");
	        if (index !== -1) {
		        referencePath = referencePath.substring(0, index);
	        }
	        return referencePath;
        };

        //Get the path to MobileHealthPlatformWeb, which may not be the ReferencePath for other independent apps (Wellness, Immunizations, etc)
        utilities.getMhpReferencePath = function () {
            return window.location.origin + "/MobileHealthPlatformWeb";
        };

        utilities.getReferencePathWithHash = function () {
            return encodeURI(utilities.getReferencePath() + window.location.hash);
        };

        utilities.deleteCookie = function (cookieName) {
            document.cookie = encodeURIComponent(cookieName) + "=deleted; expires=" + new Date(0).toUTCString();
        };

        utilities.cleanUpSession = function () {
            window.sessionStorage['token'] = null;
            $.ajaxPrefilter(function (options, originalOptions, jqXHR) {
                jqXHR.setRequestHeader("Authorization", "");
            });
            utilities.deleteCookie('JSESSIONID');
        };

        utilities.findMessage = function (errMsg) {
            var lines = errMsg.split('\n');

            for (var i = 0; i < lines.length; i++) {
                if (lines[i].indexOf('Message:') >= 0) {
                    return lines[i];
                }
            }
            return errMsg;
        };

        utilities.parseTokenAndRegisterOauthHandler = function () {
            var params = {}, queryString = location.search.substring(1),
                regex = /([^&=]+)=([^&]*)/g, m;
            while (m = regex.exec(queryString)) {
                params[decodeURIComponent(m[1])] = decodeURIComponent(m[2]);
            }

            var token = params['token'];

            if (typeof token !== 'undefined' && token !== 'undefined' && token !== null && token !== 'null') {
                window.sessionStorage['token'] = JSON.stringify(token);
            }

            var storedToken = window.sessionStorage['token'];

            if (typeof storedToken !== 'undefined' && storedToken !== 'undefined' && storedToken !== null && storedToken !== 'null') {
                $.ajaxPrefilter(function (options, originalOptions, jqXHR) {
                    var currentToken = JSON.parse(storedToken);
                    jqXHR.setRequestHeader("Authorization", "Bearer " + currentToken);
                });
            } else {
                utilities.cleanUpSession();
            }

	        $(document).ajaxError(function(e, jqxhr, settings, exception) {
		        var noErrorPopup = vent.noErrorPopup;

		        vent.noErrorPopup = false;

                if (!navigator.onLine) {
                    vent.trigger('show:errorpopup', 'Your internet appears to be unavailable. Please try again when you regain connectivity.');
                } else if ( jqxhr.status == 0 || jqxhr.status == 200 || jqxhr.status == 400 || jqxhr.status == 404) {

		        } else if (jqxhr.status == 204) {
                    $.mobile.loading('hide');
                    $('.ui-loader').removeAttr('style');
                    $('#dialog-screen').addClass('hidden');

                } else if (jqxhr.status == 403) {
                    if (!$('#roa').length) {
                        location.reload();
                    }
		        } else if (jqxhr.status == 401) {
			        if (jqxhr.responseText.indexOf('invalid_token') > 0) {
				        utilities.cleanUpSession();
				        $.ajax(settings);
			        }
		        } else if (jqxhr.status == 302 ){
			        utilities.loginRedirect();
		        } else if (!noErrorPopup) {
                    vent.trigger('show:errorpopup', 'Action could not be completed at this time due to an error.');
		        }

	        });

            $.ajaxSetup({
                cache:false
            });

        };
        utilities.parseTokenAndRegisterOauthHandler();

        utilities.registerLoadingSpinner = function (options) {
            $(document)
                .ajaxStart(function (event, xhr, settings) {
                    $.mobile.loading('show', options || {theme:'a', text: 'Loading', textVisible: true});
                    $('.ui-loader').css('display', 'block');
                    $('#dialog-screen').removeClass('hidden');
                })
                .ajaxStop(function () {
                    $.mobile.loading('hide');
                    $('.ui-loader').removeAttr('style');
                    $('#dialog-screen').addClass('hidden');
                })
                .ajaxError(function () {
                    $.mobile.loading('hide');
                    $('.ui-loader').removeAttr('style');
                    $('#dialog-screen').addClass('hidden');
                });
        };

	    utilities.registerLabelFocus = function () {
		    $('body')
			    .on('focus', 'input[type="radio"], input[type="checkbox"]', function (e) {
				    document.querySelector('[for="'+ e.currentTarget.id + '"]').classList["add"]('focus-label');
			    })
			    .on('blur', 'input[type="radio"], input[type="checkbox"]', function (e) {
				    document.querySelector('[for="'+ e.currentTarget.id + '"]').classList["remove"]('focus-label');
			    });
	    };

        utilities.create = function (page) {
            try {
                page.page("destroy");
            } catch (err) {
                // if page has not been initialized yet, destroy is not necessary
            }
            page.page();
        };

	    utilities.loginRedirect = function () {
		    window.location = '../login';
	    };

	    utilities.browserAllowed = function () {
		    if (!Modernizr.audio && navigator.userAgent.indexOf('PhantomJS') < 0) {
			    $('body').css({
				    'background-color': 'transparent',
				    'background-image': 'url("../app/img/bg_app_main.jpg")',
				    'background-position': 'center center',
				    'background-size': '100%'
			    });
			    alert('Your browser does not support this application. Please upgrade your browser or device.\n\n');
			    window.location = '../newbrowser.html';
		    }
	    };

        utilities.masks = function(){
            return {
                USPhone: '(000) 000-0000'
            }
        };

        utilities.regexes = function(){
            return {
                USPhone: /^\(\d{3}\)\s{1}\d{3}-\d{4}$/,
                StdDate: /^\d{2}\/\d{2}\/\d{4}$/,
                email: /^[A-z0-9._%+-]+@[A-z0-9.-]+\.[A-z]{2,3}$/g
            }
        };

        utilities.doErrorChecking = function(checks){
            function getFieldValidationError(element){
                return element.closest('.validator');
            }
            function getValidationError(element){
                return getFieldValidationError(element).find('.validation-error');
            }
            _.each(checks, function(check){
                if(check.condition){
                    getFieldValidationError(check.field).addClass('field-validation-error');
                    getValidationError(check.field).html(check.error);
                }
                else{
                    getFieldValidationError(check.field).removeClass('field-validation-error');
                    getValidationError(check.field).html('');
                }
            });
        };

        utilities.getCheckedVal = function(eles, multi){
            var vals = _.map(this.getCheckedEle(eles, multi), function(ele){
                return $(ele).val();
            });
            if(multi){
                return vals;
            }
            return vals[0];
        };

        utilities.getCheckedEle = function(eles, multi){
            var checked = _.filter(eles, function(ele){
                return $(ele).is(':checked');
            });
            if(multi){
                return checked;
            }
            return $(checked[0]);
        };

        utilities.getElementValue = function(ele, container){
            if(ele.is(":checkbox") || ele.is(":radio")){
                var vals,
                    length = ele.length;
                if(container){
                    var eles    = container.find('[name="' + ele.attr('name') + '"]'),
                    length      = eles.length,
                    multi       = length > 1;
                    return utilities.getCheckedVal(eles, multi);
                }
                vals = utilities.getCheckedVal(ele, multi) || false;
                if(length == 1 ){
                    if(_.isArray(vals) && vals.length == 0){
                        return false;
                    }
                    if(!_.isArray(vals)){
                        return vals;
                    }
                }
                return utilities.getCheckedVal(ele, multi);
            }
            return ele.val();
        };

        utilities.getValidationObject = function(ele, validationObject){
            var identifier  = ele.attr('name') || ele.attr('id'),
                obj         = validationObject[identifier],
                condition,
                doCondition,
                error;

            var doCondition = function(){
                return /(true|false)/.test(condition);
            };

            if(obj){
                if(obj.condition){
                    condition = obj.condition();

                }
                if(obj.error){
                    error = obj.error;
                }
                return {
                    condition:  condition,
                    doCondition: doCondition(),
                    error:      error,
                    field:      ele
                };
            }
            return {
                field: ele,
                doCondition: false
            };
        };

        utilities.validateElement = function(ele, validationObject){
            var validation = utilities.getValidationObject(ele, validationObject);
            return utilities.validateElementByType(ele, validation);
        };

        utilities.validateElementByType = function(ele, validationObject){
            var error = {},
                doError = function(errStr){
                    return validationObject.error || errStr;
                };

            if(ele.is(":radio")){
                error = {
                    condition: (validationObject.doCondition ? validationObject.condition : !utilities.getCheckedVal(ele)),
                    field: ele, 
                    error: doError('Please select an option.')
                };
            }
            else if(ele.is(":checkbox")){
                var name = ele.attr('name'),
                    eles = ele,
                    vals;

                if(name){
                    eles = ele.closest('.validator').find('[name="' + name + '"]');
                }
                vals = utilities.getCheckedVal(eles, true);
                error = {
                    condition: (validationObject.doCondition ? validationObject.condition : vals.length == 0),
                    field: ele,
                    error: doError((eles.length > 1 ? 'Please select 1 or more options.' : 'Please select the option.'))
                };
            }
            else if(ele.is("select")){
                error = {
                    condition: (validationObject.doCondition ? validationObject.condition : !ele.val().trim().length),
                    field: ele,
                    error: doError('Please select an option.')
                };
            }
            else{
                error = {
                    condition: (validationObject.doCondition ? validationObject.condition : !ele.val().trim().length),
                    field: ele,
                    error: doError('Please enter data in the field.')
                };
            }
            return error;
        };

        utilities.validate = function(container, customValidationConditions){
            //Validate Required Fields
            var that    = this,
                fields  = container.find('[required]:not(:input):visible'),
                eles,
                errors  = [],
                condition,
                error,
                obj;

            var getValidationObject = function(ele){
                var identifier = ele.attr('name') || ele.attr('id');
                error       = null;
                condition   = null;
                obj = customValidationConditions[identifier] || {};
                if(obj.condition){
                    condition = obj.condition();
                }
                if(obj.error){
                    error = obj.error;
                }
                return obj;
            };

            var doCondition = function(){
                return /(true|false)/.test(condition);
            };

            var doError = function(errStr){
                return error || errStr;
            };

            eles = _.filter(_.map(fields, function(field){
                field = $(field);
                if(field.is('label')){
                    return container.find('[name="' + field.attr('for') + '"]');
                }
                return field.find(':input:first');
            }), function(ele){
                return ele.length > 0;
            });

            _.each(eles, function(ele){
                ele         = $(ele);
                validation  = getValidationObject(ele);

                if(ele.is(":radio")){
                    var name = ele.attr('name'),
                        eles = ele,
                        vals;

                    if(name){
                        eles = ele.closest('.validator').find('[name="' + name + '"]');
                    }

                    errors.push({
                        condition: (doCondition() ? condition : !utilities.getCheckedVal(eles)),
                        field: ele, 
                        error: doError('Please select an option.')
                    });
                }
                else if(ele.is(":checkbox")){
                    var name = ele.attr('name'),
                        eles = ele,
                        vals;

                    if(name){
                        eles = ele.closest('.validator').find('[name="' + name + '"]');
                    }
                    vals = utilities.getCheckedVal(eles, true);
                    errors.push({
                        condition: (doCondition() ? condition : vals.length == 0),
                        field: ele,
                        error: doError((eles.length > 1 ? 'Please select 1 or more options.' : 'Please select the option.'))
                    });
                }
                else if(ele.is("select")){
                    errors.push({
                        condition: (doCondition() ? condition : !ele.val().trim().length),
                        field: ele,
                        error: doError('Please select an option.')
                    });
                }
                else{
                    errors.push({
                        condition: (doCondition() ? condition : !ele.val().trim().length),
                        field: ele,
                        error: doError('Please enter data in the field.')
                    });
                }
            });

            utilities.doErrorChecking(errors);
            return errors;
        };

	    utilities.removeToken = function () {
		    var locationArray = window.location.href.split('?token='),
			    location = locationArray[0],
			    token = locationArray[1];

		    if (token && token.length) {
			    var base = document.getElementsByTagName('head')[0].getElementsByTagName('base')[0];
			    base.setAttribute('href',base.getAttribute('href').split('?token')[0]);
			    if (history.replaceState) {
				    history.replaceState({}, '', '/veteran-appointment-requests/');
			    } else {
				    window.location.href = location;
			    }
		    }
	    };

	    utilities.getSetSeenInstructions = function (config) {
		    config = config || {};

		    var encryptedId = utilities.encrypt(config.userId),
			    itemName = window.location.pathname + "/seenInstructions/" + encryptedId,
			    action = config.action + "Item",
			    setValue = config.setValue;

		    return localStorage[action](itemName, setValue);
	    };

        utilities.eventAggregator = _.extend({}, Backbone.Events);
        utilities.eventAggregator.on("logout", function () {
            utilities.cleanUpSession();
        });

        utilities.registerLoadingSpinner();
	    utilities.registerLabelFocus();

        return {
            getUtilities: function () {
                return utilities;
            }
        };
    }
);
