define([
	'Global',
	'Vent',
	'text!../html/footer.html',
	'text!../../version.json'
], function(Global,	vent, footerTemplate, version){
	'use strict';
	
	return Backbone.Marionette.ItemView.extend({
		template: _.template(footerTemplate),
		ui:{
			version: ".username-bar-secondary",
			username: '.username-recognition'
		},
		modelEvents: {
			change: 'render'
		},
		initialize: function(config, options){
			this.options = options || {};
			var versionObj = $.parseJSON(version);
			this.build = versionObj.build;
			this.appVersion = versionObj.version;
		},
		onRender: function(){
			this.ui.version.text(this.ui.version.text() + this.appVersion);
		},
		onShow: function(){
			this.$el.parent().trigger("create");
		},
		logoutUser: function (e) {
			vent.trigger("do:logout", e);
		},
		handleKeyDown: function (e) {
			if (e.which === 13 || e.which === 32) {
				e.preventDefault();
				$(e.currentTarget).trigger('tap');
			}
		},
		showLoggedInUser: function(){
			return /^true$/i.test(this.options.showUser);
		},
        htmlDecode: function (string) {
            var e = document.createElement('div');
            e.innerHTML = string;
            return e.childNodes[0].nodeValue;
        },
		templateHelpers: function(){
			var that = this;
			return {
				showLoggedInUser: function(){
					return that.showLoggedInUser();
				},
				hasBuild: function(){
					return that.build;
				},
				getVersion: function(){
					var version = that.appVersion;
					if(that.build){
						version += "." + that.build;
					}
					return version;
				},
                getUserName: function () {
                    return that.htmlDecode(this.firstName + " " + this.lastName);
                }
			}
		}
	});
});

	