define([
	'Global',
	'Vent',
	'User',
	'MessageItemView',
	'text!../html/appointmentRequestDetailsMessages.html'

], function(Global, Vent, User, MessageItemView, appointmentRequestDetailsMessagesTemplate){
	'use strict';
	
	return Backbone.Marionette.CompositeView.extend({
		itemView: MessageItemView,
		itemViewContainer: "#convo",
		template: _.template(appointmentRequestDetailsMessagesTemplate),
		defaultPlaceholder: "Enter Text Here... (maximum of 100 characters). Messages are NOT intended to be used for Cancelling an appointment. Messages are NOT intended to be used for “clinical” questions that may need provider attention;",
		ui:{
			sendMessage: "#sendMessage",
			newMessage: "#newMessage",
			newMessageC: ".new-message"
		},
		events:{
			"click #sendMessage": "sendMessage",
			"keyup #newMessage": "displayMaxLength"
		},
		collectionEvents:{
			sync: "onShow"
		},
		modelEvents:{
			change: "render onShow"
		},
		sendMessage: function(e){
			if(!this.sendMessageAllowed){
				return;
			}
			var message = this.ui.newMessage.val().trim(),
				mlength = message.length,
				messageModel = {};
			this.disableSend(true);

			if( mlength < 0 || mlength > this.ui.newMessage.data("maxlength") ){
				return;
			}

			Vent.trigger("send:appointment:message", {message: message, request: this.model});
			this.ui.newMessage.val("");
		},
		displayMaxLength: function () {
			var val 	= this.ui.newMessage.val().trim(),
				length 	= val.length,
				max 	= this.ui.newMessage.data("maxlength");
			this.disableSend(!length ||  max < length);
			this.ui.newMessageC.attr("data-balance", (max - length) );
		},
		allowVeteranToSendMessage: function(){
			/*
				No messages allowed after:
				(a)	Appointment Date has passed
				(b)	Appointment has been cancelled
				(c) BusinessDays rule
			*/

			var request 		= this.model,
				messages 		= this.collection.models,
				vetMessages		= _.filter(this.collection.models, function(message){ return message.isVetMessage() }),
				initiating 		= false,
				vetLastMessage	= false,
				lastMessage		= null,
				today 			= new Date(),
				oneYear			= new Date().setFullYear(new Date().getFullYear() + 1),
				appointmentDate = new Date(request.getAppointmentDate() || oneYear),
				dayOfWeek 		= appointmentDate.getDay(),
				considerWeekend = dayOfWeek === 1 ? 2 : 0,
				oneBusinessDaysBefore = new Date(new Date(appointmentDate).setDate(appointmentDate.getDate() - considerWeekend)),
				twoBusinessDaysBefore = new Date(new Date(appointmentDate).setDate(appointmentDate.getDate() - considerWeekend - 1 )),
				placeholder 	= this.defaultPlaceholder,
				disabled 		= false;

				today.setHours(0);
				today.setMinutes(0);
				today.setSeconds(0);
				today.setMilliseconds(0);

			//IMPORTANT .val('') needed to reset placeholder in IE9, IE10
			// if(messages.length ){
			// 	lastMessage = this.collection.at(0);
			// 	vetLastMessage = this.vetCreatedMessage(lastMessage);
			// }
			// if(messages.length == 1 ){
			// 	initiating = this.vetCreatedMessage(lastMessage);
			// }

			if (request.isCancelled() ){
					placeholder	= 'Appointment has been cancelled';
					disabled	= true;
			}
			else if (today.getTime() > appointmentDate.getTime() ){
				placeholder 	= 'Appointment Date has passed';
				disabled		= true;
			}
			else if( lastMessage ){
				if( vetLastMessage ){
					if ( initiating && today.getTime() > twoBusinessDaysBefore.getTime() ){
						placeholder 	= 'Please Call. Two business days required to initiate a message';
						disabled		= true;
					}
					else if (!initiating && today.getTime() >= oneBusinessDaysBefore.getTime() && today.getTime() <= appointmentDate.getTime()){
						placeholder		= 'Please Call. One business day required to respond to a message';
						disabled		= true;
					}
					// else{
					// 	placeholder 	= 'Must wait for a response before sending another message.';
					// 	disabled		= true;
					// }
				}
				if( vetMessages.length >= 2 ){
					placeholder 	= "Can't send more than two messages...";
					disabled 		= true;
				}
			}
			if( vetMessages.length >= 2 ){
				placeholder 	= "Can't send more than two messages...";
				disabled 		= true;
			}

			if(request.get("hasVeteranNewMessage")){
				this.clearNewMessageFlag();
			}

			this.ui.newMessage.val("").prop({
				placeholder: placeholder,
				disabled: disabled
			});
			return !disabled;
		},
		clearNewMessageFlag: function(){
			var that = this;
			this.model.set("hasVeteranNewMessage", false);
			$.ajax({
				type: "POST",
				url: this.model.get('appointment-request-new-message-flag').href,
				data: "{}",
				contentType: "application/json; charset=utf-8",
				error: function (err) {
					that.model.set("hasVeteranNewMessage", true);
                    if (navigator.onLine) {
                        Vent.trigger("show:errorpopup", "Error clearing read messages flag for appointment request " + that.model.get('dataIdentifier').uniqueId + ". Error was " + err.message);
                    }
				}
			});

		},
		vetCreatedMessage: function(message){
			return message.get("senderId") === User.get("id");
		},
		disableSend: function(state){
			this.ui.sendMessage[0].disabled = state;
		},
		onBeforeClose: function(){
			this.ui.newMessage.prop({
				placeholder: this.defaultPlaceholder
			});
		},
		onRender: function(){
			this.delegateEvents();
		},
		onShow: function(){
			this.$el.trigger("create");
			this.disableSend(true);
			this.sendMessageAllowed = this.allowVeteranToSendMessage();
		}
	});
});