define([
	'Global',
	'Vent',
	'User',
	'AppointmentRequestsListItemView',
	'AppointmentRequestListEmptyView',
	'NotificationSettingsView',
	'text!../html/appointmentRequestList.html',
	'text!../html/statusFilter.html',
	'NoJqmCollapsible'
], function( Global, vent, User, AppointmentRequestsListItemView, AppointmentRequestListEmptyView, NotificationSettingsView, appointmentRequestListTemplate, statusFilterTemplate){
	'use strict';
	
	return Backbone.Marionette.CompositeView.extend({
		template: _.template(appointmentRequestListTemplate),
		itemViewContainer: "ul",
		itemView: AppointmentRequestsListItemView,
		emptyView: AppointmentRequestListEmptyView,
		statusFilterKey: 'status-filter',

		collectionEvents:{
			reset: "getStatuses",
			sync: "updateInstructionalHeader getStatuses render"
		},

		events: {
			'change #status-filter': 'filterList'
		},
		ui:{
			list: "#appointment-request-list",
			statusFilter: "#status-filter-container",
			requests: ".requests-collapsible",
			notifications: ".notifications-collapsible",
			notificationsSection: ".notifications-collapsible .content"
		},
		initialize: function () {
			this.bindUIElements();
			this.notificationSettingsView = new NotificationSettingsView();
		},
		onShow: function(){
			this.delegateEvents();
			if(this.rendered){
				this.updateInstructionalHeader();
			}
			this.rendered = true;
			$(window).resize();
			this.openCloseSections();
			this.setSeenRequestListInstructions();
			if(this.ui.statusFilter.find('select').val() != 'All') {
				this.ui.statusFilter.find('select').trigger('change');
			}
		},
		onRender: function () {
			$('.content').addClass('cal-content');
			this.getStatuses();
			this.notificationSettingsView.model = this.notificationSettingsView.model || this.model;
			this.ui.notificationsSection.append(this.notificationSettingsView.render().$el);
			this.ui.requests.noJqmCollapsible();
			this.ui.notifications.noJqmCollapsible();
		},
		setSeenRequestListInstructions: function () {
			var getSetSeenInstructions = Global.getUtilities().getSetSeenInstructions,
				userId = User.id,
				seenInstructions = getSetSeenInstructions({action: "get", userId: userId});

			if (!seenInstructions) {
				getSetSeenInstructions({action: "set", userId: userId, setValue: true});
			}
		},
		updateInstructionalHeader: function(){
			if(!this.collection.length){
				vent.trigger('update:instructional-header', 'You have not requested any appointments');
			}
			else{
				vent.trigger('update:instructional-header', 'Read notifications or update your notification settings');
			}
		},
		filterList: function (e) {
			var filter = e.currentTarget.querySelector(':checked').value;
			localStorage.setItem(this.statusFilterKey, filter);
			if (filter === 'All') {
				this.showAllListItems();
				return;
			}
			this.filterListOn(filter);
		},
		addFilterDropDown: function (statuses, appointmentsLength) {
			var statusesKeys, statusesKeysLength, status, i, statusCount, $statusFilter,
				statusFilterKeyValue = localStorage.getItem(this.statusFilterKey);

			this.ui.statusFilter.empty();

			statusesKeys = Object.keys(statuses);
			statusesKeysLength = statusesKeys.length;

			if (statusesKeysLength > 1) {
				$statusFilter = this.ui.statusFilter.append(statusFilterTemplate).find('#status-filter');
				$statusFilter.append('<option value="All">All ('+ appointmentsLength +')</option>');
				for (i = 0; i < statusesKeysLength; i++) {
					status = statusesKeys[i];
					statusCount = statuses[status];
					$statusFilter.append('<option value="'+status+'">'+status+' ('+statusCount+')</option>');
				}
				$statusFilter.selectmenu();
				if (statusFilterKeyValue) {
					$statusFilter.val(statusFilterKeyValue).trigger('change');
				}
			}
		},
		getStatuses: function () {
			var statuses = {},
				appointments = this.ui.list.find('[data-status]'),
				appointmentsLength = appointments.length,
				status, i;

			if (appointmentsLength === 0) {
				return;
			}

			for (i = 0; i < appointmentsLength; i++) {
				status = appointments.eq(i).data('status');
				statuses[status] = statuses[status] ? statuses[status] + 1 : 1;
			}

			this.addFilterDropDown(statuses, appointmentsLength);
		},
		filterListOn: function (filter) {
			var matches = this.ui.list.find('[data-status="' + filter + '"]');
			this.hideAllListItems();
			matches.parents('li').removeClass('hidden');
		},
		showAllListItems: function () {
			this.ui.list.children().removeClass('hidden');
		},
		hideAllListItems: function () {
			this.ui.list.children().addClass('hidden');
		},
		openCloseSections: function () {
			switch (this.openSection) {
				case "requests":
					this.ui.notifications.noJqmCollapsible().toggle();
					break;
				case "settings":
					this.ui.requests.noJqmCollapsible().toggle();
					break;
			}
		},
		goBack: function () {
			vent.trigger("show:landing");
		},
		onBeforeClose: function () {
			this.ui.statusFilter.find("select").selectedIndex = 0;
		},
		templateHelpers: function(){
			var that = this;
			return {
				getNumUnreadMessages: function(){
					var seenInstructions = (Global.getUtilities().getSetSeenInstructions({action: "get", userId: User.id}) || that.collection.length) ? 0 : 1,
						numUnreadMessages = that.collection.countUnreadMessages();

					return numUnreadMessages ? numUnreadMessages : seenInstructions;
				}
			};
		}
	});

});