define([
	'Global',
	'Vent',
	'CalendarLegendView',
	'text!../html/calendarDayNav.html'
],

	function(Global, vent, CalendarLegendView, template){
		'use strict';
		return Backbone.Marionette.ItemView.extend({
			className: "calendar-nav sub-header",
			template: _.template(template),
			events: {
				'click #cal-back-to-month': 'goBackToCalendar',
				'click #cal-prev-day': 'goToPrevDay',
                'click #cal-next-day': 'goToNextDay',
                'click #cal-legend': 'toggleLegend',
                'blur #cal-legend': 'hideLegend'
			},
			initialize: function(){
				this.minDate = this.clearTime(moment().add(1, 'days'));
    			this.maxDate = this.clearTime(moment().add(90, 'days'));
			},
			onRender: function(){
				$('.content').addClass('cal-content');
				this.delegateEvents();

                this.validatePrevDay();
                this.validateNextDay();
			},
			goBackToCalendar: function() {
				var $facilityProviderInfo = $('.facility-provider-info');

				$facilityProviderInfo.removeClass("display-none");

				this.model.set("alreadyFetched", true);
				vent.trigger('show:ds-calendar-month:view');
			},
			goToPrevDay: function(e){
                
                var $showDate = this.$el.find('h2'),
                	selectedDate = this.model.get('selectedDate'),
            		currentDate = moment(selectedDate).subtract(1, 'days').format('ddd, MMM DD, YYYY');

            	this.model.set('selectedDate', currentDate);

                this.$el.find('#cal-next-day').removeAttr('disabled');

                this.validatePrevDay();

                $showDate.text(currentDate);

            },
            validatePrevDay: function () {
                var currentDate = moment(this.model.get('selectedDate'));

                if (currentDate <= this.minDate) {
                    this.$el.find('#cal-prev-day').attr('disabled', 'disabled');
                }
            },
            goToNextDay: function(e){
            	var $showDate = this.$el.find('h2'),
            		selectedDate = this.model.get('selectedDate'),
            		currentDate = moment(selectedDate).add(1, 'days').format('ddd, MMM DD, YYYY');

            	this.model.set('selectedDate', currentDate);

                this.$el.find('#cal-prev-day').removeAttr('disabled');
    
			    this.validateNextDay();
			    
			    $showDate.text(currentDate);
            },
            validateNextDay: function () {
                var currentDate = moment(this.model.get('selectedDate'));

                if (currentDate >= this.maxDate) {
                    this.$el.find('#cal-next-day').attr('disabled', 'disabled');
                }
            },
			toggleLegend: function(){
				var $calLegendBtn = $('#cal-legend'),
					$calLegend = $('.cal-legend');

				if (!$calLegendBtn.hasClass('active')){
					$calLegendBtn.addClass('active');
					$calLegend.removeClass('slide-up').addClass('slide-down');
				} else {
					$calLegendBtn.removeClass('active');
					$calLegend.removeClass('slide-down').addClass('slide-up');
				}
			},
			hideLegend: function(){
				var $calLegendBtn = $('#cal-legend'),
					$calLegend = $('.cal-legend');
					
				$calLegendBtn.removeClass('active');
				$calLegend.removeClass('slide-down').addClass('slide-up');
			},
            clearTime: function (date) {
                date = moment(date);

                return date.hours(0).minutes(0).seconds(0).milliseconds(0);
            }
		});
	}

);