define([
	'Global',
	'Vent',
	'CalendarLegendView',
	'text!../html/calendarMonthNav.html'
],

	function(Global, Vent, CalendarLegendView, template){
		'use strict';
		return Backbone.Marionette.ItemView.extend({
			className: "calendar-nav sub-header",
			template: _.template(template),
			events: {
				'click #back-btn': 'goToClinicSelectView',
				'click #cal-prev-month': 'goToPrevMonth',
                'click #cal-next-month': 'goToNextMonth'
			},
			initialize: function(){
				this.today = this.clearTime(moment().add(1,'days'));
    			this.maxDate = this.clearTime(moment().add(90, 'days'));
			},
			onRender: function(){
				var date = this.getShortMonth(this.model.get('selectedDate'));

				$('.content').addClass('cal-content');	
				this.$el.find('h2').text(date);

				this.delegateEvents();

                this.validatePrevMonth();
                this.validateNextMonth();
			},
			goToClinicSelectView: function () {
				Vent.trigger('show:ds-clinics-no-fetch:view');
			},
			goToPrevMonth: function(){
                $('#calendar').fullCalendar('prev');

                var fullCalendarDate = $('#calendar').fullCalendar('getDate'),
                	$showDate = $('.calendar-nav h2');

                this.model.set('selectedDate', moment(fullCalendarDate).format('MM/DD/YYYY'));

                $('#cal-next-month')[0].disabled = false;

                this.validatePrevMonth();

                $showDate.text(this.getShortMonth(fullCalendarDate));
            },
            validatePrevMonth: function() {
                var currentDate = moment(this.model.get('selectedDate'));

                if (currentDate.month() <= this.today.month() &&
                    currentDate.year() <= this.today.year()){
                    this.$el.find('#cal-prev-month')[0].disabled = true;
                }
            },
            goToNextMonth: function(){
            	$('#calendar').fullCalendar('next');

                var $showDate = $('.calendar-nav h2'),
                	fullCalendarDate = $('#calendar').fullCalendar('getDate');

                this.model.set('selectedDate', moment(fullCalendarDate).format('MM/DD/YYYY'));

                $('#cal-prev-month')[0].disabled = false;
    
			    this.validateNextMonth();
			    
			    $showDate.text(this.getShortMonth(fullCalendarDate));
            },
            validateNextMonth: function () {
                var currentDate = moment(this.model.get('selectedDate'));

                if (currentDate.month() >= this.maxDate.month() &&
                    currentDate.year() >= this.maxDate.year()) {
                    this.$el.find('#cal-next-month')[0].disabled = true;
                }
            },
           	getShortMonth: function (date)	{
				return moment(date).format("MMM YYYY");
			},
            clearTime: function (date) {
                date = moment(date);

                return date.hours(0).minutes(0).seconds(0).milliseconds(0);
            }
		});
	}

);