define([
    'Global',
    'Vent',
    'Calendars',
	'CalendarData',
	'text!../html/calendar.html',
    'text!../html/monthAvailableButton.html'
],
    function (Global, Vent, Calendars, CalendarData,
			  template, availableButtonTemplate) {
        'use strict';

        var CalendarView = Backbone.Marionette.ItemView.extend({

            className: "calendar-section",
            template: _.template(template),
            initialize: function () {
	            this.minDate = moment().toDate();
                this.maxDate = moment().add(90, 'days').toDate();

                this.availableButtonTemplate = _.template(availableButtonTemplate);
            },

            onShow: function () {
                this.showCalendar();
                this.$el.trigger('create');
                $('.facility-provider-info').removeClass('display-none');

            },

            events: {
	            'click .available': 'clickInnerButton',
	            'click .available button': 'showCalendarDayView'
            },

			showCalendar: function () {
                var that = this,
                    date = new Date(this.model.get('selectedDate'));

                that.calendar = that.$el.find('#calendar').fullCalendar({
                    header: {
                        left: '',
                        center: '',
                        right: ''
                    },
                    defaultView: 'month',
                    editable: false,
                    allDaySlot: false,
                    weekMode: 'liquid',
                    viewDisplay: function () {
                        that.handleApptSlots();
                    }
                });

                this.calendar = this.calendar.data().fullCalendar;

                this.calendar.gotoDate(date);

                this.delegateEvents();
			},

	        clickInnerButton: function (e) {
		        $(e.currentTarget).find('button').trigger('click');
	        },

            showCalendarDayView: function(e){
                var date = new moment($(e.currentTarget).parent().attr('data-date')).format('ddd, MMM DD, YYYY');
                this.model.set({'selectedDate': date});
                Vent.trigger('show:ds-calendar-day:view');
            },

            handleApptSlots: function(){
                var slots = this.model.getSlots(),
                    today = this.makeDataDate(),
                    that = this;


                //Make sure call icon is always on todays date block
                $('td[data-date="'+ today +'"]').find('div:first').addClass('day-slot-container icon-call-clinic');

                _.each(slots, function(slot){
                    var startDate = that.makeDataDate(new Date(slot.start)),
                        $daySlot = $('td[data-date="'+ startDate +'"]'),
                        daySlotHTML = $daySlot.html(),
                        $daySlotContainer = $daySlot.find('.day-slot-container');

                    if(startDate === that.makeDataDate()){
                        $daySlot.find('div:first').addClass('day-slot-container icon-call-clinic');
                    } else if (slot.appointmentId) {
                        $daySlot.find('div:first').addClass('patient-appt day-slot-container');
                    } else if (!$daySlot.hasClass('available') &&
                                !that.model.isSlotHidden(slot) &&
                                that.minDate < slot.start && that.maxDate > slot.start){
                        $daySlot.addClass('available').empty().append(that.availableButtonTemplate());
                        $daySlot.find('button').append(daySlotHTML);

                        $daySlotContainer = $daySlot.find('button > div');
                        $daySlotContainer.addClass('day-slot-container');

                        if ($daySlot.hasClass('patient-appt')) {
                            $daySlot.removeClass('patient-appt');
                            $daySlotContainer.addClass('patient-appt');
                        }
                    }
                    
                });
            },
            makeDataDate: function (date) {
                return moment(date).format('YYYY-MM-DD');
            }
        });

		return CalendarView;
    }
);
