define([
    'Global',
    'Vent',
    'User',
    'text!../html/confirmCancellationAlertDialog.html'
],
    function (Global, vent, User, dialogTemplate){
        'use strict';

        return Backbone.Marionette.ItemView.extend({
            template:_.template(dialogTemplate),
            id: 'request-cancellation',
            events: {
                'click #go-back': 'goBack',
                'click #confirm-cancellation-btn': 'cancelRequest'
            },
            modelEvents:{
                "change:status": "updateRequestStatus"
            },
            onRender: function(){
                this.delegateEvents();
            },
            initialize: function(options){
                _.bindAll(this);
                var options = options || {};
                $.extend(this.attributes, options.attributes);
            },
            templateHelpers: function(){
                var that = this;
                return {
                    showNotice: function(){
                        return that.model.isBookedWithDaysLeft(4)
                    }
                };
            },
            goBack: function () {
				this.undelegateEvents();
				vent.trigger("close:popup");
            },
            cancelRequest: function(){
                this.model.set("status", "Cancelled");
            }, 
            updateRequestStatus: function(){
                var that = this,
                    request = this.model.clone(),
                    detailCode,
                    appointmentRequestDetailCode,

                    identifier = request.get('dataIdentifier').uniqueId;

                if( request.isBooked() ){
                    detailCode = {
                        userId: User.get("id"),
                        detailCode: {
                            code: "DETCODE8"
                        }
                    };
                    appointmentRequestDetailCode = request.getDetailCode();

                    request.set("appointmentRequestDetailCode", appointmentRequestDetailCode || []);
                    appointmentRequestDetailCode.unshift(detailCode);
                }

                request.setDetailCodeForSave();

                request.unset('link');
                request.unset('dataIdentifier');

                request.save({ 
                        id: identifier
                    },{
                        url:this.model.get("self").href,
                        success: function(){
                            vent.trigger("close:popup");
                            vent.trigger("show:request:listing", {refetch: true});
                        },
                        error:function(){
                            that.model.set("status", that.model.previousAttributes().status, {silent: true});
                            vent.trigger("close:popup");
                            vent.trigger("show:request:listing");
                            if (navigator.onLine) {
                                vent.trigger("show:errorpopup", "Error canceling request.  Please try again later or contact technical support for assistance.");
                            }
                        }
                    }
                );
            },
            onShow: function () {
                this.delegateEvents();
            }
        });
    }
);