define(['Global', 'Vent', 'text!../html/dsCancelAppt.html'],
	function(Global, vent, template){
		'use strict';

		return Backbone.Marionette.ItemView.extend({
			template: _.template(template),
			className: 'cancel-appt',
			events:{
				"click #no-cancel": "goBack",
				"click #cancel": "cancelAppt",
				"change #reason": "enableCancelBtn",
				"keyup #notes": "updateRemainingLimit"
			},
			ui:{
				reason: "#reason",
				notes: "#notes",
				cancel: "#cancel",
				remaining: "#remaining-chars",
				notesRule: "#notes-rule",
				noApptsAvail: "#no-appointments-available"
			},
			modelEvents: {
				sync: 'render'
			},
			updateRemainingLimit: function(e){
				var max = this.ui.notes.attr("maxlength");
				var val	= this.ui.notes.val();
				var remaining = max-val.length;
				this.ui.remaining.text(remaining + " " + this.getPluralism("character", remaining));
				this.ui.notesRule[(remaining < 0 ? "add" : "remove") + "Class"]("warning");	
			},
			enableCancelBtn: function(){
				this.ui.cancel[0].disabled = !this.ui.reason[0].value.length;
			},
			cancelAppt: function(){
				var model = new Backbone.Model({
					reason: this.ui.reason.val(),
					notes: this.ui.notes.val()
				});
				vent.trigger( "do:cancel:my-appointment", this.appt, model, this.facility );
			},
			templateHelpers:function(){
				var that = this;
				return {
					getFacility: function(){
						return that.facility;
					},
					getAppointmentDateTime: function(){
						return moment(that.appt.get('start')).format("dddd, MMMM DD, YYYY @ HH:mm");
					},
					getFacilityClinic: function(){
						return that.facility.get('facilityName') + ", " + that.appt.get('clinic').name;
					},
					getProvider: function(){
						return that.appt.get('providerName');
					}
				}
			},
			onRender: function(){
				this.updateRemainingLimit();
				this.enableCancelBtn();
				this.delegateEvents();
				this.ui.reason.selectmenu();
			},
			onShow: function(){
				this.enableCancelBtn();
				this.updateRemainingLimit();
				this.ui.reason.selectmenu();
			},
			onClose: function(){
				this.appointment = null;
				this.facility = null;
			},
			goBack: function () {
				vent.trigger("show:direct-scheduling:appointment-listing");
			},
			getPluralism: function(word, count){
				return word += (count != 1 ) ? "s" : "";
			}
		});

	}
);