define(
	[
		'Global',
		'DSClinicListItemView',
		'Vent',
        'DatePicker',
		'text!../html/dsClinicSelectionList.html'
	],
	function(Global, DSClinicListItemView, Vent, datePicker, template){
		"use strict";

		return Backbone.Marionette.CompositeView.extend({
			attributes:{
				id: "direct-scheduling-clinics",
				class: "direct-scheduling-clinics"
			},
			itemView: DSClinicListItemView,
			template: _.template(template),
			itemViewContainer: "#clinic-list",
			collectionEvents: {
				sync: 'render'
			},
			events: {
				'change #desiredDate, [name="clinic"]': 'enableDisableFindAppointmentButton',
                'keyup #reason-for-appt': 'updateRemainingLimit',
				'click #find-appointment': 'showCalendarSelection'
			},
			ui:{
				desiredDate: '#desired-date',
				findAppointmentButton: '#find-appointment',
                apptReason: "#reason-for-appt",
                remaining: "#remaining-chars",
                notesRule: "#notes-rule"
            },
            templateHelpers: function(){
            	var that = this;
            	return {
					hasAvailableSlots: function (hideIfTrue) {
						var bool = that.collection.totalSlots > 0;
						return hideIfTrue ? (bool ? "hidden" : "") : (bool ? "" : "hidden");
					}
            	}
            },
            updateRemainingLimit: function(){
                var max = this.ui.apptReason.attr("maxlength");
                var val	= this.ui.apptReason.val();
                var remaining = max - val.length;
                this.ui.remaining.text(remaining + " " + this.getPluralism("character", remaining));
                this.ui.notesRule[(remaining < 0 ? "add" : "remove") + "Class"]("warning");

                this.enableDisableFindAppointmentButton();
            },
            showCalendarSelection: function(e){
                var that = this;

                this.model.set({
                    'apptReason': that.ui.apptReason.val(),
                    'desiredDate': new Date(that.ui.desiredDate.val()),
                    'selectedDate': moment(this.ui.desiredDate.val()).format('MM/DD/YYYY')
                });

                e.preventDefault();
	            Vent.trigger("set:ds-calendar-data", this.model);
                Vent.trigger("show:ds-calendar-month:view");
            },
			enableDisableFindAppointmentButton: function () {
				var desiredDate = !this.ui.desiredDate.val().length,
                    providerChecked = !this.$el.find('[name="clinic"]:checked').length,
					appointmentReason = !this.ui.apptReason.val().length;

				this.ui.findAppointmentButton[0].disabled = desiredDate || providerChecked || appointmentReason;
			},
			onRender: function(){
				var that, today;

				today = Date.now();
				that = this;
				this.delegateEvents();
				$('#date-picker').remove();
				this.ui.desiredDate.mobile508datepicker({
					onClose: function () { that.enableDisableFindAppointmentButton(); },
					"leading-zero": true,
					MIN: moment(today).add(1, 'days').toDate(),
					MAX: moment(today).add(90, 'days').toDate()
				});

				if (this.collection.totalSlots) {
					this.updateRemainingLimit();
				}

				this.$el.removeClass('hidden');
			},
			onShow: function () {
				if (!this.showView) {
					this.$el.addClass('hidden');
				} else {
					this.showView = true;
				}
			},
			getPluralism: function(word, count){
				return word += (count != 1 ) ? "s" : "";
			}
		});
	}
);