define(
    ['Global', 'Vent', 'text!../html/dsVeteranEmail.html'],
    function(Global, vent, template){
        'use strict';
        return Backbone.Marionette.ItemView.extend({
            template: _.template(template),
            ui: {
                '$email': '#email',
                '$confirmEmailWrapper': '.confirm-email-wrapper',
                '$confirmEmail': '#confirm-email',
                '$validator': '.validator',
                '$validationError': '.validationError',
                '$okBtn': '#ok-btn'
            },
            events: {
                'keyup #confirm-email': 'validateEmail',
                'change #email': 'validateEmail',
                'click #ok-btn': 'showConfirmation',
                'click #cancel-btn': 'showLanding'
            },
            onShow: function(){
                this.delegateEvents();
            },
            showLanding: function(){
                vent.trigger('show:landing');
            },
            showConfirmation: function(){
                var path = this.model.get('path'),
                    clone = this.model.clone();

                clone.set( 'emailAddress', this.ui.$email.val() );
                if(!path){
                    var facility = clone.get('facility') || {};
                    facility.name = facility.facilityName;
                    clone.set('appointmentTime', clone.get('appointmentDate') + ' ' + clone.get('appointmentTime') + ':00');
                    clone.set('facility', facility);
                }
                vent.trigger('show:email-confirmation', clone, path);
            },
            validateEmail: function(){

                var emailRegex = Global.getUtilities().regexes().email,
                    emailStr = this.ui.$email.val().toLowerCase(),
                    confirmEmailStr = this.ui.$confirmEmail.val().toLowerCase(),
                    emailMatch = emailRegex.test(emailStr),
                    emailsMatch = emailStr === confirmEmailStr;

                Global.getUtilities().doErrorChecking([
                    {field: this.ui.$email, condition: !emailMatch, error: 'Please enter a valid email address, i.e. example@example.com'}
                ]);

                Global.getUtilities().doErrorChecking([
                    {field: this.ui.$confirmEmail, condition: confirmEmailStr.length && !emailsMatch, error: 'Your emails do not match'}
                ]);

                if (emailMatch){
                    this.ui.$confirmEmail.removeAttr('disabled');
                    this.ui.$confirmEmailWrapper.removeClass('disabled');
                } else {
                    this.ui.$confirmEmail.attr('disabled', 'disabled');
                    this.ui.$confirmEmailWrapper.addClass('disabled');
                }

                if ( emailsMatch && emailMatch ){
                    this.ui.$okBtn.removeAttr('disabled');
                } else {
                    this.ui.$okBtn.attr('disabled', 'disabled');
                }
            },
            templateHelpers: function () {
                var that = this;
                return {
                    getAppointmentDateTime: function(){
	                    var startTime = that.model.get('start');

                        return moment(startTime).format('dddd, MMMM D, YYYY @ HH:mm');
                    },
	                getFacilityClinic: function(){
		                var facility = that.model.get('facility') || {},
			                facilityName = facility.facilityName || that.model.get('facilityName'),
			                clinic = that.model.get('clinic'),
			                clinicName = typeof clinic === "string" ? clinic : clinic.name;

		                return facilityName + ", " + clinicName;
	                },
	                getBookingNotes: function() {
		                return that.model.get('bookingNotes') || that.model.get('notes');
	                },
	                getHeaderIcon: function () {
		                var path = that.model.get('path') || 'view';

		                return path.toLowerCase() === 'view' ? 'fa-user' : 'icon-calendar';
	                }
                }
            }
        });
    }
);