define(['Global', 'Vent', 'FacilityListItem', 'FacilityTitle', 'text!../html/facilitySelectionList.html'],
	function(Global, Vent, FacilityListItem, FacilityTitle, template){
		"use strict";

		return Backbone.Marionette.CompositeView.extend({
			itemViewContainer: "#facilities .list",
			attributes:{
                "class": "auto-center"
			},
            events: {
                'keyup #facility-search': 'setSearchTimer'
            },
            ui: {
              facilitySearch: '#facility-search',
              facilitySearchResults: '#facilities input[type=radio]:first-child'
            },
            modelEvents: {
                'change': 'setFacilityName'
            },
			getItemView: function(item){
                if(item.get('title')){
                    return FacilityTitle;
                }
                return FacilityListItem;
            },
			template: _.template(template),
            initialize: function(options){
                options = options || {};
                this.getFacilityLink = options.getFacilityLink;
                this.timer = 0;

            },
            onShow: function(){
                this.delegateEvents();
            },
            setSearchTimer: function(){
                var that = this;
                clearTimeout(this.timer);
                if (this.ui.facilitySearch.val().length > 1){
                    this.timer = setTimeout( function(){that.getFacilities()}, 500);
                } else {
                    this.collection.reset();
                }
            },
            getFacilities: function(){
                var that = this;
                this.collection.fetch({
                    url: that.getFacilityLink + "?parentFacilitiesOnly=true&term=" + this.ui.facilitySearch.val()
                });
            },
            setFacilityName: function(){
                this.ui.facilitySearch.val(this.model.get('facilityName'));
                this.collection.reset();
            }
		});
	}
);