define([
        'Global',
        'Vent',
        'Providers',
        'RequestMetaData',
        'Resources',
        'ProviderSelectView',
        'text!../html/newRequestDialog.html',
        'text!../html/newRequestByPhone.html',
        'AppointmentRequestDetailsDetailsView',
        'FacilitySelectionList',
        'ErrorView',
        'User',
        'AppointmentRequestMessage',
        'VarResources',
        'ErrorPopupView',
        'WarningsView'
    ],
    function (Global, Vent, Providers, RequestMetaData, Resources, ProviderSelectView, newRequestTemplate, newRequestByPhoneTemplate, AppointmentRequestDetailsDetailsView, FacilitySelectionList, ErrorView, User, AppointmentRequestMessage, VarResources, ErrorPopupView, WarningsView) {
        'use strict';
        return Backbone.Marionette.Layout.extend({
            template: _.template(newRequestTemplate),
            id: 'new-request',
            events: {
                'click #cancel-request': 'cancelRequest',
                'change #providerId': 'setProviderOptionDisplay',
                'click #review-request-btn': 'reviewRequest',
                'click #edit-request': 'editRequest',
                'click #submit-request': 'submitRequest',

                //updated
                'change [name=optionTime1]': 'showNextOptionalDate',
                'change [name=optionDate1]': 'showNextOptionalDate',
                'change [name=optionTime2]': 'showNextOptionalDate',
                'change [name=optionDate2]': 'showNextOptionalDate',
                'change [name=optionDate3]': 'showNextOptionalDate',
                'change [name=optionTime3]': 'showNextOptionalDate',
                'change .optionDates': 'activateAmPm',
                'change input:not([name="providerId"]), textarea': 'updateModel',
                'change #facilityName': 'updateFacility',
                'change #purposeOfVisit': 'toggleRequiredPurpose',
                'change [name=visitType]': 'toggleTypeOfVisitConditions',
                'change [name="appointmentType"]': 'enableDisableProviderSelect',
                'change [name$="phoneNumber"]': 'verifyPhone',
                'change [name="verifyPhoneNumber"]': 'verifyPhone',
                'update input[type="radio"], input[type="checkbox"]': 'updateEnabledDisabled',
                'keyup #initialRequestMessage': 'updateRemainingLimit'

            },
            ui: {

                //Non Inputs
                facilitiesContainer: '#facilities-container',
                form: '#request-form-form',
                otherPurpose: '#otherPurposeOfVisitContainer',
                providerOptionContainer: '#providerOptionContainer',
                providerListedOptions: '#providerListedOptions',
                providerNotListedOptions: '#providerNotListedOptions',
                providerContainerLegend: '#provider-container legend',
                purposeOfVisitSection: '#purpose-of-visit-section',
                videoConferenceCaveatText: '#videoConferenceCaveatText',
                visitTypesSection: '#visitTypes',
                warnings: '#warning-view',
                notesRule: '#notes-rule',
                remaining: '#remaining-chars',
                initialRequestLbl: '[for="initialRequestMessage"]',

                //Labels
                bestTimeLbl: '#bestTimetoCall',
                mentalHealthLbl: 'label[for="Mental Health"]',
                phoneNumberLbl: '[for=phoneNumber]',
                verifyPhoneLbl: '[for=verifyPhoneNumber]',
                primaryCareLbl: 'label[for="Primary Care"]',
                phoneCallLbl: 'label[for="Phone Call"]',


                //Inputs
                appointmentType: '[name=appointmentType]',
                bestTime: '[name=bestTimetoCall]',
                initialMessage: '#initialRequestMessage',
                optionDate1: '#optionDate1',
                optionDate2: '#optionDate2',
                optionDate3: '#optionDate3',
                optionTime1: '[name=optionTime1]',
                optionTime2: '[name=optionTime2]',
                optionTime3: '[name=optionTime3]',
                phoneNumber: '#phoneNumber',
                provider: '#providerId',
                providerOption: '[name=providerOption]',
                purpose: '#purposeOfVisit',
                requestCall: '#call-checkbox',
                verifyPhoneNumber: '#verifyPhoneNumber',
                visitType: '[name=visitType]',

                //Buttons
                reviewRequestContent: '#review-request-content',
                reviewRequest: '#review-request',
                submitRequest: '#submit-request',
                editRequest: '#edit-request'

            },
            initialize: function (options) {
                options = options || {byPhone: false};

                if (options.byPhone) {
                    this.byPhone = options.byPhone;
                    this.template = _.template(newRequestByPhoneTemplate);
                }

                $.extend(this.attributes, options.attributes);

                this.optionDateTimeCounter = 0;
                this.requests = options.requests;
                this.utilities = Global.getUtilities();
            },
            regions: {
                facilitiesContainer: '#facilities-container',
                providerOptionsContainer: '#provider-container',
                notificationPreferences: '.notification-preferences'
            },
            templateHelpers: function () {
                var that = this;
                return {
                    getAppointmentTypes: function () {
                        return that.requestMetaData.appointmentTypes;
                    },
                    getVisitTypes: function () {
                        return that.requestMetaData.visitTypes;
                    },
                    getPurposeOfVisitOptions: function () {
                        return that.requestMetaData.purposeOfVisitOptions;
                    },
                    getBestTimetoCallOptions: function () {
                        return that.requestMetaData.bestTimetoCallOptions;
                    }
                }
            },
            showNextOptionalDate: function(e){
                var optionTimeNumber = $(e.currentTarget).attr('name').match(/(\d+)/g),
                    $optionTime = $('[name=optionTime' + optionTimeNumber + ']'),
                    $optionTimeIsChecked = $('[name=optionTime' + optionTimeNumber + ']:checked').val(),
                    $optionDate = $('#optionDate' + optionTimeNumber).val();
                if ($optionDate){
                    $optionTime.attr({"disabled": false});
                }
                if ($optionDate && $optionTimeIsChecked && (optionTimeNumber < 3)){
                    $('.desired-dates .validator:eq(' + optionTimeNumber + ')').removeClass('hidden');
                }
            },
            activateAmPm: function(e){
                var currentOptionDate = $(e.currentTarget);

                if (this.ui.optionDate2.val() || this.ui.optionDate3.val()){
                    currentOptionDate.attr('name') === 'optionDate2' ? this.ui.optionTime2[0].click() : this.ui.optionTime3[0].click();
                }
            },
            onShow: function () {
                var that = this, fiveDaysFromNow, maxDate, options;

                this.warningsView = new WarningsView({el: this.ui.warnings});

                if (!this.byPhone) {
                    fiveDaysFromNow = moment().add(5, 'days');
                    maxDate = moment(fiveDaysFromNow).add(90, 'days');
                    options = {
                        "leading-zero": true,
                        MIN: fiveDaysFromNow.toDate(),
                        MAX: maxDate.toDate()
                    };

                    this.ui.optionDate1.mobile508datepicker($.extend(options, {id: 'date-1', onClose: function () {
                        that.ui.optionDate1.change();
                    }}));
                    this.ui.optionDate2.mobile508datepicker($.extend(options, {id: 'date-2', onClose: function () {
                        that.ui.optionDate2.change();
                    }}));
                    this.ui.optionDate3.mobile508datepicker($.extend(options, {id: 'date-3', onClose: function () {
                        that.ui.optionDate3.change();
                    }}));
                    this.updateRemainingLimit();
                }

                this.toggleTypeOfCare();

                this.$el.trigger("create");

                this.ui.phoneNumber.mask(Global.getUtilities().masks().USPhone);
                this.ui.verifyPhoneNumber.mask(Global.getUtilities().masks().USPhone);

                this.ui.verifyPhoneNumber[0].disabled = true;

                this.delegateEvents();
                that.providerOptionsContainer.on('show', function () {
                    that.bindUIElements();
                });
            },
            customValidators: function () {
                var that = this,
                    optionTime1 = that.$el.find('[name="optionTime1"]:checked').val() || "",
                    optionDateTime1 = that.ui.optionDate1.val() + " " + optionTime1,
                    optionDate2 = that.ui.optionDate2.val() || "",
                    optionTime2 = that.$el.find('[name="optionTime2"]:checked').val() || "",
                    optionDateTime2 = optionDate2 + " " + optionTime2,
                    optionDate3 = that.ui.optionDate3.val() || "",
                    optionTime3 = that.$el.find('[name="optionTime3"]:checked').val() || "",
                    optionDateTime3 = optionDate3 + " " + optionTime3,
                    optionDateCondition1 = optionDateTime1 && (optionDateTime1 === optionDateTime2 || optionDateTime1 === optionDateTime3),
                    optionDateCondition2 = optionDateTime2 && (optionDateTime2 === optionDateTime1 || optionDateTime2 === optionDateTime3),
                    optionDateCondition3 = optionDateTime3 && (optionDateTime3 === optionDateTime1 || optionDateTime3 === optionDateTime2),
                    phoneValidation = {
                        condition: function () {
                            var phoneVal = that.ui.phoneNumber.val().trim(),
                                vPhoneVal = that.ui.verifyPhoneNumber.val().trim();
                            return !that.utilities.regexes().USPhone.test(phoneVal);
                        },
                        error: 'Please enter a valid US Phone number i.e. (703) 123-1234'
                    },
                    verifyPhoneValidation = {
                        condition: function () {
                            var phoneVal = that.ui.phoneNumber.val(),
                                vPhoneVal = that.ui.verifyPhoneNumber.val();
                            return phoneVal !== vPhoneVal;
                        },
                        error: 'Your phone numbers do not match.'
                    }

                if (this.byPhone) {
                    var phoneVal = that.ui.phoneNumber.val().trim();
                    var vPhoneVal = that.ui.verifyPhoneNumber.val().trim();
                    var error = (phoneVal != vPhoneVal) ? 'Phone Numbers do not match' : 'Please verify phone number.';
                    return {
                        phoneNumber: phoneValidation,
                        verifyPhoneNumber: {
                            condition: function () {
                                return phoneVal != vPhoneVal || vPhoneVal === ''
                            },
                            error: error
                        }
                    };
                }
                return {
                    phoneNumber: phoneValidation,
                    verifyPhoneNumber: verifyPhoneValidation,
                    optionDate1: {
                        condition: function () {
                            return !that.utilities.regexes().StdDate.test(that.ui.optionDate1.val()) || !that.utilities.getCheckedVal(that.ui.optionTime1);
                        },
                        error: 'Please select a Date and AM/PM.'
                    }, 
                    optionTime1: {
                        condition: function () {
                            if (that.optionDateTimeCounter < 1) {
                                that.checkOptionDateTime(that.ui.optionTime1);
                            } else {
                                that.optionDateTimeCounter = 0;
                            }
                            return optionDateCondition1;
                        },
                        error: 'Please select a unique date preference'
                    },   
                    optionDate2: {
                        condition: function () {
                            return optionDateCondition2;
                        },
                        error: 'Please select a unique date preference'
                    },
                    optionTime2: {
                        condition: function () {
                            return optionDateCondition2;
                        },
                        error: 'Please select a unique date preference'
                    },
                    optionDate3: {
                        condition: function () {
                            return optionDateCondition3;
                        },
                        error: 'Please select a unique date preference'
                    },
                    optionTime3: {
                        condition: function () {
                            return optionDateCondition3;
                        },
                        error: 'Please select a unique date preference'
                    },
                    initialRequestMessage: {
                        condition: function () {
                            return that.ui.initialMessage.val().length < that.ui.initialMessage.data('maxLength');
                        },
                        error: 'Message should be 100 characters or less'
                    }
                };
            },
            checkOptionDateTime: function(element){

                var checks = [];
                
                if (element.attr('name') === 'optionTime1') {
                    this.optionDateTimeCounter++;
                    if (!this.ui.optionTime2.is(":disabled")){
                        checks.push(Global.getUtilities().validateElement(this.ui.optionDate2, this.customValidators()));
                        checks.push(Global.getUtilities().validateElement(this.ui.optionTime2, this.customValidators()));
                    }
                    if (!this.ui.optionTime3.is(":disabled")){
                        checks.push(Global.getUtilities().validateElement(this.ui.optionDate3, this.customValidators()));
                        checks.push(Global.getUtilities().validateElement(this.ui.optionTime3, this.customValidators()));
                    }
                }
                if (element.attr('name') === 'optionTime2') {
                    this.optionDateTimeCounter++;
                    checks.push(Global.getUtilities().validateElement(this.ui.optionDate1, this.customValidators()));
                    checks.push(Global.getUtilities().validateElement(this.ui.optionTime1, this.customValidators()));
    
                    if (!this.ui.optionTime3.is(":disabled")){
                        checks.push(Global.getUtilities().validateElement(this.ui.optionDate3, this.customValidators()));
                        checks.push(Global.getUtilities().validateElement(this.ui.optionTime3, this.customValidators()));
                    }
                    
                }
                if (element.attr('name') === 'optionTime3') {
                    this.optionDateTimeCounter++;
                    checks.push(Global.getUtilities().validateElement(this.ui.optionDate1, this.customValidators()));
                    checks.push(Global.getUtilities().validateElement(this.ui.optionTime1, this.customValidators()));
                    checks.push(Global.getUtilities().validateElement(this.ui.optionDate2, this.customValidators()));
                    checks.push(Global.getUtilities().validateElement(this.ui.optionTime2, this.customValidators()));
                    
                }
                Global.getUtilities().doErrorChecking(checks);
            },
            toggleTypeOfCare: function () {
                if (this.requests.primaryCareLimitReached()) {
                    this.ui.primaryCareLbl.addClass("ui-disabled");
                    this.warningsView.addMessage({message: 'Primary Care Request Limit of one (1) reached.'});
                }
                if (this.requests.mentalHealthLimitReached()) {
                    this.ui.mentalHealthLbl.addClass("ui-disabled");
                    this.warningsView.addMessage({message: 'Mental Health Request Limit of two (2) reached.'});
                }
                if (this.warningsView.collection.length) {
                    this.warningsView.addMessage({message: 'Please cancel a pending request or wait for the VA to process one or more current requests in order to proceed with submitting a new request.'});
                }
            },
            cancelRequest: function () {
                Vent.trigger('show:bookingLandingPage');
            },
            setFacility: function (facility) {

                this.model.set('facility', facility.toJSON());
                this.model.set('facilityName', facility.get('name'));
                this.model.set('facilityCode', facility.get('facilityCode'));
                this.updateFacility();
            },
            setProvider: function (provider) {
                this.model.set('provider', provider.toJSON());
                this.model.set('providerName', provider.get('providerName'));
                this.model.set('providerId', provider.get('providerId'));
                this.utilities.doErrorChecking([this.utilities.validateElement(this.ui.provider, this.customValidators())]);
            },
            updateFacility: function () {
                var that = this,
                    appointmentType = this.utilities.getCheckedVal(this.ui.appointmentType);
                if (appointmentType) {
                    this.createProviderSelectView(appointmentType);
                }
            },
            updateRemainingLimit: function () {
                var max = this.ui.initialMessage.attr("maxlength") || 100;
                var val = this.ui.initialMessage.val();
                var remaining = max - val.length;
                this.ui.remaining.text(remaining + " " + this.getPluralism("character", remaining));
                this.ui.notesRule[(remaining < 0 ? "add" : "remove") + "Class"]("warning");
            },
            getPluralism: function (word, count) {
                return word += (count != 1 ) ? "s" : "";
            },
            updateModel: function (e) {
                var ele = $(e.currentTarget),
                    check = ele.is(':checkbox'),
                    container,
                    val;

                if (check) {
                    container = this.ui.form;
                }

                this.utilities.doErrorChecking([this.utilities.validateElement(ele, this.customValidators())]);

                val = this.utilities.getElementValue(ele, container);
                this.model.set((ele.attr('name') || ele.attr('id')), val, {silent: true});
            },
            toggleRequiredPurpose: function (e) {
                var purpose = this.utilities.getElementValue(this.ui.purpose);

                this.ui.otherPurpose[(/^Other$/i.test(purpose) ? 'show' : 'hide')]();
                this.updateForMedicationConcern();
                this.updateModel(e);
            },
            verifyPhone: function () {
                if (this.ui.phoneNumber.val().length && this.ui.verifyPhoneNumber[0].disabled) {
                    this.ui.verifyPhoneNumber[0].disabled = false;
                    this.ui.verifyPhoneNumber.focus();
                }

                var phVal = this.ui.phoneNumber.val();
                var vPhVal = this.ui.verifyPhoneNumber.val();

                Global.getUtilities().doErrorChecking([
                    {
                        field: this.ui.verifyPhoneNumber,
                        error: 'The phone numbers do not match.',
                        condition: phVal != vPhVal && vPhVal.length
                    }
                ]);
            },
            updateForMedicationConcern: function () {
                if (this.ui.purpose.val() === 'Medication concern') {
                    var primaryCareSelected = (this.utilities.getCheckedVal(this.ui.appointmentType) === 'Primary Care');
                    if (primaryCareSelected) {
                        this.ui.visitType.removeAttr('checked');
                        _.each(this.ui.visitType, function (type) {
                            type = $(type);
                            if (type.val() === 'Phone Call') {
                                type.attr('checked', true).trigger('change');
                                type.next('label').click().addClass('no-top-border');
                            }
                            else {
                                type.hide();
                                type.next('label').hide();
                                type.attr('disabled', true);
                            }
                        });

                        new ErrorPopupView({title: 'Please Note', message: 'A telephone visit is recommended for medication concerns, the request form has been set to telephone visit for you.'}).render();

                        return;
                    }

                }
                _.each(this.ui.visitType, function (type) {
                    type = $(type);
                    type.show();
                    type.next('label').show().removeClass('no-top-border');
                    type.removeAttr('disabled');
                });

            },
            resetProviderOption: function () {
                _.each(this.ui.providerOption, function (ele) {
                    $(ele).attr('checked', false);
                });
            },
            setProviderOptionDisplay: function () {
                var providerValue = this.ui.provider.val();
                this.resetProviderOption();
                this.ui.providerOptionContainer.removeClass('display-none');
                if (providerValue != '0') {
                    this.ui.providerListedOptions.removeClass('display-none');
                    this.ui.providerNotListedOptions.addClass('display-none');
                } else {
                    this.ui.providerListedOptions.addClass('display-none');
                    this.ui.providerNotListedOptions.removeClass('display-none');
                }
                if (!providerValue || providerValue === "") {
                    this.ui.providerListedOptions.addClass('display-none');
                    this.ui.providerNotListedOptions.addClass('display-none');
                }
            },
            enableDisableProviderSelect: function () {
                this.updateForMedicationConcern();
                if (this.byPhone) {
                    return;
                }
                var that = this,
                    appointmentType = Global.getUtilities().getCheckedVal(this.ui.appointmentType);

                if (appointmentType) {
                    this.ui.visitTypesSection.removeClass('display-none');
                    this.ui.purposeOfVisitSection.removeClass('display-none');

                    if (this.model.has('facility')) {//facilityName.length) {
                        that.createProviderSelectView(appointmentType);

                    } else {
                        that.ui.provider.selectmenu('refresh').selectmenu('disable');
                        this.ui.providerOptionContainer.addClass('display-none');
                        this.ui.providerContainerLegend.show();
                    }
                } else {
                    this.ui.visitTypesSection.addClass('display-none');
                    this.ui.purposeOfVisitSection.addClass('display-none');
                }
            },
            createProviderSelectView: function (appointmentType) {
                var that = this;
                var linkName = /^Mental/i.test(appointmentType) ? 'mental-health-providers' : 'primary-care-providers';
                var url = VarResources.instance().get(linkName).get('href') + '?facilitySiteCode=' + this.model.get('facilityCode');
                var providers = new Providers();

                providers.fetch({url: url}).done(function () {
                    that.providerOptionsContainer.show(
                        new ProviderSelectView({
                            collection: providers
                        })
                    );
                    that.ui.providerContainerLegend.hide();
                    that.ui.provider.selectmenu('refresh');
                });
            },
            toggleTypeOfVisitConditions: function () {
                var visitType = this.utilities.getCheckedVal(this.ui.visitType);
                this.ui.videoConferenceCaveatText
                    [visitType == 'Video Conference' ? "removeClass" : "addClass"]("display-none");
            },
            removeOptions: function (appointmentRequest) {
                delete appointmentRequest.attributes['bestTimetoCallOptions'];
                delete appointmentRequest.attributes['visitTypes'];
                delete appointmentRequest.attributes['purposeOfVisitOptions'];
                delete appointmentRequest.attributes['appointmentProcessedAsPerUserNeedOptions'];
                delete appointmentRequest.attributes['appointmentRequestStatusOptions'];
                delete appointmentRequest.attributes["appointmentTypes"];
                delete appointmentRequest.attributes["link"];
            },
            reviewRequest: function () {
                var utils = Global.getUtilities();

                var errors = _.filter(utils.validate(this.ui.form, this.customValidators()), function (error) {
                    return error.condition;
                });

                if (errors.length > 0) {
                    return;
                }

                this.model.set('requestedPhoneCall', this.byPhone);
                new AppointmentRequestDetailsDetailsView({
                    model: this.model,
                    viewContainer: this.ui.reviewRequestContent,
                    byPhone: this.byPhone
                }).render();

                this.ui.reviewRequest.toggleClass('display-none');
                this.ui.form.toggleClass('display-none');
            },
            editRequest: function () {
                this.ui.reviewRequest.toggleClass('display-none');
                this.ui.form.toggleClass('display-none');
            },
            submitRequest: function () {
                var appointmentRequest = this.model,
                    view = this;

                appointmentRequest.set({'cid': appointmentRequest.cid});
                appointmentRequest.unset("dataIdentifier", {silent: true});
                appointmentRequest.prepareForSave();
                appointmentRequest.setDetailCodeForSave();
                appointmentRequest.set('requestedPhoneCall', this.byPhone);
                if (this.byPhone) {
                    appointmentRequest.set('optionDate1', 'No Date Selected');
                    appointmentRequest.set('optionTime1', 'No Time Selected');
                }

                this.removeOptions(appointmentRequest);

                appointmentRequest.save({}, {
                    url: VarResources.instance().get("var-appointments").get('href'),
                    success: function (response) {
                        view.collection.fetch();
                        var initialRequestMessage = appointmentRequest.get("initialRequestMessage");
                        if (initialRequestMessage !== undefined && initialRequestMessage.trim().length) {
                            view.saveAppointmentRequestMessage(response, initialRequestMessage);
                        }
                        response.set({byPhone: view.byPhone});
                        Vent.trigger('show:request:submitted', response);
                    },
                    error: function (resp, error) {
                        var responseMessage,
                            getError = function (responseText) {
                                responseText = JSON.parse(responseText);

                                var errorMessage = responseText.errors[0].errorMessage.split(':');

                                return errorMessage;
                            };
                        responseMessage = getError(error.responseText);
                        new ErrorPopupView({
                            title: responseMessage[0] ? responseMessage[0] : 'Error',
                            message: responseMessage[1] ? responseMessage[1] : 'An error has occurred on the server. Please try again later.'
                        }).render();
                        $('.divide button').attr("disabled", false);

                    }
                });
            },
            saveAppointmentRequestMessage: function (appointmentRequest, messageText) {
                Global.getUtilities().flattenLinks(appointmentRequest);
                var msgDetails = {
                        messageText: messageText,
                        AppointmentRequestId: appointmentRequest.get('dataIdentifier').uniqueId,
                        senderId: User.get('id')
                    },
                    messagesUrl = appointmentRequest.get('appointment-request-messages').href,
                    aptMessages = new AppointmentRequestMessage();
 
                aptMessages.save(msgDetails, {
                    url: messagesUrl,
                    error: function (resp, error) {
                        new ErrorPopupView({
                            message: 'An error occurred while saving the message. Please try again later.'
                        }).render();
                    }
                });
            },
            goBack: function () {
                if (this.ui.form.is(':visible')) {
                    Vent.trigger('show:bookingLandingPage');
                } else if (this.ui.reviewRequest.is(':visible')) {
                    this.editRequest();
                }
            },
            onClose: function () {
                while($('[id^=date-].ui-popup-container').length){ 
                    $('[id^=date-].ui-popup-container').detach(); 
                    $('[id^=date-].ui-popup-screen').detach();
                }
            }
        });
    }
);