define([
	'Global',
	'text!../html/notificationSettings.html'

], function( Global, template){
	'use strict';

	return Backbone.Marionette.ItemView.extend({
		template: _.template(template),

		events: {
			'keyup input': 'updateSelections',
			'change input': 'updateSelections',
			'change #email-updates': 'showHideEmailFrequency',
			'click #save-preferences': 'updatePreferences'
		},
		ui:{
			emailSection: ".email-section",
			success: ".success",
			successIcon: ".success .block-icon",
			successText: ".success .text",
			savePreferences: "#save-preferences",
			emailUpdates: "#email-updates",
			email: "#email"
		},
		initialize: function () {
			this.bindUIElements();
		},
		onRender: function(){
			this.delegateEvents();
			this.setNotificationPreferencesFields();
			$(window).resize();
		},
		updateSelections: function () {
			var emailAllowed = this.ui.emailUpdates[0].checked,
				frequency = emailAllowed ? "Each new message" : "Never",
				emailAddress = emailAllowed ? this.ui.email.val() : "";

			this.ui.success.addClass('hidden');

			this.preferences.set({
				emailAllowed: emailAllowed,
				emailAddress: emailAddress,
				notificationFrequency: frequency
			});

			this.enableSaveButton();
		},
		showHideEmailFrequency: function () {
			if (this.ui.emailUpdates[0].checked) {
				this.ui.emailSection.removeClass('hidden');
                this.ui.email.focus();
			} else {
				this.ui.emailSection.addClass('hidden');
			}
		},
		validateEmail: function(){
			var emailRegex = Global.getUtilities().regexes().email,
				emailStr = this.ui.email.val().toLowerCase(),
				emailMatch = emailRegex.test(emailStr);

			Global.getUtilities().doErrorChecking([
				{field: this.ui.email, condition: !emailMatch, error: 'Please enter a valid email address, i.e. example@example.com'}
			]);

			return emailMatch;
		},
		setNotificationPreferencesFields: function () {
			var emailAllowed = this.model.get('emailAllowed');

			this.ui.emailUpdates[0].checked = this.model.get('emailAllowed');

			if (emailAllowed) {
				this.ui.emailSection.removeClass('hidden');
			}

			this.preferences = this.model.clone();
		},
		enableSaveButton: function () {
			var currentPreferences = JSON.stringify(this.preferences.toJSON()),
				savedPreferences = JSON.stringify(this.model.toJSON()),
				emailAllowed = this.preferences.get('emailAllowed'),
				validEmail = this.ui.email.val().length && this.validateEmail(),
				disabled = true,
				formComplete = !emailAllowed || validEmail;

			if (currentPreferences !== savedPreferences && formComplete) {
				disabled = false;
			}

			this.ui.savePreferences[0].disabled = disabled;
		},
		updatePreferences: function () {
			var that = this;

			this.model = this.preferences.clone();

			this.model.save(this.model.toJSON(), {
				error: function () {
					that.ui.successText.text("There was an error saving your preferences, please try again later.");
					that.ui.successIcon.removeClass("icon-check");
					that.ui.successIcon.addClass("icon-alert-2");
					that.ui.success.removeClass('hidden');
				},
				success: function () {
					that.ui.successText.text("Preferences Saved!");
					that.ui.successIcon.removeClass("icon-alert-2");
					that.ui.successIcon.addClass("icon-check");
					that.ui.success.removeClass('hidden');
                    that.ui.savePreferences[0].disabled = true;
				}
			});
		}
	});

});