define([
    'Global',
    'Vent',
    'text!../html/requestNoticeDialog.html'
],
    function (Global, Vent, dialogTemplate){
        'use strict';

        return Backbone.Marionette.ItemView.extend({
            template:_.template(dialogTemplate),
			'id': 'request-notice',
            initialize: function(options){
                var options = options || {}
                $.extend(this.attributes, options.attributes);
            },
            ui:{
                doNotShowNotice: "#do-not-show-notice"
            },
            events: {
                'click #cancel-feedback': 'close',
                'click #dismiss-notice-btn': 'setDoNotShowNotice'
            },
            onRender: function(){
                if (this.config.close){
                    this.$el.find("#dismiss-notice-btn").text('OK');
                } else {
                    this.$el.find("#dismiss-notice-btn").text('Continue');
                }
            },
            onShow: function(){
                this.delegateEvents();

                this.checkShowNoticeCheckbox();
            },
            onClose: function(){
                this.undelegateEvents();
				Vent.trigger("close:popup");
            },
            checkShowNoticeCheckbox: function(){
                if (window.localStorage.getItem('var-web/do-not-show-notice') === "true"){
                     this.$el.find('#do-not-show-notice').prop('checked', true);
                }
            },
            showNotice: function(options){
                options = options || {byPhone: false};
                this.byPhone = options.byPhone;
                var show = true;
                if (Modernizr.localstorage) {
                    if (window.localStorage.getItem('var-web/do-not-show-notice')){
                        show = /false/i.test(window.localStorage.getItem("var-web/do-not-show-notice"));
                    }
                } 
                else {
                    this.ui.doNotShowNotice
                        .siblings('label')
                        .remove()
                        .end()
                        .remove();
                }
                return show;
            },
            setDoNotShowNotice: function(){
                var doNotShow = this.ui.doNotShowNotice[0].checked;
                try{
                    window.localStorage.setItem("var-web/do-not-show-notice", doNotShow)
                }catch(err){}
                this.continueToNewRequest();
            },
            continueToNewRequest: function(){
                this.undelegateEvents();
                var config = {doNotShowAgain: true};
                if (this.config.close){
                    Vent.trigger("show:bookingLandingPageView");
                } else{
                    if(this.byPhone){
                        Vent.trigger("show:request:new-phone", config);
                    }
                    else{
                        Vent.trigger("show:request:new", config);
                    }
                }

				this.close();
            }
        });
    }
);
