define(['Global', 'User', 'text!../html/roa.html'],
	function(Global, User, template){
		'use strict';

		return Backbone.Marionette.ItemView.extend({
			template: _.template(template),
			id: 'roa-view',
			className: 'roa-view',
			ui: {
				roa: '#roa',
				formReview: '#form-review',
				roaImage: '#roa-image'
			},
			events: {
				'click #launchpad': 'launchpad',
				'click #next': 'clickNext',
				'click #cancel': 'resetForm',
				'click .ui-header button': 'zoomInOut',
				'click #verify-btn': 'verify'
			},
			initialize: function () {
				this.User = User;
			},
			onShow: function () {
				var ratio = 1.295638126009693,
					that = this;
				this.availableWidth = Math.round(this.ui.formReview.width() - 30);
				this.scaledHeight = Math.round(this.availableWidth * ratio);

				$('.ui-footer').hide();
				this.delegateEvents();
				this.resetForm();

				$(window).resize(function () {
					that.resizeROA();
				});
			},
			resizeROA: function () {
				var $roa = $('#roa'),
					$container = $roa.is(':visible') ? $roa : $('#form-review'),
					$content = $('.roa-content'),
					containerHeight = $container.outerHeight(),
					headerHeight = $container.find('.ui-header').outerHeight(),
					footerHeight = $container.find('.footer').outerHeight();

				$content.outerHeight(containerHeight - headerHeight - footerHeight);

				$content.css({"margin-top": headerHeight + "px"});
			},
			onClose: function () {
				$('.ui-footer').show();
			},
			resetForm: function () {
				$('.ui-title')
					.eq(0)
					.text('Right of Access')
					.parent().find('ui-btn-right').remove();
				this.ui.roa.show();
				this.ui.formReview.hide();
			},
			clickNext: function () {
				var roaImage = this.ui.roaImage[0];
				$('.ui-title')
					.eq(0)
					.text('Review');
//					.after('<div data-role="controlgroup" data-type="horizontal" class="ui-btn-right" ><button type="button" title="in" class="ui-btn-up-d ui-corner-left"><span class="ui-btn-inner"><span class="ui-icon ui-icon-plus ui-icon-shadow">&nbsp;</span></span></button><button type="button" title="out" class="ui-btn-up-d ui-corner-right ui-controlgroup-last"><span class="ui-btn-inner"><span class="ui-icon ui-icon-minus ui-icon-shadow">&nbsp;</span></span></button></div>');
				this.ui.roa.hide();
				this.ui.formReview.show();

				roaImage.width = this.availableWidth;
				roaImage.height = this.scaledHeight;

				$(window).resize();
			},
			verify: function () {
				var user = this.User;
				user.save({rightOfAccessAccepted: true}, {
					url: user.get("right-of-access").href,
					success: function() {
						window.location = window.location.protocol + "//" + window.location.host + window.location.pathname;
					},
					statusCode: {
						302 : function() {
						//	loginRedirect();
						}
					}
				});
			},
			launchpad: function () {
				window.location.href = '../launchpad';
			},
			zoomInOut: function (e) {
				var roaImage = this.ui.roaImage[0],
					multiplier = e.currentTarget.title === 'in' ? 1.2 : .8;
				roaImage.width = this.ui.roaImage.width() * multiplier;
				roaImage.height = this.ui.roaImage.height() * multiplier;
			}
		});
	}
);
