define([
    'Global',
    'Vent',
    'text!../html/timeSlotSelected.html'
],
    function(Global, vent, template){
        'use strict';
        return Backbone.Marionette.ItemView.extend({
            template: _.template(template),
            attributes: {
                "class": "time-selected confirm"
            },
            events: {
                "keyup #booking-notes": "updateRemainingLimit",
                'click #book-btn': "bookAppointment"
            },
            ui: {
                notes: "#booking-notes",
                remaining: "#remaining-chars",
                notesRule: "#notes-rule",
                bookingBtn: "#book-btn"
            },
            onRender: function () {
                var $facilityProviderInfo = $('.facility-provider-info');

                $facilityProviderInfo.addClass('display-none');
                this.updateRemainingLimit();
                this.delegateEvents();

                this.enableDisableBookingBtn();
            },
            bookAppointment: function(e){
	            e.currentTarget.disabled = true;
	            this.model.set('bookingNotes', this.ui.notes.val());
                vent.trigger('show:showDirectSchedulingConfirmBookingAppt:view', this.model);
            },
            setSlot: function (slot, clinicData) {
                var start = moment(slot.get('start'));

                slot.set({
                    apptLength: clinicData.get('appointmentLength'),
                    apptType: 'P',
                    bookingNotes: clinicData.get('apptReason'),
                    clinic: clinicData.get('clinicName'),
                    clinicId: clinicData.get('clinicId'),
                    date: start.format('ddd, MMM DD, YYYY'),
                    dateTime: slot.get('startDateTime'),
                    desiredDate: moment(clinicData.get('desiredDate')).format('MM/DD/YYYY 00:00:00'),
                    ekg: '',
                    facilityName: clinicData.get('facilityName'),
                    lab: '',
                    lvl: '1',
	                providerName: clinicData.get('providerName'),
                    purpose: 9,
                    xRay: ''
                });

                this.model = slot;

                return this;
            },
            updateRemainingLimit: function(){
                var max = this.ui.notes.attr("maxlength");
                var val	= this.ui.notes.val();
                var remaining = max-val.length;
                this.ui.remaining.text(remaining + " " + this.getPluralism("character", remaining));
                this.ui.notesRule[(remaining < 0 ? "add" : "remove") + "Class"]("warning");

                this.enableDisableBookingBtn();
            },
            enableDisableBookingBtn: function () {
                var apptReason = !this.ui.notes.val().length;

                this.ui.bookingBtn[0].disabled = apptReason;
            },
            goBack: function(){
	            var $facilityProviderInfo = $('.facility-provider-info');

	            $facilityProviderInfo.removeClass("display-none");

                vent.trigger('show:ds-calendar-day:view');
            },
	        templateHelpers: function () {
		        var that = this;
		        return {
			        getAppointmentDateTime: function(){
				        return moment(new Date(that.model.get('startDateTime'))).format('dddd, MMMM D, YYYY @ HH:mm');
			        }
		        }
	        },
	        getPluralism: function(word, count){
		        return word += (count != 1 ) ? "s" : "";
	        }
        });
    }

);