require "page-object"
require "spec_helper"
require "./spec/spec_helper"

class VeteranAppointments
	include PageObject

	PageObject.javascript_framework = :jquery

	attr_accessor :hash, :id, :skip_landing_check

	def initialize(driver, id, kwargs = {})
		super(driver)
		@driver = driver
		@id 	= id
		@hash   = kwargs[:hash] || ""
		@protocol = "http"
		@host 	= "localhost"
		@app 	= "/veteran-appointment-requests"
		@port 	= "8080"
		@skip_landing_check = false
		self.class.button(:landing_login_btn, :css=>"button#login")
		self.class.div(:main, :id=>"main")
		self.class.element(:date_picker, "section", :id=>'date-picker')
		self.class.button(:date_picker_set, :id=>'set-btn')
		self.class.link(:date_picker_close, :css=>'#date-picker a.cancel')
		default_element_wait=(kwargs[:timeout] || 5)
		max_width, max_height = @driver.execute_script("return [window.screen.availWidth, window.screen.availHeight];")
		@driver.manage().window().move_to(10,10)
		@driver.manage().window().resize_to((kwargs[:max_width] || ( max_width/4)),max_height)
	end

	def landing_login
		self.landing_login_btn if self.landing_login_btn?
	end

	def launch_home
		@driver.get(self.app_url) 
		if not @skip_landing_check
			begin
				self.landing_login_btn_element.wait_until(10) do
					self.landing_login_btn_element.visible?
				end
				self.landing_login_btn
				self.landing_login_btn_element.wait_until(5) do
					not self.landing_login_btn_element.visible?
				end
			rescue
			end
		end
		wait_until(10){
			main_element.visible?
		}
	end

	def launch_page
		@driver.get(self.app_url + "/#" + self.hash)
	end

	def find(name, how, limit = 60)
		counter = 0
		ele = element(name, how)
		while ele.nil?
			ele = element(name,how)
			sleep 1
			counter += 1
			break if counter >= limit
		end
		ele
	end

	def elementLocation(target)
		self.driver.find_element(:name, target).location
	end

	def exists?(how, what)
		element_present?(how, what)
	end

	def element_enabled?(element)
		disabled = element.attribute("aria-disabled")
		disabled = element.attribute("data-disabled") if disabled.nil?
		disabled = element.attribute("disabled") if disabled.nil?
		if disabled.nil? or disabled.empty?
			true
		else
			disabled.match(/(true|disabled)/).nil?
		end
	rescue
		true
	end

	def wait_until_enabled(element)
		wait_until(10){
			enabled = element_enabled?(element)
			if enabled and block_given?
				yield element
				break
			end
		}
	end

	def format_date(time)
		time.strftime("%m/%d/%Y")
	end

	def wait_for_va_ajax
		wait_for_ajax
	end

	def setOptionDate(id, option_date)
		date_jquery_script = "$('##{id}').trigger('datebox', {
									'method': 'set',
									'value': #{option_date}
								}).trigger('datebox', {
									'method': 'doset'
								});"
		self.execute_script(date_jquery_script)
		sleep 1
	end 

	def add_day
		60*60*24
	end

	def tomorrow
		Time.new + add_day
	end

	def tomorrow_str
		date_as_str tomorrow
	end

	def days90
		Time.new + (add_day*90)
	end

	def days120
		Time.new + (add_day * 120)
	end

	def add_days hash = {}
		(hash[:date] || Time.new) + (add_day * ( hash[:days] || 0) )
	end

	def date_as_str date = Time.new
		date.strftime("%m/%d/%Y")
	end

	def populate_date_picker element, date = self.tomorrow
		set_date_picker = "$('##{element.attribute('id')}').val('#{self.date_as_str(date)}')"
		self.execute_script(set_date_picker)
	end

	def populate_date_picker_show element, date = self.tomorrow
		set_date_picker = "$('##{element.attribute('id')}').val('#{self.date_as_str(date)}')"
		self.execute_script(set_date_picker)
		self.open_date_picker(element)
		# click_set_date
	end

	def open_date_picker element
		element.click
	end

	def click_set_date
		execute_script("var args = arguments[0]; $(args).click();", date_picker_set.element)
	end

	protected
		def root_url
			@protocol + "://" + @host + ":" + @port
		end

		def app_url
			self.root_url + @app
		end

end