require "selenium-webdriver"
require "rspec"
require "./pages/veteran_login"
require "./pages/veteran_appointment_request"
require "./pages/about_view"
require "./pages/user_menu"
require 'rubygems'
include RSpec::Expectations

describe "Veteran Appointment Requests - About Page" do
	
	before(:all) do
	    @driver = Selenium::WebDriver.for :firefox

	    @login = VeteranLogin.new(@driver)
	    @login.loginAsPatient66()

	    @about = About.new @driver
	    @user_menu = UserMenu.new @driver
	    @about.skip_landing_check = true
	    @user_menu.userMenu_button
	    @user_menu.about_button

		puts "About Page Tests"
	end

	after(:all) do
		@driver.save_screenshot("./landingFailure.png")
		@login.logout
		@driver.quit
	end

	it "has the specific dialog content section" do
		expect(@about.popup_specific_content?).to be true
	end

	it "has a title" do
		expect(@about.popup_title).to eq("About")
	end

	it "has the app version displayed" do
		@about.app_version_element.when_visible(5) do
			expect(@about.app_version).to match(/Appointment Requests v\d+\.\d+\.\d+/)
		end
	end

	it "contains about text" do
		expect(@about.popup_content_element.text.length).to be > 0
	end

	describe "About Page has functional buttons" do

		before(:all) do
			@about.launch_home
		end

		before(:each) do
			@user_menu.wait_for_ajax(30)
			@user_menu.userMenu_button
			@user_menu.about_button
		end

		it "has an OK button to exit about dialog" do
			expect(@about.ok_button?).to eq true
			@about.ok_button_element.scroll_into_view
			@about.ok_button
			@about.about_element.when_not_visible(5) do 
				expect(@about.about_element).not_to be_visible
			end
		end
	end

end

