#This file tests the VA staff Veteran Appointment Request application.
#

require "selenium-webdriver"
require "rspec"
require "./pages/veteran_login"
require "./pages/request_listing"
require "./pages/appointment_request_details"
require "rubygems"
include RSpec::Expectations


describe "Veteran Appointment Request: Appointment Request Details" do

	before(:all) do

		@driver = Selenium::WebDriver.for :firefox
		
		@login = VeteranLogin.new(@driver)
		@login.loginAsPatient66()

		@requestListing = RequestListing.new @driver
		@requestDetails = AppointmentRequestDetails.new @driver

		@requestListing.launch_home

		puts "Appointment Request Details"
	end

	after(:all) do
		@driver.save_screenshot("./newAppointmentRequestFailure.png")
		@login.logout		
		@driver.quit
	end


	describe "Booked Appointments Details" do
		before(:each) do
			@requestListing.launch_page
			@requestListing.open_appt_by_status "Booked"
		end

		it "has a status of ...to Booked" do
			if @requestListing.found
				expect(@requestDetails.request_status.downcase).to include("status changed to booked")
			end
		end

	end

	describe "Not Booked Appointments Details" do

		before(:each) do
			@requestListing.launch_page
			@requestListing.open_appt_by_status "Not Booked"
			@requestDetails.wait_for_va_ajax
		end

		it "has a status of ... to Not Booked" do
			if @requestListing.found
				expect(@requestDetails.request_status.downcase).to include("status changed to not booked")
			end
		end

		it "does not have Appointments button" do 
			if @requestListing.found
				expect(@requestDetails.appointments?).to be false
			end
		end

	end

	describe "Additional Request Details - Messaging" do

		before(:each) do
			@requestListing.launch_page
		end

		it "states that messaging is limited to 4 messages" do
			@requestListing.open_appt_by_status "Not Booked"
			if @requestListing.found
				@requestDetails.wait_for_va_ajax
				expect(@requestDetails.new_message).to include("This exchange is limited to four (4) messages")
			end
		end

		it "states that Appointment has been Cancelled" do
			@requestListing.open_appt_by_status "Cancelled"
			if @requestListing.found
				@requestDetails.wait_for_va_ajax
				expect(@requestDetails.message_element.attribute("placeholder")).to include("been cancelled")
			end
		end

		it "does not allow Sending message without message" do
			@requestListing.open_appt_by_status "Submitted"
			if @requestListing.found
				@requestDetails.wait_for_va_ajax
				expect(@requestDetails).not_to be_element_enabled(@requestDetails.send_message_element)
			end
		end

		it "allows sending of message when message entered" do
			@requestListing.open_appt_by_status "Submitted"
			if @requestListing.found
				@requestDetails.wait_for_va_ajax
				if @requestDetails.message_element.attribute("placeholder").include?("Enter Text Here...")
					@requestDetails.message = "Help me provider!"
					@requestDetails.back_btn_element.focus
					expect(@requestDetails).to be_element_enabled(@requestDetails.send_message_element)
				end
			end
		end

		it "shows messages from the VA when applicable" do
			@requestDetails.statuses.each do |status|

				@requestListing.launch_page
				@requestListing.open_appt_by_status status
				@requestDetails.wait_for_va_ajax

				if @requestListing.found
					@requestDetails.received_elements.each do |message|
						expect(message).to be_visible
					end
				end

			end
		end

		it "shows messages from the Veteran when applicable" do
			@requestDetails.statuses.each do |status|

				@requestListing.launch_page
				@requestListing.open_appt_by_status status
				@requestDetails.wait_for_va_ajax

				if @requestListing.found
					@requestDetails.sent_elements.each do |message|
						expect(message).to be_visible
					end
				end

			end
		end

	end

	describe "Navigate back to request listing" do

		it "allows user to go back to request listing from details" do 
			@requestDetails.statuses.each do |status|

				@requestListing.launch_page
				@requestListing.open_appt_by_status status
				@requestDetails.wait_for_va_ajax

				if @requestListing.found
					expect(@requestDetails.back_btn?).to be true
					@requestDetails.back_btn
					expect(@requestDetails.page_title).to eq "Notifications"
				end

			end
		end

	end



end