#This file tests the VA staff Veteran Appointment Request application.
#

require "selenium-webdriver"
require "rspec"
require "./pages/veteran_login"
require "./pages/ds_facility_view"
require "./pages/ds_cancel_appt_view"
require "rubygems"
include RSpec::Expectations


describe "Direct Scheduling - Cancel Appointment" do

	before(:all) do

		@driver = Selenium::WebDriver.for :firefox

		@login = VeteranLogin.new(@driver)
		@login.loginAsPatientEleven()

		@dsFacility = DSFacilityView.new @driver
		@dsCancel = DSCancelApptView.new @driver
		
		puts "Direct Scheduling - Cancel Appointment Tests"
	end

	after(:all) do
		@driver.save_screenshot("./directScheduling_CancelApptFailure.png")
		@login.logout		
		@driver.quit
	end

	describe "Don't cancel - Veteran decides to leave page" do

		before(:all) do
			@dsFacility.launch_page
			@dsFacility.select_first_facility
			@dsFacility.cancel_appt
		end

		it "has a title of Cancel Appointment" do
			expect(@dsCancel.page_title).to eq "Cancel Appointment"
		end

		it "allows user to click Back button" do
			@dsCancel.back_btn
			expect(@dsCancel.page_title).not_to eq "Cancel Appointment"
		end

	end

	describe "Cancel Appointment" do

		before(:all) do
			@dsFacility.launch_page
			@dsFacility.select_first_facility
			@dsFacility.cancel_appt	
		end

		it "allows selection of an appointment" do
			@dsCancel.select_first_appointment
			expect(@dsCancel.selected).to be true
		end

		it "prohibits cancellation until appointment and reason are selected" do
			expect(@dsCancel.element_enabled?(@dsCancel.cancel_btn_element)).to be false
		end

		it "allows selection of a reason" do
			@dsCancel.select_reason
			expect(@dsCancel.reason_options).to include(@dsCancel.reason)
		end

		it "allows user to enter a note and track remaining characters" do
			theNote = "I'm not feeling well."
			maxLength = @dsCancel.notes_element.attribute("maxlength").to_i
			@dsCancel.notes = theNote

			expect(@dsCancel.remaining).to include((maxLength - theNote.length).to_s)
			expect(@dsCancel.notes).to eq theNote
		end

		it "allows user to cancel appointment when appointment and reason are selected" do
			@dsCancel.wait_until_enabled(@dsCancel.cancel_btn_element){
				expect(@dsCancel.element_enabled?(@dsCancel.cancel_btn_element)).to be true
				@dsCancel.cancel_btn
			}
			expect(@dsCancel.page_title).not_to eq "Cancel Appointment"
		end

	end


end