#This file tests the VA staff Veteran Appointment Request application.
#

require "selenium-webdriver"
require "rspec"
require "./pages/veteran_login"
require "./pages/ds_facility_view"
require "./pages/ds_provider_view"
require "./pages/ds_month_view"
require "./pages/ds_day_view"
require "rubygems"
include RSpec::Expectations


describe "Direct Scheduling - Day View" do

	before(:all) do

		@driver = Selenium::WebDriver.for :firefox

		@login = VeteranLogin.new(@driver)
		@login.loginAsPatientEleven()

		@dsFacility = DSFacilityView.new @driver
		@dsProvider = DSProviderView.new @driver
		@dsMonth = DSMonthView.new @driver
		@dsDay 	= DSDayView.new @driver


		puts "Direct Scheduling - Day View Tests"

		@dsFacility.launch_page
		@dsFacility.select_first_facility
		@dsFacility.book_appt
		@dsProvider.populate_form_and_find
		@dsMonth.select_available_day
	end

	after(:all) do
		@driver.save_screenshot("./directScheduling_CancelApptFailure.png")
		@login.logout		
		@driver.quit
	end

	it "shows a header" do
		expect(@dsDay.page_title?).to be true
	end

	it "has clickable navigation bar" do 
		initialDay = @dsDay.day_title
		@dsDay.next_day
		selectedDay = @dsDay.day_title
		expect(initialDay).not_to eq(selectedDay)
		@dsDay.prev_day
		selectedDay = @dsDay.day_title
		expect(initialDay).to eq(selectedDay)
	end

	it "shows the facility name and clinic name at the bottom of the calendar day view" do
		expect(@dsMonth.facility_provider_info?).to be true
	end
end