#This file tests the VA staff Veteran Appointment Request application.
#

require "selenium-webdriver"
require "rspec"
require "./pages/veteran_login"
require "./pages/ds_facility_view"
require "./pages/ds_provider_view"
require "./pages/ds_month_view"
require "./pages/app_menu"
require "./pages/book_landing_page"

require "rubygems"
include RSpec::Expectations


describe "Direct Scheduling - Capture Desired Appointment Date" do

	before(:all) do

		@driver = Selenium::WebDriver.for :firefox

		@login = VeteranLogin.new(@driver)
		@login.loginAsPatientEleven()

		@dsFacility = DSFacilityView.new @driver
		@dsProvider = DSProviderView.new @driver
		@dsMonth 	= DSMonthView.new @driver
		@appMenu 	= AppMenu.new @driver
		@bookLanding = BookLandingPage.new @driver

		@dsFacility.skip_landing_check = true

		puts "Direct Scheduling - Desired Date Tests"

	end

	after(:all) do
		@driver.save_screenshot("./directScheduling_CaptureDesiredAppointmentDateFailure.png")
		@login.logout		
		@driver.quit
	end

	before(:each) do
		@appMenu.appMenu_button
		@appMenu.schedule_appt
		@bookLanding.sch_appt
		@appMenu.wait_for_ajax(90)
		@dsFacility.select_facility
		@dsFacility.book_appt
	end

	it "allows user to click the date input and pops up a date picker" do
		@dsProvider.populate_date_picker_show(@dsProvider.desired_date_element)
		expect(@dsProvider.date_picker_element).to be_visible
	end

	it "starts at date specified from previous value if entered" do
		@dsProvider.populate_date_picker_show(@dsProvider.desired_date_element, @dsProvider.tomorrow)
		@dsProvider.date_picker_set
		expect(@dsProvider.desired_date).to eq @dsProvider.tomorrow_str
	end

	it "limits user to 90 days into future" do
		too_far_date = @dsProvider.days120
		@dsProvider.populate_date_picker_show(@dsProvider.desired_date_element, too_far_date)
		@dsProvider.date_picker_set
		expect(too_far_date).to be > @dsProvider.days90
		expect(@dsProvider.desired_date).to eq @dsProvider.date_as_str(@dsProvider.days90)
	end

	it "limits user to tomorrow as earliest available date" do
		past_date = @dsProvider.add_days({:days=>-5})
		@dsProvider.populate_date_picker_show(@dsProvider.desired_date_element, past_date)
		@dsProvider.date_picker_set
		expect(past_date).to be < @dsProvider.tomorrow
		expect(@dsProvider.desired_date).to eq @dsProvider.date_as_str(@dsProvider.tomorrow)
	end

end