#This file tests the VA staff Veteran Appointment Request application.
#

require "selenium-webdriver"
require "rspec"
require "./pages/veteran_login"
require "./pages/ds_facility_view"
require "./pages/ds_provider_view"
require "./pages/ds_month_view"
require "./pages/app_menu"
require "./pages/book_landing_page"

require "rubygems"
include RSpec::Expectations

describe "Direct Scheduling - Month View" do

	before(:all) do

		@driver = Selenium::WebDriver.for :firefox

		@login = VeteranLogin.new(@driver)
		@login.loginAsPatientEleven()

		@dsFacility = DSFacilityView.new @driver
		@dsProvider = DSProviderView.new @driver
		@dsMonth 	= DSMonthView.new @driver
		@appMenu 	= AppMenu.new @driver
		@bookLanding = BookLandingPage.new @driver

		puts "Direct Scheduling - Month View Tests"

		@appMenu.appMenu_button
		@appMenu.schedule_appt
		@bookLanding.sch_appt
		@appMenu.wait_for_ajax(90)
		@dsFacility.select_facility
		@dsFacility.book_appt
		@dsProvider.populate_form_and_find
		@dsProvider.wait_for_ajax(60)
	end

	after(:all) do
		@driver.save_screenshot("./directScheduling_CancelApptFailure.png")
		@login.logout		
		@driver.quit
	end

	it "has a title of Schedule My Appointment" do
		expect(@dsMonth.page_title).to eq "Schedule My Appointment"
	end

	it "has an instructional header with text" do
		expect(@dsMonth.instruction).to eq "Select a day with available appointments"
	end

	it "has a subheader with calendar controls" do
		expect(@dsMonth.sub_header_element).to be_visible
	end

	it "allows user to see info when clicking on info button" do
		expect(@dsMonth.info_box_element).to be_visible
	end

	it "has a subheader with the current Month/Year" do
		expect(@dsMonth.date_display_element).to be_visible
		expect(@dsMonth.date_display).to eq @dsMonth.tomorrow.strftime("%b %Y")
	end

	it "allows the user to scroll month to month" do
		start = @dsMonth.month_abbr
		@dsMonth.next_month
		expect(@dsMonth.date_display).not_to include start
		@dsMonth.prev_month
		expect(@dsMonth.date_display).to include start
	end

	it "limits user present and 90 days in future" do
		range_start = @dsMonth.month_range_start
		puts "Range start #{range_start}"
		puts "Rand avail #{@dsMonth.month_available_range}"
		expect(@dsMonth.prev_month_element).not_to be_visible if range_start == 0

		(range_start..@dsMonth.month_available_range - 1).each do
			@dsMonth.next_month
		end
		expect(@dsMonth.element_enabled?(@dsMonth.next_month_element)).to be false
	end

	it "shows month view calendar" do
		expect(@dsMonth.month_days_elements.size).to be > 27
		expect(@dsMonth.month_days_elements.fetch(0)).to be_visible
	end

	it "shows available slots per day if available" do
		if not @dsMonth.available_days_elements.empty?
			expect(@dsMonth.available_days_elements.size).to be > 0
		end
	end

	it "shows a phone icon on current today indicating to call clinic for assistance" do
		@dsMonth.start_month
		expect(@dsMonth.call_clinic?).to be true
	end

	it "shows the facility name and clinic name at the bottom of the calendar view" do
		@dsMonth.start_month
		expect(@dsMonth.facility_provider_info?).to be true
	end

	xit "displays an icon on dates with existing appointments" do

	end
end