#This file tests Direct Scheduling Application

require "selenium-webdriver"
require "rspec"
require "./pages/veteran_login"
require "./pages/landing_page_not_logged_in"
require 'rubygems'
include RSpec::Expectations

describe "Veteran Appointment Request: VAR-490 Not Logged In Landing Page"  do	

	before(:all) do
		@driver = Selenium::WebDriver.for :firefox

    	@login = VeteranLogin.new(@driver)
		@landing = LandingPageNotLoggedIn.new(@driver, {:timeout=>2})
		@landing.skip_landing_check = true
		@landing.launch_home

	    puts "Landing Page - Not Logged In Tests"
	end

	after(:all) do
		@driver.save_screenshot("./landingNotLoggedInFailure.png")
		@login.logout
		@driver.quit
	end

	it "has a header" do
		expect(@landing.page_title).to eq("VA Appointments")
	end

	it "has an introduction to the app" do
		expect(@landing.intro).to include("Request Primary Care and Mental Health Appointments")
	end

	it "has the VA branding logo" do
		expect(@landing).to be_va_brand
	end

	it "has a login button" do
		expect(@landing).to be_login
	end

	it "has a login button that takes user to ssoeproxy login" do
		@landing.login
		expect(@landing.find(:loginForm, :id=>"logonForm")).to be_an_instance_of(PageObject::Elements::Element)
	end

	it "redirects to landing page logged in after logging in" do
    	@login.loginAsPatient66(true, true)
    	@landing.wait_until(10){
    		@landing.page_title_element.visible?
    	}
    	expect(@landing.page_title).to eq("VA Appointments")
	end

end