#This file tests Direct Scheduling Application

require "selenium-webdriver"
require "rspec"
require "./pages/veteran_login"
require "./pages/landing_page_logged_in"
require 'rubygems'
include RSpec::Expectations

describe "Veteran Appointment Request: Story# VAR-432" do

  before(:all) do

    @driver = Selenium::WebDriver.for :firefox

    @login = VeteranLogin.new(@driver)
    @login.loginAsPatient66()

    @landingLoggedIn = LandingPageLoggedIn.new(@driver, {:timeout=>2})

    puts "Landing Page Tests"

  end

  after(:all) do
    @driver.save_screenshot("./landingFailure.png")
    @login.logout
    @driver.quit
  end

  it "has a title of VA Appointments" do
    expect(@landingLoggedIn.page_title).to eq(@landingLoggedIn.landing_title)
  end

  it "has VA health branding : AC #VAR-439; Test #VAR-445" do
    expect(@landingLoggedIn).to be_va_brand
  end

  it "has new unread message icon : AC #VAR-436; Test #VAR-442" do
    expect(@landingLoggedIn.unread_message_icon_element).to be_visible if @landingLoggedIn.unread_message_icon?
  end

  it "has 4 options for veteran to choose" do
    expect(@landingLoggedIn.landing_page_options_links_elements.size).to eq 4
  end

  describe "Veteran Landing Page has accessible options" do

    it "allows user to access Schedule an Appointment" do
      expect(@landingLoggedIn.page_title).to eq(@landingLoggedIn.landing_title)
      expect(@landingLoggedIn.schedule_appointment?).to be true
    end

    it "allows user to access View or Cancel Appointments" do
      expect(@landingLoggedIn.page_title).to eq(@landingLoggedIn.landing_title)
      expect(@landingLoggedIn.view_appointments?).to be true
    end

    it "allows user to access Requests" do
      expect(@landingLoggedIn.page_title).to eq(@landingLoggedIn.landing_title)
      expect(@landingLoggedIn.requests?).to be true
    end    

    it "allows user to access Enroll in a PACT" do
      expect(@landingLoggedIn.page_title).to eq(@landingLoggedIn.landing_title)
      expect(@landingLoggedIn.enroll?).to be true
    end

  end

end





