#This file tests Direct Scheduling Application - navigation menus

require "selenium-webdriver"
require "rspec"
require "./pages/veteran_login"
require "./pages/landing_page_logged_in"
require "./pages/landing_page_not_logged_in"
require "./pages/app_menu"
require "./pages/user_menu"
require "./pages/feedback_view"
require "./pages/about_view"
require "./pages/settings_view"
require "./pages/appointment_request_details"
require "./pages/request_listing"



require 'rubygems'
include RSpec::Expectations

describe "Veteran Appointment Request: Story# VAR-504" do

  before(:all) do

    @driver = Selenium::WebDriver.for :firefox

    @login = VeteranLogin.new(@driver)
    @login.loginAsPatient66()

    @app_menu = AppMenu.new @driver
    @user_menu = UserMenu.new @driver
    @about_view      = About.new(@driver, {:timeout=>5})
    @feedback_view   = Feedback.new(@driver, {:timeout=>5})
    @settings_view   = Settings.new(@driver, {:timeout=>5})
    @request_listing = RequestListing.new @driver
    @appointment_request_details = AppointmentRequestDetails.new @driver
    @landing_page_logged_in = LandingPageLoggedIn.new @driver

    # @app_menu.launch_home
    @app_menu.skip_landing_check = true
    @user_menu.skip_landing_check = true

    puts "---Naviagtion Menu Tests---"

  end

  after(:all) do
    @driver.save_screenshot("./navFailure.png")
    @login.logout
    @driver.quit
  end


  describe "Left Navigation Menu" do

    before(:each) do
      @app_menu.appMenu_button
    end

    after(:each) do
      @app_menu.appMenu_button
      @app_menu.wait_until(3){
        not @app_menu.appMenu_panel_element.visible?
      }
    end

    it " App Menu - Home button : AC #VAR556; #VAR-557" do

      expect(@app_menu.menu_header_element).to be_visible
      expect(@app_menu.home_button?).to be true

    end

    it " App Menu - Schedule VA Appt" do

      expect(@app_menu.menu_header_element).to be_visible
      expect(@app_menu.schedule_appt?).to be true

    end

    it " App Menu - View or Cancel Appointments" do

      expect(@app_menu.menu_header_element).to be_visible
      expect(@app_menu.view_appt?).to be true

    end

    it " App Menu - Requests" do

      expect(@app_menu.menu_header_element).to be_visible
      expect(@app_menu.requests?).to be true

    end

  end


  describe "Right Navigation Menu" do

    before(:each) do
        @user_menu.wait_for_ajax(10)
        @user_menu.wait_until(10){
          @user_menu.userMenu_button?
        }
        @user_menu.userMenu_button
    end

    after(:each) do
      @user_menu.launch_home
    end

    it "User Menu - About : AC #VAR-559; Test #VAR-560" do

      expect(@user_menu.menu_header_element).to be_visible
      @user_menu.about_button
      @about_view.wait_until(10){
        @about_view.ok_button_element.visible?
      }
      expect(@about_view.about_element).to be_visible

    end

    it "User Menu - Feedback : AC #VAR-559; Test #VAR-560" do
      expect(@user_menu.menu_header_element).to be_visible
      @user_menu.feedback_button
      @feedback_view.wait_until(10){
        @feedback_view.popup_element.visible?
      }
      expect(@feedback_view.popup_title_element).to be_visible

    end
  
  end

  # describe "Requests Sub Menu" do

  #   before(:each) do
  #     @request_listing.launch_page
  #   end

  #   it "Request - New Request Button : AC #VAR-561; Test #VAR-562" do
   
  #     expect(@request_listing.page_title).to eq("Requests")
  #     @request_listing.new_req_btn_element.visible?

  #   end

  #   it "Request - Back Button : AC #VAR-561; Test #VAR-562" do
 
  #     expect(@request_listing.page_title).to eq("Requests")
  #     @request_listing.open_appt_by_status("Submitted")
  #     if @request_listing.found
  #       @appointment_request_details.wait_until{
  #         @appointment_request_details.back_btn_element.visible?
  #       }
  #       expect(@appointment_request_details.back_btn_element).to be_visible
  #     end
  #   end

  #   it "Request - Cancel Button : AC #VAR-561; Test #VAR-562" do

  #     expect(@request_listing.page_title).to eq("Requests")
  #     @request_listing.open_appt_by_status("Submitted")
  #     if @request_listing.found
  #       @appointment_request_details.wait_until{
  #         @appointment_request_details.cancel_btn?
  #       }
  #       expect(@appointment_request_details.cancel_btn_element).to be_visible
  #     end
  #   end

  # end

end


