#This file tests the VA staff Veteran Appointment Request application.
#

require "selenium-webdriver"
require "rspec"
require "./pages/veteran_login"
require "./pages/landing_page_not_logged_in"
require "./pages/landing_page_logged_in"
require "./pages/new_appointment_request_phone"
require "./models/appointment_request"
require "rubygems"
include RSpec::Expectations


describe "Veteran Appointment Request: New Appointment Request By Phone" do

  before(:all) do

    @driver = Selenium::WebDriver.for :firefox

    @login = VeteranLogin.new(@driver)
    @username = @login.loginAsPatientEleven()

    AppointmentRequest.new({:username => @username}).clear_past_requests

    @newApptRequest = NewAppointmentRequestPhone.new(@driver, {:timeout=>10})

    puts "New Appointment Request By Phone - Tests"

  end
  
  after(:all) do
  	@driver.save_screenshot("./newAppointmentRequestByPhoneFailure.png")
    @login.logout
    @driver.quit
  end

  describe "Cancel out of a new Request" do

    before(:all) do
      @newApptRequest.launch_page
    end

    it "has a Cancel button" do
      @newApptRequest.cancel_request_btn_element.when_visible(10)
      expect(@newApptRequest.cancel_request_btn_element).to be_visible
    end

    it "allows user to cancel/exit the new request" do
      @newApptRequest.cancel_request_btn
      expect(@newApptRequest.page_title).not_to eq "New Request"
    end

  end


  describe "Create a new request" do

    before(:all) do
      # @newApptRequest.launch_home
      @newApptRequest.launch_page
      @newApptRequest.populateForm
    end
    
    it "has a title heading of New Request" do
      expect(@newApptRequest.page_title).to eq("New Request")
    end

    it "prohibits user from reviewing when a required field is blank" do
      @newApptRequest.phone_number= ''
      @newApptRequest.review_request_btn
      expect(@newApptRequest.page_title).to eq("New Request")
      @newApptRequest.phone_number= '555-555-8888'
    end

    it "allows user to review request when all required fields completed" do
      @newApptRequest.phone_number_lbl_element.click
      @newApptRequest.review_request_btn
      expect(@newApptRequest.review_request_element).to be_visible
    end 
   
   end
end