#This file tests the VA staff Veteran Appointment Request application.
#

require "selenium-webdriver"
require "rspec"
require "./pages/veteran_login"
require "./pages/landing_page_not_logged_in"
require "./pages/landing_page_logged_in"
require "./pages/new_appointment_request"
require "./models/appointment_request"
require "rubygems"
include RSpec::Expectations


describe "Veteran Appointment Request: New Appointment Request Story# VAR-306" do

  before(:all) do

    @driver = Selenium::WebDriver.for :firefox

    @login = VeteranLogin.new(@driver)
    @username = @login.loginAsPatientEleven()

    AppointmentRequest.new({:username => @username}).clear_past_requests

    @newApptRequest = NewAppointmentRequest.new(@driver, {:timeout=>10})
    # @newApptRequest.launch_home

    puts "New Appointment Request Tests"

  end
  
  after(:all) do
  	@driver.save_screenshot("./newAppointmentRequestFailure.png")
    @login.logout
    @driver.quit
  end

  describe "Cancel a new Request" do

    before(:all) do
      @newApptRequest.launch_page
    end

    it "has a Cancel button" do
      @newApptRequest.cancel_request_btn_element.when_visible(10)
      expect(@newApptRequest.cancel_request_btn_element).to be_visible
    end

    it "allows user to cancel/exit the new request" do
      @newApptRequest.cancel_request_btn
      expect(@newApptRequest.page_title).not_to eq "New Request"
    end

  end


  describe "Create a new request" do

    before(:all) do
      # @newApptRequest.launch_home
      @newApptRequest.launch_page
      @newApptRequest.populateForm
    end
    
    it "has a title heading of New Request" do
      expect(@newApptRequest.page_title).to eq("New Request")
    end

    it "allows user to enter Other Purpose of Visit when visit type is Other and hides when not" do
      @newApptRequest.purpose_of_visit = 'Other'
      @newApptRequest.wait_until(10){
        @newApptRequest.other_purpose_of_visit_element.visible?
      }
      expect(@newApptRequest.other_purpose_of_visit_element).to be_visible

      @newApptRequest.purpose_of_visit = 'Routine follow-up'
      @newApptRequest.wait_until(10){
        not @newApptRequest.other_purpose_of_visit_element.visible?
      }
      expect(@newApptRequest.other_purpose_of_visit_element).not_to be_visible
    end

    xit "changes the type of visit to phone call when purpose of visit is medication concern and type of care is Primary" do
      expect(@newApptRequest.phone_c_selected?).to be false
      expect(@newApptRequest.primary_c_selected?).to be true
      @newApptRequest.purpose_of_visit= 'Medication concern'
      expect(@newApptRequest.phone_c_selected?).to be true
    end

    xit "does not change the type of visit to phone call when purpose of visit is medication concern and type of care is Mental" do
      @newApptRequest.office_visit_element.click
      @newApptRequest.mental_health_element.click
      @newApptRequest.purpose_of_visit = 'New issue'
      expect(@newApptRequest.phone_c_selected?).to be false
      expect(@newApptRequest.primary_c_selected?).to be false
      expect(@newApptRequest.purpose_of_visit).not_to eq('Medication concern')
      @newApptRequest.purpose_of_visit = 'Medication concern'
      expect(@newApptRequest.phone_c_selected?).to be false

      @newApptRequest.primary_care_element.click
      @newApptRequest.select_provider_index
    end

    it "display a caveat for Video Conference - Type of Visit" do
      @newApptRequest.video_conference_element.click
      expect(@newApptRequest.video_caveat_element).to be_visible
    end

    it "does not display caveat for when Type of Visit not Video Conference" do 
      @newApptRequest.office_visit_element.click
      expect(@newApptRequest.video_caveat_element).not_to be_visible
    end

    it "prohibits user from reviewing when a required field is blank" do
      @newApptRequest.phone_number= ''
      @newApptRequest.review_request_btn
      expect(@newApptRequest.page_title).to eq("New Request")
      @newApptRequest.phone_number= '555-555-8888'
    end

    it "allows user to review request when all required fields completed" do
      @newApptRequest.phone_number_lbl_element.click
      @newApptRequest.review_request_btn
      expect(@newApptRequest.review_request_element).to be_visible
    end 
   
   end
end