#This file tests the VA staff Veteran Appointment Request application.
#

require "selenium-webdriver"
require "rspec"
require "./pages/veteran_login"
require "./pages/request_listing"
require "rubygems"
include RSpec::Expectations


describe "Veteran Appointment Request: Request Listing & Notification Settings" do

	before(:all) do

		@driver = Selenium::WebDriver.for :firefox

		@login = VeteranLogin.new(@driver)
		@login.loginAsPatient66()

		@requestListing = RequestListing.new @driver
		@requestListing.launch_page
		@requestListing.wait_for_ajax(60)

		puts "Request Listing & Notification Settings"
	end

	after(:all) do
		@driver.save_screenshot("./requestListingFailure.png")
		@login.logout		
		@driver.quit
	end

	it "notifies user if new message on a request with title 'New Unread Message(s)'" do
		if @requestListing.new_message_icon?
			expect(@requestListing.new_message_icon_element).to be_visible 
			expect(@requestListing.new_message_icon_element.attribute("title")).to eq "New Unread Message(s)"
		end
	end

	it "filters the requests" do
		@requestListing.status_filter_options.each do |status|
			@requestListing.status_filter = status
			expect(@requestListing.compare_appt_status(status)).to eq true
		end
	end

	it "has a default notifications setting of Log in..." do
		expect(@requestListing.default_pref).to include "Log in to the app"
	end

	it "allows user to set notifications settings" do
		expect(@requestListing.element_enabled?(@requestListing.save_prefs_element)).to be false
		@requestListing.email_updates_element.click
		if @requestListing.email_element.visible?
			expect(@requestListing.email_element).to be_visible
			@requestListing.email = 'someguy@domain'
			@requestListing.default_pref_element.click
			expect(@requestListing.element_enabled?(@requestListing.save_prefs_element)).to be true
		else
			expect(@requestListing.email_element).to_not be_visible
		end
	end

end