require "selenium-webdriver"
require "rspec"
require "./pages/veteran_login"
require "./pages/settings_view"
require "./pages/new_appointment_request"
require "./pages/user_menu"
require 'rubygems'
include RSpec::Expectations

describe "Veteran Appointment Requests - Settings Page" do
	
	before(:all) do
	    @driver = Selenium::WebDriver.for :firefox

	    @login = VeteranLogin.new @driver
	    @login.loginAsPatient66()

	    @settings = Settings.new(@driver, {:max_width=>600})
	    @user_menu = UserMenu.new @driver

	    @user_menu.userMenu_button
	    @user_menu.settings_button

		puts "Settings Page Tests"
	end

	after(:all) do
		@driver.save_screenshot("./settingsFailure.png")
		@login.logout
		@driver.quit
	end

	it "has a title" do
		expect(@settings.popup_title).to eq('Notification Preferences')
	end

	it "has text messaging question" do
		expect(@settings.text_messages).to include('Text messages')
	end

	it "has email messaging question" do
		expect(@settings.email_messages).to include('Email messages')
	end

	it "allows submit when text and email is NO" do
		@settings.set_text_messages(false)
		@settings.set_email_messages(false)
		expect(@settings).to be_element_enabled(@settings.submit_element)
	end

	it "allows submit when text message is YES and has a phone number" do
		@settings.populate_phone_number	
		@settings.wait_until(10){
			@settings.submit_element.visible?
		}
		expect(@settings).to be_element_enabled(@settings.submit_element)
		@settings.set_text_messages(false)
	end

	it "allows submit when email message is YES and has email" do
		@settings.populate_email
		@settings.submit_element.when_visible(10){
			expect(@settings).to be_element_enabled(@settings.submit_element)
		}
		@settings.set_email_messages(false)
	end

	it "allows submit of settings" do
		@settings.wait_until(10){
			@settings.element_enabled?(@settings.submit_element)
		}
		expect(@settings).to be_element_enabled(@settings.submit_element)
	end

end

