define(['Global', 'Vent', 'DSAppointmentListingItem', 'BookedAppointment', 'text!../html/dsAppointmentListingItem.html', 'text!../html/dsAppointmentListingItemCancel.html'],
	function(Global, vent, DSAppointmentListingItem, BookedAppointment, template, cancellable){
		"use strict";

		var appointmentsListItemView = new DSAppointmentListingItem({model: new BookedAppointment({
			clinic: {
				name: "Primary Care",
				id: ""
			},
			appointmentTime: "09/23/2014 13:00",
			appointmentDesignation: "PRIMARY_CARE"
		}, {parse: true})});

		appointmentsListItemView.facility = new Backbone.Model({
			facilityName: "TEST VAMC 3"
		});
		appointmentsListItemView.render();

		describe("Appointments List Item", function() {
			beforeEach(function () {
				spyOn(vent, 'trigger');
			});

			it('returns to the direct scheduling screen', function(){
				appointmentsListItemView.cancel();

				expect(vent.trigger).toHaveBeenCalledWith('show:direct-scheduling:cancel', appointmentsListItemView.model, appointmentsListItemView.facility);
			});

			it('has a template helper to get the facility name', function () {
				expect(appointmentsListItemView.templateHelpers().getFacility()).toEqual('TEST VAMC 3');
			});

			it('has a template helper to get the clinic name', function () {
				expect(appointmentsListItemView.templateHelpers().getClinic()).toEqual('Primary Care');
			});

			it('has a template helper to format the date', function () {
				expect(appointmentsListItemView.templateHelpers().getAppointmentDate()).toEqual('Tuesday, September 23, 2014 @ 13:00')
			});

			it('returns the correct template', function () {
				_.template = function (template) { return template };

				expect(appointmentsListItemView.getTemplate()).toEqual(cancellable);

				appointmentsListItemView.model.set({
					appointmentDesignation: "OTHER"
				});

				expect(appointmentsListItemView.getTemplate()).toEqual(template);

				appointmentsListItemView.model.set({
					appointmentDesignation: "PRIMARY_CARE"
				});

				appointmentsListItemView.facility.set({
					nonPACT: true
				});

				expect(appointmentsListItemView.getTemplate()).toEqual(template);
			});
		});
	}
);