define(['Global', 'Vent', 'DSCancelAppt'],
    function(Global, vent, DSCancelAppt){
        "use strict";

        var appt = new Backbone.Model({
	            clinic: {name:'Clinic'},
	            providerName: 'Provider',
	            start: '09/14/2014 09:00:00'
	        }),
	        view = new DSCancelAppt({
	            collection: new Backbone.Collection(),
	            model: new Backbone.Model({
		            cancelReasonsList:[
		                {
		                    text: "weather",
			                number: "1"
	                    }
	                ]
	            })
	        });

        view.facility = new Backbone.Model({facilityName: 'Facility'});
        view.appt = appt;

        view.render();
        
        describe('Direct Scheduling Cancel View', function() {

	        describe('Helper Functions', function () {
		        it('gets the pluralism for most words', function () {
			        expect(view.getPluralism('thing', 5)).toEqual('things');
			        expect(view.getPluralism('thing', 1)).toEqual('thing');
		        });
	        });

	        describe('View updaters', function () {
				it('updates the remaining limit for the notes section', function () {
					var text = "text";

					//Empty textarea
					view.ui.notes.val('');

					view.updateRemainingLimit();

					expect(view.ui.remaining.text()).toEqual('160 characters');

					//Textarea with some text
					view.ui.notes.val(text);

					view.updateRemainingLimit();

					expect(view.ui.remaining.text()).toEqual('156 characters');

					//Only one character left
					text = "123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789";

					view.ui.notes.val(text);

					view.updateRemainingLimit();

					expect(view.ui.remaining.text()).toEqual('1 character');

					//Full textarea
					text += "0";

					view.ui.notes.val(text);

					view.updateRemainingLimit();

					expect(view.ui.remaining.text()).toEqual('0 characters');

					//Negative textarea
					text += "1";

					view.ui.notes.val(text);

					view.updateRemainingLimit();

					expect(view.ui.remaining.text()).toEqual('-1 characters');
					expect(view.ui.notesRule.hasClass('warning')).toBeTruthy();
				});

		        it('enables/disables the cancel button based on whether a cancel reason is selected', function () {
			        view.ui.reason.val('');

			        view.enableCancelBtn();

			        expect(view.ui.cancel[0].disabled).toBeTruthy();


			        view.ui.reason.val('1');

			        view.enableCancelBtn();

			        expect(view.ui.cancel[0].disabled).toBeFalsy();
		        });
	        });
        	
        	describe('Vent triggers', function(){
        		beforeEach(function(){
        			spyOn(vent, 'trigger');
        		});

		        it('sends appt off for cancellation', function(){
		        	view.cancelAppt();

			        var args = vent.trigger.mostRecentCall.args,
				        event = args[0],
				        appt = args[1],
				        model = args[2],
				        facility = args[3];

			        expect(vent.trigger).toHaveBeenCalled();
			        expect(event).toEqual("do:cancel:my-appointment");
			        expect(appt).toEqual(view.appt);
			        expect(model.get('reason')).toEqual(view.ui.reason.val());
			        expect(model.get('notes')).toEqual(view.ui.notes.val());
			        expect(facility).toEqual(view.facility);
		        });

		        it('allows the user to go back to the appointment list', function () {
			        view.goBack();

			        expect(vent.trigger).toHaveBeenCalledWith("show:direct-scheduling:appointment-listing");
		        });
        	});

        	describe('Template Helpers', function(){

		        it('to get facility', function(){
		        	expect(view.templateHelpers().getFacility()).toBeDefined();
		        });

		        it('to get formatted appt date time', function(){
		        	expect(view.templateHelpers().getAppointmentDateTime()).toEqual('Sunday, September 14, 2014 @ 09:00');
		        });

		        it('to get Facility Clinic', function(){
		        	expect(view.templateHelpers().getFacilityClinic()).toEqual('Facility, Clinic');
		        });

		        it('to get provider name', function(){
		        	expect(view.templateHelpers().getProvider()).toEqual('Provider');
		        });

        	});

        });
    }
);