define(['Global', 'Vent', 'Layout', 'DSCancelApptConfirm'],
	function(Global, vent, Layout, DSCancelApptConfirm){
		'use strict';

		describe("Direct Scheduling - Cancel Appointment Confirmation View", function(){
			var view, appt;
			var layout = new Layout();
			layout.render();

			beforeEach(function(){
				appt = new Backbone.Model({
					start: "07/15/2014 15:00:00",
					facility: {
						facilityName: "TEST VAMC 3"
					},
					clinic: {
						name: "Primary Care"
					},
					reason: "None",
					notes: "Not needed"
				});
				view = new DSCancelApptConfirm({model:appt});
				layout.content.show(view);
				spyOn(vent, "trigger");
			});

			it("allows user to view the appointment that was cancelled", function(){
				var text = view.$el.text();
				expect(text).toContain(moment(new Date(appt.get("start"))).format('dddd, MMMM D, YYYY @ HH:mm'));
				expect(text).toContain(appt.get("facility").facilityName);
				expect(text).toContain(appt.get("clinic").name);
				expect(text).toContain(appt.get("reason"));
				expect(text).toContain(appt.get("notes"));
			});

			it("allows user to click ok to close out confirmation of cancellation", function(){
				view.$el.find("#ok-btn").trigger("click");
				expect(vent.trigger).toHaveBeenCalledWith("show:landing");
			});

			it("shows the email confirmation screen", function () {
				view.showVeteranEmailForm();
				expect(vent.trigger).toHaveBeenCalledWith("show:veteran-email-form", view.model);
			});
		});

	}
);