define(['Global', 'Layout', 'CalendarLegendView', 'Vent', 'CalendarDayNavView'],
	function(Global, Layout, CalendarLegendView, vent, CalendarDayNavView){
		"use strict";

		var selectedDate = moment().add(1, 'days'),
			model = new Backbone.Model({selectedDate: selectedDate}),
			calendarDayNavView = new CalendarDayNavView({model: model}),
			layout = new Layout(),
			legendView = new CalendarLegendView();

		describe("Calendar Day Nav", function(){

			beforeEach(function () {
				spyOn(vent, "trigger");

				layout.render();
				layout.$el.append("<div id='info-box'></div>");
				layout.content.show(calendarDayNavView);
				layout.infoBox.show(legendView);
			});

			it("has a function for clearing the time from a moment object", function(){
				var dateTime = calendarDayNavView.clearTime(calendarDayNavView.model.get('selectedDate'));

				calendarDayNavView.model.set('selectedDate', dateTime);

				expect(dateTime.format("HH:mm:ss:SSS")).toEqual("00:00:00:000");
			});

			it("starts with the previous button disabled and the next button enabled", function () {
				expect(calendarDayNavView.$el.find('#cal-prev-day')[0].disabled).toBeTruthy();
				expect(calendarDayNavView.$el.find('#cal-next-day')[0].disabled).toBeFalsy();
			});

			it("can go back to the month view", function () {
				calendarDayNavView.goBackToCalendar();

				expect(calendarDayNavView.model.get('alreadyFetched')).toBeTruthy();
				expect(vent.trigger).toHaveBeenCalledWith('show:ds-calendar-month:view');
			});

			it("can go to the next day", function () {
				var nextDay = calendarDayNavView.clearTime(moment().add(2, 'days'));

				calendarDayNavView.goToNextDay();

				expect(calendarDayNavView.model.get('selectedDate')).toEqual(nextDay.format('ddd, MMM DD, YYYY'));
				expect(calendarDayNavView.$el.find('#cal-prev-day')[0].disabled).toBeFalsy();
				expect(calendarDayNavView.$el.find('#cal-next-day')[0].disabled).toBeFalsy();
			});

			it("can go to the previous day, but not before tomorrow", function () {
				var minDate = calendarDayNavView.clearTime(moment().add(1, 'days'));

				calendarDayNavView.goToPrevDay();

				expect(calendarDayNavView.model.get('selectedDate')).toEqual(minDate.format('ddd, MMM DD, YYYY'));
				expect(calendarDayNavView.$el.find('#cal-prev-day')[0].disabled).toBeTruthy();
				expect(calendarDayNavView.$el.find('#cal-next-day')[0].disabled).toBeFalsy();
			});

			it("does not go further than 90 days", function () {
				var maxDate = calendarDayNavView.clearTime(moment().add(90, 'days'));

				for (var i = 0; i < 89; i++) {
					calendarDayNavView.goToNextDay();
				}

				expect(calendarDayNavView.model.get('selectedDate')).toEqual(maxDate.format('ddd, MMM DD, YYYY'));
				expect(calendarDayNavView.$el.find('#cal-prev-day')[0].disabled).toBeFalsy();
				expect(calendarDayNavView.$el.find('#cal-next-day')[0].disabled).toBeTruthy();
			});

		});
	}
);