define(['Global', 'Layout', 'CalendarMonthNavView', 'CalendarLegendView', 'CalendarView', 'CalendarData'],
	function(Global, Layout, CalendarMonthNavView, CalendarLegendView, CalendarView, CalendarData){
		'use strict';

		describe("Direct Scheduling - Month Nav View", function(){

			var go 			= false;
			var layout 		= new Layout();
			var monthNav 	= new CalendarMonthNavView();
			var legendView 	= new CalendarLegendView();
			var calendarView = new CalendarView();
			var selectedDate = moment().format('ddd, MMM DD, YYYY');
			var calData 	= new CalendarData({selectedDate: selectedDate});

			monthNav.model = calData;

			beforeEach(function(done){

				calData.today = moment('07/16/2014', 'MM/DD/YYYY');

				calData.fetch({
					url:'../data/clinicSlots.json',
					success:function(data){
						calendarView.model = calData;
						layout.render();

						legendView.render();
						layout.content.show(calendarView);
						layout.subHeader.show(monthNav);
						$('#info-box').empty().append(legendView.$el);

						go = true;
					}
				});

				waitsFor(function(){
					return go;
				}, "Should have received clinic slots", 2000);

			});

			it("has method to get short month", function(){
				expect(monthNav.getShortMonth(new Date("10/14/2014"))).toEqual("Oct 2014");
			});

			it("has a max date set for selection +90 days from today", function(){
				var today 	= moment();
				var max 	= moment();
				max.add(90, 'days');
				expect(moment(new Date(monthNav.maxDate)).format("MM/DD/YYYY")).toEqual(max.format("MM/DD/YYYY"));
			});

			it("allows user to cycle a 90 day date range from today", function(){
				var next = monthNav.$el.find("#cal-next-month"),
					display = monthNav.$el.find("h2"),
					prevMonthDisplay = display.text();
				while(next.is(":enabled")){
					monthNav.goToNextMonth();
					expect(display.text()).not.toEqual(prevMonthDisplay);
				}
				expect(next.is(":disabled")).toBe(true);
			});

			it("allows user to cycle back to current Month", function(){
				var prev = monthNav.$el.find("#cal-prev-month"),
					display = monthNav.$el.find("h2");
				while(prev.is(":enabled")){
					monthNav.goToPrevMonth();
				}
				expect(display.text().trim()).toEqual(display.text().trim(monthNav.getShortMonth(new Date())));
			});

			it("has 2 available slots", function(){
				expect(calData.get("appointmentTimeSlot").length).toEqual(2);
			});

			it("retrieves slots for a given day as 0 when none available", function(){
				var d = new Date();
				d.setDate(d.getDate() + 5);
				expect(calData.getSlotsForDay(d).length).toEqual(0);
			});

			it("retrieves slots for a given day with available slots", function(){
				var d = moment('07/17/2014', 'MM/DD/YYYY');
				d.hours(0).minutes(0).milliseconds(0);
				expect(calData.getSlotsForDay(d).length).toEqual(1);
			});

		});

	}
);