define(['Global', 'Layout', 'CalendarView', 'Vent', 'CalendarData', 'BookedAppointments', 'text!../../app/data/clinicSlots.json', 'text!../../app/data/bookedAppts.json'],
	function(Global, Layout, CalendarView, vent, CalendarData, BookedAppointments, clinicSlotsJSON, bookedApptsJSON){
		"use strict";

		describe("Calendar Month View", function(){
			var selectedDate = moment('08/01/2014', 'MM/DD/YYYY').format('ddd, MMM DD, YYYY'),
				model = new CalendarData(JSON.parse(clinicSlotsJSON), {parse: true}),
				monthView = new CalendarView({model: model}),
				layout = new Layout();

			beforeEach(function () {
				spyOn(vent, "trigger");

				monthView.model.set('selectedDate', selectedDate);

				monthView.minDate = moment('08/01/2014', 'MM/DD/YYYY').toDate();
				monthView.maxDate = moment('08/31/2014', 'MM/DD/YYYY').toDate();

				layout.render();
				layout.content.show(monthView);
			});

			it("has a function for converting the date format", function () {
				var date = moment("08/01/2014", 'MM/DD/YYYY');

				expect(monthView.makeDataDate(date)).toEqual("2014-08-01");
			});

			it("shows clinic availability on the calendar", function () {
				var $available = $('#calendar td.available');

				$available.removeClass('available');

				monthView.handleApptSlots();

				$available = $('#calendar td.available');

				expect($available.length).toBeTruthy();
			});

			it("allows the user to go to the day view when a day is clicked", function(){
				var e = {currentTarget: $('td[data-date="2014-08-15"] button')};

				monthView.showCalendarDayView(e);

				expect(vent.trigger).toHaveBeenCalledWith('show:ds-calendar-day:view');
			});

			it("does not show availability on the current date", function () {
				var makeDataDate = monthView.makeDataDate,
					$day;

				spyOn(monthView, "makeDataDate").andCallFake(function (date) {
					if (!date) {
						date = moment('08/01/2014', 'MM/DD/YYYY').toDate();

						return makeDataDate(date);
					}
				});

				$('#calendar td.available').removeClass('available');

				monthView.handleApptSlots();

				$day = $('td[data-date="2014-08-01"]');

				expect($day.hasClass('available')).toBeFalsy();
			});

		});
	}
);