define(['Global', 'Vent', 'FacilitiesProvidersCollection', 'FacilityProvidersModel'],
	function(Global, vent, FacilitiesProvidersCollection, FacilityProvidersModel){
		"use strict";

		var varResources = {
				get: function (linkName) {
					return {
						get: function () {
							return linkName
						}
					}
				}
			},
			facilitiesCollection = new FacilitiesProvidersCollection({varResources: varResources});

		describe("Facilities / Providers Collection", function() {
			beforeEach(function () {
				spyOn(vent, 'trigger');
			});

			it('changes the url based on passed in options', function(){
				expect(facilitiesCollection.url).toEqual('facility-provider-all');

				facilitiesCollection = new FacilitiesProvidersCollection();

				expect(facilitiesCollection.url).toEqual(undefined);

				facilitiesCollection = new FacilitiesProvidersCollection({
					varResources: varResources,
					PACTs: true
				});

				expect(facilitiesCollection.url).toEqual('facility-provider');
			});

			it('correctly parses the collection', function () {
				expect(facilitiesCollection.parse()).toEqual([]);

				var response = {
						facilityProviderList: "The list"
					};

				expect(facilitiesCollection.parse(response)).toEqual("The list");
			});

			it('correctly removes duplicates from the collection', function () {
				var list1 = [
						{
							facilityId: "123"
						},
						{
							facilityId: "456"
						},
						{
							facilityId: "789"
						}
					],
					list2 = [
						{
							facilityId: "789"
						}
					],
					list3 = [
						{
							facilityId: "123"
						},
						{
							facilityId: "456"
						}
					],
					list4 = [
						{
							facilityName: ""
						},
						{
							facilityName: "test"
						}
					],
					collection2 = new Backbone.Collection(list2),
					testCollection = new Backbone.Collection(list3, {model: FacilityProvidersModel});

				facilitiesCollection.set(list1);

				facilitiesCollection.removeDuplicates(collection2);

				expect(facilitiesCollection.toJSON()).toEqual(testCollection.toJSON());

				facilitiesCollection.removeDuplicates(collection2);

				expect(facilitiesCollection.toJSON()).toEqual(testCollection.toJSON());

				facilitiesCollection.removeDuplicates(new Backbone.Collection());

				expect(facilitiesCollection.toJSON()).toEqual(testCollection.toJSON());

				facilitiesCollection.removeDuplicates(new Backbone.Collection(list4));

				expect(facilitiesCollection.toJSON()).toEqual(testCollection.toJSON());
			});
		});
	}
);