define(['Global', 'Vent', 'text!../../test/spec_helpers/collapsibleTemplate.html', 'NoJqmCollapsible'],
	function(Global, vent, template){
		"use strict";

		describe("No-JQM Collapsible", function(){
			var view = Backbone.Marionette.ItemView.extend({
					template: _.template(template)
				}),
				$collapsible;

			view = new view();

			view.render();

			$collapsible = view.$el.find('.collapsible');

			$collapsible.noJqmCollapsible();

			it("correctly builds the collapsible", function () {
				expect($collapsible.hasClass('no-jqm-collapsible')).toBeTruthy();

				expect($collapsible.find('.header').hasClass('no-jqm-collapsible-header')).toBeTruthy();
				expect($collapsible.find('.header span.no-jqm-button-text').length).toBeTruthy();

				expect($collapsible.find('.content').hasClass('no-jqm-collapsible-content')).toBeTruthy();
			});

			it("correctly toggles the collapsible open and closed", function () {
				expect($collapsible.hasClass('no-jqm-collapsible-open')).toBeFalsy();

				$collapsible.noJqmCollapsible().toggle();

				expect($collapsible.hasClass('no-jqm-collapsible-open')).toBeTruthy();

				$collapsible.noJqmCollapsible().toggle();

				expect($collapsible.hasClass('no-jqm-collapsible-open')).toBeFalsy();

				$collapsible.find('.header').trigger('click');

				expect($collapsible.hasClass('no-jqm-collapsible-open')).toBeTruthy();

				$collapsible.find('.header').trigger('click');

				expect($collapsible.hasClass('no-jqm-collapsible-open')).toBeFalsy();
			});

			it("allows for the collapsible to be open by default", function () {
				view.render();

				$collapsible = view.$el.find('.collapsible');
				$collapsible.attr('data-open', 'true');

				$collapsible.noJqmCollapsible();

				expect($collapsible.hasClass('no-jqm-collapsible-open')).toBeTruthy();
			});

		});
	}
);