define(['Global', 'AppointmentRequestDetailsHeaderView', 'Vent'],
	function(Global, AppointmentRequestDetailsHeaderView, vent){
		"use strict";
		var appointmentDate = new Date(),
			AppointmentRequest = Backbone.Model.extend({
				appointmentDate: "",
				secondRequest: "",
				secondRequestAllowed: "",
				status: "",
				getAppointmentDate: function () {
					return this.get('appointmentDate');
				},
				getStatus: function () {
					return this.get('status');
				},
				isSecondRequest: function () {
					return this.get("secondRequest");
				},
				isBooked: function () {
					return this.getStatus() === "Booked";
				},
				isSubmitted: function () {
					return this.getStatus() === "Submitted";
				},
				isNotBooked: function () {
					return this.getStatus() === "Not Booked";
				},
				isSecondRequestAllowed: function () {
					return this.get('secondRequestAllowed');
				}
			}),
			appointmentRequest = new AppointmentRequest(),
			detailsHeaderView = new AppointmentRequestDetailsHeaderView({model: appointmentRequest}).render();

		appointmentDate.setDate(appointmentDate.getDate() + 1);

		appointmentRequest.set("appointmentDate", appointmentDate);

		describe("Request Details Header", function(){

			beforeEach(function () {
				spyOn(vent, "trigger");

				detailsHeaderView.delegateEvents();
			});

			it("has 2 buttons", function(){
				var $detailsHeaderButtons = detailsHeaderView.$el.find('button');

				expect($detailsHeaderButtons.length).toEqual(2);
				expect($detailsHeaderButtons.eq(0).text().trim()).toEqual("Back");
				expect($detailsHeaderButtons.eq(1).text().trim()).toEqual("Cancel");
			});

			it("navigates correctly", function () {
				detailsHeaderView.$el.find('#requests').trigger('click');
				expect(vent.trigger).toHaveBeenCalledWith("show:request:listing");

				detailsHeaderView.$el.find('#cancel-request-btn').trigger('click');
				expect(vent.trigger).toHaveBeenCalledWith("show:request:cancel", appointmentRequest);
			});

			it("correctly shows or hides the Cancel buttons", function () {
				appointmentRequest.set('status', 'Submitted');
				detailsHeaderView.toggleAvailableButtons();
				expect(detailsHeaderView.$el.find('#cancel-request-btn').hasClass('display-none-important')).toBeFalsy();

				appointmentRequest.set('status', 'Not Booked');
				detailsHeaderView.toggleAvailableButtons();
				expect(detailsHeaderView.$el.find('#cancel-request-btn').hasClass('display-none-important')).toBeTruthy();

				appointmentRequest.set('secondRequestAllowed', true);
				detailsHeaderView.toggleAvailableButtons();
				expect(detailsHeaderView.$el.find('#cancel-request-btn').hasClass('display-none-important')).toBeTruthy();

				appointmentRequest.set('status', 'Booked');
				detailsHeaderView.toggleAvailableButtons();
				expect(detailsHeaderView.$el.find('#cancel-request-btn').hasClass('display-none-important')).toBeFalsy();

				appointmentRequest.set('appointmentDate', new Date());
				appointmentRequest.set('secondRequest', true);
				detailsHeaderView.toggleAvailableButtons();
				expect(detailsHeaderView.$el.find('#cancel-request-btn').hasClass('display-none-important')).toBeFalsy();

				appointmentRequest.set('status', 'Cancelled');
				detailsHeaderView.toggleAvailableButtons();
				expect(detailsHeaderView.$el.find('#cancel-request-btn').hasClass('display-none-important')).toBeTruthy();
			});
		});
	}
);