define(['Global', 'SessionWarningView', 'SessionExpiredView'],
	function(Global, SessionWarningView, SessionExpiredView){
		'use strict';

		var UserSession = Backbone.Model.extend({});
		var timeLeft,
			sessionWarningView,
			sessionExpiredView,
			userSession = new UserSession({ sessionTimeoutIn: 3, timeUnits: "minutes"});

		SessionWarningView.prototype.onClose = function(){
			this.rendered = false;
		};

		describe( "Session Warning", function(){
			sessionWarningView = new SessionWarningView({model:userSession});
			sessionWarningView.render();

			beforeEach(function(){
				timeLeft = sessionWarningView.ui.timeleft;
			});

			it( "has been rendered", function(){
				expect(sessionWarningView.$el.children.length).toBeTruthy();
			});

			it( "updates the remaining session time", function(){
				expect(1*timeLeft.text().trim()).toEqual(3);
			});

			it("updates the remaining session time on userSession set sessionTimeoutIn", function(){
				userSession.set("sessionTimeoutIn", 2);
				expect(1*sessionWarningView.ui.timeleft.text().trim()).toEqual(2);
			});

			it("can be closed", function(){
				sessionWarningView.close();
				expect(sessionWarningView.rendered).toBeFalsy();
			});
		});

		describe( "Session Expired", function(){
			sessionExpiredView = new SessionExpiredView();
			sessionExpiredView.render();

			it("has been rendered", function(){
				expect(sessionExpiredView.$el.attr("id")).toEqual("session-expired")
			});
		});

	}
);